#****h* Temanejo/properties
# NAME
#   properties.py
#   - provide infrastructure for properties of tasks and edges
# DESCRIPTION
#   The function names and ids reported by the runtime are stored
#   in the class Properties. It is based on collections.OrderedDict
#   and holds properties such as function names.
#   This class behaves as a dict but remembers the order in which keys where 
#   inserted.
#
# USAGE
#   	funcnames = Property(1, "function")
#   	funcnames[0x100] = "foo"
#   	funcnames[0x200] = "bar"
#   	funcnames.label()	-> "function"
#   	funcnames.index("bar") 	-> 1
#   	funcnames.ids() 	->  [0x100, 0x200]
#   	funcname.labels() 	-> ["foo", "bar"]
#   	len(funcnames) 		-> 2
# PORTABILITY
#   The following modules are imported:
#   * collections
# AUTHOR
#   Jose Gracia, HLRS <gracia@hlrs.de>
# COPYRIGHT
#   (C) 2014-2015 HLRS, University of Stuttgart
#   Temanejo is published under the terms of the BSD license.
#******


import collections
#****c* Properties/OrderedDictWithIndex
# NAME
#   Base class for class Property
# DESCRIPTION
#   This class adds the method index to collections.OrderedDict for convenience
# DERIVED FROM
#   collections.OrderedDict
# METHODS
#   index
# SOURCE
class OrderedDictWithIndex(collections.OrderedDict):
    ''' A ordered dictinory enhanced with index() method. '''
    def index(self, key):
        return list(self.keys()).index(key)
#******

#****c* Properties/Property
# NAME
#   Class which holds the properties of tasks and dependencies
# DESCRIPTION
#   Class Property has attributes (id, label) for the actual property,
#   e.g id=1 and label="function" for a function name property.
#   In addition a property holds arbitrarily many property values as
#   (id, label) pairs. These are available through the dictionary access
#   mechanism. Ids and lables of values can also be retrieved through the
#   methods ids(), labels(), respectivily. These should help in building
#   widgets, etc.
# DERIVED FROM
#   OrderedDictWithIndex
# METHODS
#   __init__
#   id
#   label
#   ids
#   labels
# SOURCE
class Property(OrderedDictWithIndex):
    ''' Properties have an id and label. They can take arbitrary many values as (id, label) pairs.
    Inherits from OrderedDictWithIndex to use the order of registration for coloring, etc. '''
    def __init__(self, id, label):
        self.__id = id
        self.__label = label
        super(Property, self).__init__()
    def id(self):
        '''Returns the id of the property.'''
        return self.__id
    def label(self):
        '''Returns the label of the property.'''
        return self.__label
    def ids(self):
        '''Returns the id of all the property's values.'''
        return self.keys()
    def labels(self):
        '''Returns the label of all the property's values.'''
        return self.values()

# prepare some well known properties
propFunc = Property(1, "function")
#******

__all__ = [Property, propFunc, ]
