#!/usr/bin/env python

#****h* Temanejo/Temanejo
# NAME
#   Temanejo
#   - display of and interaction with a task dependency graph
# DESCRIPTION
#   Temanejo connects to task parallel applications which are instrumented
#   with calls to the Ayudame library.
#   It performs the following tasks:
#   * display the task graph receiving runtime information by Ayxudame events.
#   * control the execution sending user request to Ayudame, and thus the
#     application
# USAGE
#   Temanejo can either connect to a running application ("remote connection")
#   or start the application itself and make all the necessary adjustments
#   ("local connection").
#
#   Find help on the command line option by typing
#     temanejo.py -h
#
#   For more in depth information please refer to the documentation provided
#   with the source.
#
# PORTABILITY
#   required python version >= 2.7, <3.0
#
#   The following standard python modules are imported:
#   * logging
#   * math
#   * networkx
#   * optparse
#   * os
#   * shlex
#   * string
#   * struct
#   * subprocess
#   * sys
#   * tempfile
#   * time
#   * version
#   * gtk
#   * gtk.gdk
#   * gtk.keysyms
#   * socket
#   * threading
#   The following local modules are impoorted
#   * statusbars
#   * temanejo_graph
#   * temanejo_graphWidget
#   * temanejo_client_config as client_conf
# AUTHOR
#    Steffen Brinkmann et al, HLRS <temanejo@hlrs.de>
# COPYRIGHT
#    (C) 2010-2015, HLRS, University of Stuttgart
#    temanejo.py is published under the terms of the BSD license.
#    See the LICENSE file for details.
#    temanejo.py uses code of xdot.py, ver. 0.4 (C) 2008 Jose Fonseca
#    which is published under the terms of the GNU Lesser General Public
#    License
#    temanejo.py uses code of NetworkX, (C) 2004-2009 by Aric Hagberg,
#    Dan Schult and Pieter Swart. Published under the BSD license.

#******

'''Visualise and interact with task dependency graphs.'''

from __future__ import division

__progname__ = 'Temanejo'

__author__ = 'Steffen Brinkmann et al'

import version
__version__ = version.__version__

#import cProfile

import logging
import math
import networkx
import optparse
import os
import shlex
# import signal
import string
import struct
import subprocess
import sys
import tempfile
import time
# import types

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass
import gtk
# check if Xserver is running
assert(len( os.getenv( 'DISPLAY', '' )) > 0), "Temanejo cannot run in text mode ... Aborting! "
assert(gtk.gdk.display_get_default() != None), "Temanejo cannot run in text mode ... Aborting! "
# check for correct versions
assert (gtk.gtk_version >= (2, 16)) and (gtk.pygtk_version > (2, 16)), "Temanejo needs GTK v2.16 or newer .. Aborting!"
import gtk.gdk
import gtk.keysyms

import socket
from threading import Thread, Event

from temanejo_graph import Graph
from temanejo_graphWidget import GraphWidget
from temanejo_graphWidget import palette_rgb_g2r
from temanejo_graphWidget import paletteFractions_to_rgb
import statusbars   # for globally accesible statusbars
import temanejo_client_config as client_conf # for client dependent configuration
from message_dialog_async import message_dialog_async
from properties import propFunc

# how many threads to display when application is run on remote computer
THREADS_DISPLAY = 16

# base path of temanejo.py, i.e. to the module
temanejo_module_base_dir = os.path.dirname(os.path.abspath(__file__))
# base path to the python executable using temanejo.py module
temanejo_exec_base_dir = os.path.dirname(os.path.abspath(sys.argv[0]))

# equivalent to enum of events and requests
event_t = ['AYU_EVENT_NULL',
           'AYU_PREINIT',
           'AYU_INIT',
           'AYU_FINISH',
           'AYU_REGISTERFUNCTION',
           'AYU_ADDTASK',
           'AYU_ADDHIDDENTASK',
           'AYU_ADDDEPENDENCY',
           'AYU_ADDTASKTOQUEUE',
           'AYU_PRESELECTTASK',
           'AYU_PRERUNTASK',
           'AYU_RUNTASK',
           'AYU_POSTRUNTASK',
           'AYU_RUNTASKFAILED',
           'AYU_REMOVETASK',
           'AYU_WAITON',
           'AYU_BARRIER',
           'AYU_ADDWAITONTASK']

request_t = ['AYU_REQUEST_NULL',
             'AYU_NOREQUEST',
             'AYU_PAUSEONEVENT',
             'AYU_PAUSEONTASK',
             'AYU_PAUSEONFUNCTION',
             'AYU_STEP',
             'AYU_BREAKPOINT',
             'AYU_BLOCKTASK',
             'AYU_PRIORITISETASK',
             'AYU_SETNUMTHREADS']

#****c* Temanejo/TemanejoWindow
# NAME
#   TemanejoWindow
#   - python class for the main window of Temanejo
# DESCRIPTION
#   This represents the main window and user interface of Temanejo.
# DERIVED FROM
#   gtk.Window
# METHODS
#   __init__
#   cleanup
#   export_graph_image
#   export_graph_text
#   get_active_text
#   get_ayudame_libraries
#   highlight_checkbox
#   on_about
#   on_clearmarked
#   on_connect
#   on_controlcheckbutton
#   on_debug
#   on_debugfunction
#   on_export
#   on_gototask
#   on_key_press_event
#   on_margcollist
#   on_nodecollist
#   on_quit
#   on_refresh
#   on_run
#   on_run_nsteps
#   on_setnumthreads
#   on_settings
#   on_shapelist
#   push_statusbar
#   register_function
#   start_debugger
#   test_get_ayudame_library_no_lib_present
#   update_ui
# ATTRIBUTES
#   ui
# SOURCE


class TemanejoWindow(gtk.Window):

    ui = '''
    <ui>
        <toolbar name='ToolBar'>
            <toolitem action='Connect'/>
            <toolitem action='Preferences'/>
            <toolitem action='Toggleinfo'/>
            <separator/>
            <toolitem action='Refresh'/>
            <toolitem action='ZoomIn'/>
            <toolitem action='ZoomOut'/>
            <toolitem action='ZoomFit'/>
            <toolitem action='Zoom100'/>
            <separator/>
            <toolitem action='Run'/>
            <toolitem action='Run_fast' name='Run_fast'/>
            <toolitem action='Debug'/>
            <separator />
            <toolitem action='Export'/>
            <separator name='Sep'/>
            <toolitem action='About'/>
            <toolitem action='Quit'/>
        </toolbar>
    </ui>
    '''
#            <toolitem action='Help'/>
#            <toolitem action='Breakpoint'/>
#            <toolitem action='Step'/>

#<toolitem action='Open'/>

    def __init__(self):
        # ---
        gtk.Window.__init__(self)

        window = self
        window.set_default_size(1280, 768)
        window.set_icon_from_file(os.path.join
                                  (temanejo_module_base_dir,
                                   'resources/temanejo_icon.png'))

        self.is_fullscreen = False

        # whether pygraphviz_layout exists in networkx
        self.pygraphviz_layout_present = False

        # orientation of toolbar. Can be "horizontal" or "vertical"
        self.toolbar_orientation = "horizontal"

        # use this line to change the application wide font:
        #self.get_settings().set_string_property('gtk-font-name',
        #                                        'sans normal 10', '')

        # Graph and Graph widget
        self.graph = Graph()
        self.widget = GraphWidget(self.graph, self)
        #self.graph.set_widget(self.widget)

        # string holding executable and its arguments
        self.executable = None
        self.executable_args = ''
        self.exec_cmd = ''

        # hostname and port
        self.connect_host = ''
        self.connect_port = 0
        self.auto_connect = False

        # how often to try to connect to the server
        self.max_retries = 5
        self.retry_interval = 1

        # which external debugger to use
        self.ext_debugger = 'ddt'

        # whether to allow requests to be sent to host is determined by three
        # variables:
        # * self.allow_requests: is set to True when AYU_INIT is received and
        # to False when AYU_FINISH is received. It is also used to allow
        # requests during initialisation of the communicator. This Temaenjo's
        # way of allowing or disallowing requests.
        # * self.host_allows_requests: If a host doesn't allow requests
        # (e.g. because the underlying runtime isn't capable of requests) this
        # variable is used. This is set (TODO) in an initial handshake routine
        # between Temanejo and the runtime.
        # * self.viewer_mode: This is the users way of enabling or disabling
        # requests. It is set by the command line option --viewer
        #
        # All have to be True to send a request (except for special cases).
        self.allow_requests = False
        self.host_allows_requests = True
        self.viewer_mode = False

        # string holding ayudame library
        libraries = self.get_ayudame_libraries(temanejo_exec_base_dir,
                                               temanejo_module_base_dir)
        if not libraries:
            self.library = None
        else:
            self.library = libraries[0]

        # last event and taskId for status bar; blocked task Id
        self.last_event = 'AYU_EVENT_NULL'
        self.last_check_event = 'AYU_EVENT_NULL'
        self.last_taskId = client_conf.no_task_id
#        self.blockedTask = 0

        # how many threads to display in list
        client_conf.nthreads = THREADS_DISPLAY

        # current number of threads
        #self.numthreads = 2

        # how many steps to run when running several steps
        self.n_steps = 10

        # communticator thread and socket
        self.comm = None
        self.sock = None

        # thread for execution of the application
        self.execut = None

        # barrier counter
        self.barrier_count = 0

        # number of registered functions
        self.func_count = 0

        # counters, just placeholders here
        self.func_counter = dict()
        self.thread_counter = dict()
        self.status_counter = dict()
        self.task_states = ["NQ", "Q", "R", "F", "X"] # might be better define in class Graph

        # Create a UIManager instance
        uimanager = self.uimanager = gtk.UIManager()

        # Add the accelerator group to the toplevel window
        accelgroup = uimanager.get_accel_group()
        window.add_accel_group(accelgroup)

        # Create an ActionGroup
        self.actiongroup = gtk.ActionGroup('Actions')

        # Create actions
        self.actiongroup.add_actions([
            ('Toggleinfo', None, None, None,
             'Toggle information panel (t)', self.on_toggleinfo),
            ('Connect', gtk.STOCK_CONNECT, None, None,
             'Connect application (c)', self.on_connect),
            ('Preferences', gtk.STOCK_PREFERENCES, None, None,
             'Preferences (p)', self.on_settings),
            ('Refresh', gtk.STOCK_REFRESH, None, None,
             'Refresh the view (r)', self.on_refresh),
            ('ZoomIn', gtk.STOCK_ZOOM_IN, None, None,
             'Zoom in (+, pageUp)', self.widget.on_zoom_in),
            ('ZoomOut', gtk.STOCK_ZOOM_OUT, None, None,
             'Zoom out (-, pageDown)', self.widget.on_zoom_out),
            ('ZoomFit', gtk.STOCK_ZOOM_FIT, None, None,
             'Zoom to fit (Escape)', self.widget.on_zoom_fit),
            ('Zoom100', gtk.STOCK_ZOOM_100, None, None,
             'Default zoom level (z)', self.widget.on_zoom_100),
            ('About', gtk.STOCK_DIALOG_INFO, 'About', None,
             'About Temanejo (i)', self.on_about),
            ('Export', gtk.STOCK_CONVERT, 'Export', None,
             'Export Graph in different file formats (e)', self.on_export),
            #('Help', gtk.STOCK_DIALOG_QUESTION, 'Help', 'h',
            # 'Get help', self.on_help),
            ('Quit', gtk.STOCK_QUIT, None, None,
             'Quit Temanejo (Ctrl+q)', self.on_quit),
            ('Run', gtk.STOCK_MEDIA_PLAY, 'Run', None,
             'Run program to next pause event (space)', self.on_run),
            ('Run_fast', gtk.STOCK_MEDIA_FORWARD,
             'Run '+str(self.n_steps)+' steps', None,
             'Run program and ignore several pause events. Change the'
             'number of events in the preferences. (Ctrl+space)',
             self.on_run_nsteps),
            ('Debug', gtk.STOCK_GO_FORWARD, 'Debug', None,
             'Debug with gdb (d)', self.on_debug)
            ])

        self.actiongroup.get_action('Run_fast').set_sensitive(False)

        # Add the actiongroup to the uimanager
        uimanager.insert_action_group(self.actiongroup, 0)

        # Add a UI descrption
        uimanager.add_ui_from_string(self.ui)

        # Create a toolbar
        self.toolbar =  gtk.Toolbar()
        self.init_toolbar()

        # Create status bars
        self.statusbar = gtk.Statusbar()
        self.statusbar.set_has_resize_grip(False)
        self.statusbar_lastmsg_cid = self.statusbar.get_context_id('last_msg')
        self.statusbar_lastmsg_id = self.statusbar.push(
            self.statusbar_lastmsg_cid, 'No last message')

        self.statusbar2 = gtk.Statusbar()
        self.statusbar2.set_has_resize_grip(True)
        self.statusbar2_lastmsg_cid = self.statusbar2.get_context_id('last_msg')
        self.statusbar2_lastmsg_id = self.statusbar2.push(
            self.statusbar2_lastmsg_cid, '0 nodes, 0 edges')

        self.warning = statusbars.warning
        self.warning.set_has_resize_grip(False)
        self.warning_lastmsg_cid = self.warning.get_context_id('last_msg')
        self.warning_lastmsg_id = self.warning.push(
            self.warning_lastmsg_cid, '')
        self.warning_eventbox = gtk.EventBox()
        self.warning_eventbox.add(self.warning)

        hbox_statusbars = gtk.HBox()
        hbox_statusbars.pack_start(self.statusbar)
        hbox_statusbars.pack_end(self.statusbar2)
        hbox_statusbars.pack_end(self.warning_eventbox)

        # Create node colour lists (treeview)
        self.node_color_property_list = ['function', 'thread', 'status', 'task_duration', 'distance_to_marked', 'nothing']
        __labels = ['function', 'thread', 'task status', 'task duration', 'distance to marked', 'nothing']
        self.label_nodecollist = gtk.Label('Node colour means')
        self.combo_nodecollist = gtk.combo_box_new_text()
        for l in __labels:
            self.combo_nodecollist.append_text(l)
        self.combo_nodecollist.set_active(0)

        hbox_nodecollist = gtk.HBox()
        hbox_nodecollist.pack_start(self.label_nodecollist)
        hbox_nodecollist.pack_start(self.combo_nodecollist)

        self.liststore_nodecol = gtk.ListStore(
            str, str, str, str, str, str, str, str)

        self.treeview_nodecol = gtk.TreeView(self.liststore_nodecol)
        self.tvnc_col0 = gtk.TreeViewColumn('function')
        self.tvnc_col1 = gtk.TreeViewColumn('NQ')
        self.tvnc_col2 = gtk.TreeViewColumn('Q ')
        self.tvnc_col3 = gtk.TreeViewColumn('R ')
        self.tvnc_col4 = gtk.TreeViewColumn('F ')
#        self.tvnc_col5 = gtk.TreeViewColumn('X ')
        self.tvnc_col6 = gtk.TreeViewColumn('total')

        self.treeview_nodecol.append_column(self.tvnc_col0)
        self.treeview_nodecol.append_column(self.tvnc_col1)
        self.treeview_nodecol.append_column(self.tvnc_col2)
        self.treeview_nodecol.append_column(self.tvnc_col3)
        self.treeview_nodecol.append_column(self.tvnc_col4)
#        self.treeview_nodecol.append_column(self.tvnc_col5)
        self.treeview_nodecol.append_column(self.tvnc_col6)

        self.cell = gtk.CellRendererText()

#        self.cell.set_property(
#            'cell-background-gdk', gtk.gdk.Color(.5, .8, .7))
        self.tvnc_col0.pack_start(self.cell, True)
        self.tvnc_col0.set_attributes(self.cell, text=0, cell_background=1)
        self.cell = gtk.CellRendererText()
#        self.cell.set_property('cell-background', 'yellow')
        self.tvnc_col1.pack_start(self.cell, True)
        self.tvnc_col1.set_attributes(self.cell, text=2)
        self.tvnc_col2.pack_start(self.cell, True)
        self.tvnc_col2.set_attributes(self.cell, text=3)
        self.tvnc_col3.pack_start(self.cell, True)
        self.tvnc_col3.set_attributes(self.cell, text=4)
        self.tvnc_col4.pack_start(self.cell, True)
        self.tvnc_col4.set_attributes(self.cell, text=5)
#        self.tvnc_col5.pack_start(self.cell, True)
#        self.tvnc_col5.set_attributes(self.cell, text=6)
        self.tvnc_col6.pack_start(self.cell, True)
        self.tvnc_col6.set_attributes(self.cell, text=7)

#        self.liststore_nodecol.append(
#            ['total', gtk.gdk.Color(255, 255, 255), '0', '0', '0', '0', '0'])
        self.liststore_nodecol.append(
            ['total', 'white', '0', '0', '0', '0', '0', '0'])

        scroll_nodecollist = gtk.ScrolledWindow()
        scroll_nodecollist.add(self.treeview_nodecol)
        scroll_nodecollist.set_policy(
            gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        # Create margin colour lists (treeview)
        self.margin_color_property_list = ['status', 'function', 'thread', 'task_duration', 'nothing']
        __labels = ['task status', 'function', 'thread', 'task duration', 'nothing']
        self.label_margcollist = gtk.Label('Margin colour means')
        self.combo_margcollist = gtk.combo_box_new_text()
        for l in __labels:
            self.combo_margcollist.append_text(l)
        self.combo_margcollist.set_active(0)

        hbox_margcollist = gtk.HBox()
        hbox_margcollist.pack_start(self.label_margcollist)
        hbox_margcollist.pack_start(self.combo_margcollist)

        self.liststore_margcol = gtk.ListStore(
            str, str, str, str, str, str, str, str)

        self.treeview_margcol = gtk.TreeView(self.liststore_margcol)
        self.tvmc_col0 = gtk.TreeViewColumn('task status')
        self.tvmc_col1 = gtk.TreeViewColumn('NQ')
        self.tvmc_col2 = gtk.TreeViewColumn('Q ')
        self.tvmc_col3 = gtk.TreeViewColumn('R ')
        self.tvmc_col4 = gtk.TreeViewColumn('F ')
#        self.tvmc_col5 = gtk.TreeViewColumn('X ')
        self.tvmc_col6 = gtk.TreeViewColumn('#')

        self.treeview_margcol.append_column(self.tvmc_col0)
        self.treeview_margcol.append_column(self.tvmc_col6)

        self.cell = gtk.CellRendererText()

#        self.cell.set_property(
#           'cell-background-gdk', gtk.gdk.Color(.5, .8, .7))
        self.tvmc_col0.pack_start(self.cell, True)
        self.tvmc_col0.set_attributes(self.cell, text=0, cell_background=1)
        self.cell = gtk.CellRendererText()
#        self.cell.set_property('cell-background', 'yellow')
        self.tvmc_col1.pack_start(self.cell, True)
        self.tvmc_col1.set_attributes(self.cell, text=2)
        self.tvmc_col2.pack_start(self.cell, True)
        self.tvmc_col2.set_attributes(self.cell, text=3)
        self.tvmc_col3.pack_start(self.cell, True)
        self.tvmc_col3.set_attributes(self.cell, text=4)
        self.tvmc_col4.pack_start(self.cell, True)
        self.tvmc_col4.set_attributes(self.cell, text=5)
#        self.tvmc_col5.pack_start(self.cell, True)
#        self.tvmc_col5.set_attributes(self.cell, text=6)
        self.tvmc_col6.pack_start(self.cell, True)
        self.tvmc_col6.set_attributes(self.cell, text=7)

#        self.liststore_margcol.append(
#            ['total', gtk.gdk.Color(255, 255, 255), '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(
            ['not queued', 'red', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(
            ['queued', 'yellow', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(
            ['running', 'green', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(
            ['finished', 'white', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(
            ['failed', 'gray', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(
            ['total', 'white', '0', '0', '0', '0', '0', '0'])

        scroll_margcollist = gtk.ScrolledWindow()
        scroll_margcollist.add(self.treeview_margcol)
        scroll_margcollist.set_policy(
            gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        # Create node shape lists (treeview)

        # to be synchronised with self.graph.shape_list!!!
        # why not do: self.shape_list = self.graph.shape_list
        self.shape_list = ['square', 'triangle', 'diamond',
                           'house', 'trapezium', 'pentagram',
                           'pentagon', 'hexagon', 'invtriangle',
                           'invhouse', 'invtrapezium', 'invpentagon',
                           'circle', 'circle', 'circle', 'circle']

        self.pb_list = []
        for sl in self.shape_list:
            self.pb_list.append(
                gtk.gdk.pixbuf_new_from_file_at_size(
                    os.path.join(
                        temanejo_module_base_dir,
                        'resources/%s_100.png' % sl),
                    25, 25))
        self.pb_list.append(
            gtk.gdk.pixbuf_new_from_file_at_size(
                os.path.join(
                    temanejo_module_base_dir,
                    'resources/arrow_100.png'),
                25, 25))

        self.node_shape_property_list = ['thread', 'function', 'nothing']
        __labels = ['thread', 'function', 'nothing']
        self.label_shapelist = gtk.Label('Node shape means')
        self.combo_shapelist = gtk.combo_box_new_text()
        for l in __labels:
            self.combo_shapelist.append_text(l)
        self.combo_shapelist.set_active(0)

        hbox_shapelist = gtk.HBox()
        hbox_shapelist.pack_start(self.label_shapelist)
        hbox_shapelist.pack_start(self.combo_shapelist)

        self.liststore_shape = gtk.ListStore(
            gtk.gdk.Pixbuf, str, str, str, str, str, str, str)

        self.treeview_shape = gtk.TreeView(self.liststore_shape)
        self.tvns_col0 = gtk.TreeViewColumn('shape')
        self.tvns_col7 = gtk.TreeViewColumn('thread')
        self.tvns_col1 = gtk.TreeViewColumn('NQ')
        self.tvns_col2 = gtk.TreeViewColumn('Q ')
        self.tvns_col3 = gtk.TreeViewColumn('R ')
        self.tvns_col4 = gtk.TreeViewColumn('F ')
#        self.tvns_col5 = gtk.TreeViewColumn('X ')
        self.tvns_col6 = gtk.TreeViewColumn('total')

        self.treeview_shape.append_column(self.tvns_col0)
        self.treeview_shape.append_column(self.tvns_col7)
        self.treeview_shape.append_column(self.tvns_col1)
        self.treeview_shape.append_column(self.tvns_col2)
        self.treeview_shape.append_column(self.tvns_col3)
        self.treeview_shape.append_column(self.tvns_col4)
#        self.treeview_shape.append_column(self.tvns_col5)
        self.treeview_shape.append_column(self.tvns_col6)

        self.cell = gtk.CellRendererPixbuf()

        self.tvns_col0.pack_start(self.cell, True)
        self.tvns_col0.set_attributes(self.cell, pixbuf=0)
        self.cell = gtk.CellRendererText()
        self.tvns_col7.pack_start(self.cell, True)
        self.tvns_col7.set_attributes(self.cell, text=1)
        self.tvns_col1.pack_start(self.cell, True)
        self.tvns_col1.set_attributes(self.cell, text=2)
        self.tvns_col2.pack_start(self.cell, True)
        self.tvns_col2.set_attributes(self.cell, text=3)
        self.tvns_col3.pack_start(self.cell, True)
        self.tvns_col3.set_attributes(self.cell, text=4)
        self.tvns_col4.pack_start(self.cell, True)
        self.tvns_col4.set_attributes(self.cell, text=5)
#        self.tvns_col5.pack_start(self.cell, True)
#        self.tvns_col5.set_attributes(self.cell, text=6)
        self.tvns_col6.pack_start(self.cell, True)
        self.tvns_col6.set_attributes(self.cell, text=7)

        self.liststore_shape.append(
            [None, 'total', '0', '0', '0', '0', '0', '0'])

        scroll_shapelist = gtk.ScrolledWindow()
        scroll_shapelist.add(self.treeview_shape)
        scroll_shapelist.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        #### Create Control Panel

        # create checkboxes

        self.check_stopontask = gtk.CheckButton('Stop before running task')
#        self.check_debugfunction = gtk.CheckButton('Debug function')
        self.label_debugfunction = gtk.Label('Debug function')
        self.check_anyEvent = gtk.CheckButton('Stop on any event')
        self.check_preselecttask = gtk.ToggleButton('Stop scheduling tasks')
        self.check_preselecttask.set_sensitive(client_conf.break_preselecttask)
        self.check_preruntask = gtk.CheckButton('Stop before running any task')
        self.check_removetask = gtk.CheckButton('Stop after running any task')
        self.check_waiton = gtk.CheckButton('Stop on "wait on" directive')
        self.check_barrier = gtk.CheckButton('Stop on barrier')
        self.eventbox_anyEvent = gtk.EventBox()
        self.eventbox_preruntask = gtk.EventBox()
        self.eventbox_removetask = gtk.EventBox()
        self.eventbox_waiton = gtk.EventBox()
        self.eventbox_barrier = gtk.EventBox()
        self.eventbox_anyEvent.add(self.check_anyEvent)
        self.eventbox_preruntask.add(self.check_preruntask)
        self.eventbox_removetask.add(self.check_removetask)
        self.eventbox_waiton.add(self.check_waiton)
        self.eventbox_barrier.add(self.check_barrier)

        # create frame for 'any events'

        frame_breakpoints = gtk.Frame()
        frame_breakpoints.add(self.eventbox_anyEvent)

        # for convenience
        self.check_preruntask.set_active(True)

        # get default style
        self.check_default_style = self.check_anyEvent.get_modifier_style()

        # create label and combo (dropdown) box with button

        self.label_stopontask = gtk.Label('-none-')

        self.combo_debugfunction = gtk.combo_box_new_text()
        self.combo_debugfunction.append_text('-none-')
        self.combo_debugfunction.set_active(0)

        self.button_debugfunction = gtk.Button('>')

        # hboxes for stopontask and debugfunction

        hbox_stopontask = gtk.HBox()
        hbox_stopontask.pack_start(self.check_stopontask)
        hbox_stopontask.pack_start(self.label_stopontask)
        hbox_debugfunction = gtk.HBox()
        hbox_debugfunction.pack_start(self.button_debugfunction)
        hbox_debugfunction.pack_start(self.label_debugfunction)
        hbox_debugfunction.pack_start(self.combo_debugfunction)

        # create control panel

        self.control_panel = gtk.Table(8)
        self.control_panel.attach(self.check_preselecttask, 0, 1, 0, 1)
#        self.control_panel.attach(frame_breakpoints, 0, 1, 1, 2)
        self.control_panel.attach(self.eventbox_preruntask, 0, 1, 2, 3)
#        self.control_panel.attach(self.eventbox_removetask, 0, 1, 3, 4)
        self.control_panel.attach(self.eventbox_waiton, 0, 1, 4, 5)
#        self.control_panel.attach(self.eventbox_barrier, 0, 1, 5, 6)
        self.control_panel.attach(hbox_stopontask, 0, 1, 6, 7)
        self.control_panel.attach(hbox_debugfunction, 0, 1, 7, 8)
        self.control_panel.set_border_width(10)

        # create frame for breakpoints control panel

        frame_breakpoints = gtk.Frame('Breakpoints')
        frame_breakpoints.add(self.control_panel)

        # create navigation buttons etc.

        self.button_goto_task = gtk.Button('Goto task')
        self.entry_goto_task = gtk.Entry()
        self.button_clear_marked = gtk.Button('Clear marked nodes and edges')

        # create navigation panel

        self.navigation_panel = gtk.Table(1, 2)
        self.navigation_panel.attach(self.button_goto_task, 0, 1, 0, 1)
        self.navigation_panel.attach(self.entry_goto_task, 1, 2, 0, 1)
        self.navigation_panel.attach(self.button_clear_marked, 0, 2, 1, 2)
        self.navigation_panel.set_border_width(10)
        self.navigation_panel.set_row_spacings(5)
        self.navigation_panel.set_col_spacings(5)

        # create frame for navigation control panel

        frame_navigation = gtk.Frame('Navigation')
        frame_navigation.add(self.navigation_panel)

        # create control frame

        self.button_setnumthreads = gtk.Button('Set number of threads')
        self.button_setnumthreads.set_size_request(220, -1)
        self.button_setnumthreads.set_sensitive(client_conf.set_numthreads)

        adjust = gtk.Adjustment(client_conf.nthreads, 1, THREADS_DISPLAY, 1)
        self.spin_numthreads = gtk.SpinButton(adjust, 1, 0)
        self.spin_numthreads.set_sensitive(client_conf.set_numthreads)

        hbox1_control = gtk.HBox()
        hbox1_control.pack_start(self.button_setnumthreads, True)
        hbox1_control.pack_start(self.spin_numthreads)

        frame_control = gtk.Frame('Control')
        frame_control.add(hbox1_control)

        # connect widgets

        self.button_goto_task.connect(
            'clicked', self.on_gototask, 'gototask')
        self.entry_goto_task.connect(
            'key-release-event', self.on_gototask, 'gototask')
        self.button_clear_marked.connect(
            'clicked', self.on_clearmarked, 'clearmarked')
        self.check_anyEvent.connect(
            'clicked', self.on_controlcheckbutton, 'anyEvent')
        self.check_preselecttask.connect(
            'clicked', self.on_controlcheckbutton, 'preselecttask')
        self.check_preruntask.connect(
            'clicked', self.on_controlcheckbutton, 'preruntask')
        self.check_removetask.connect(
            'clicked', self.on_controlcheckbutton, 'removetask')
        self.check_waiton.connect(
            'clicked', self.on_controlcheckbutton, 'waiton')
        self.check_barrier.connect(
            'clicked', self.on_controlcheckbutton, 'barrier')
        self.check_stopontask.connect(
            'clicked', self.on_controlcheckbutton, 'stopontask')
        self.combo_nodecollist.connect(
            'changed', self.on_nodecollist, 'combo_nodecollist')
        self.combo_margcollist.connect(
            'changed', self.on_margcollist, 'combo_margcollist')
        self.combo_shapelist.connect(
            'changed', self.on_shapelist, 'combo_shapelist')
        self.button_debugfunction.connect(
            'clicked', self.on_debugfunction, 'debugfunction')

        self.button_setnumthreads.connect(
            'clicked', self.on_setnumthreads, 'setnumthreads')

        # Create log window

        self.prelog = ''

        self.logbuffer = gtk.TextBuffer()
        self.logbuffer.set_text('Start session, ' + time.ctime())

        self.logview = gtk.TextView(self.logbuffer)
        self.logview.set_editable(False)
        logscroll = gtk.ScrolledWindow()
        logscroll.add(self.logview)
        #self.logview.set_size_request(-1, 100)

        # Build global layout

        vbox1 = gtk.VBox()
        #vbox1.set_size_request(100, -1)
        vbox1.pack_start(hbox_nodecollist, False)
        vbox1.pack_start(scroll_nodecollist, True)
        vbox1.pack_start(hbox_margcollist, False)
        vbox1.pack_start(scroll_margcollist, True)
        vbox1.pack_start(hbox_shapelist, False)
        vbox1.pack_start(scroll_shapelist, True)

        vbox2 = gtk.VBox()
        vbox2.pack_start(frame_navigation, False, padding=5)
        vbox2.pack_start(frame_breakpoints, False, padding=5)
        vbox2.pack_start(frame_control, False, padding=5)

        # fill notebook
        self.notebook = gtk.Notebook()
        self.notebook.set_size_request(350, -1)
        self.notebook.append_page(vbox1, gtk.Label('display'))
        self.notebook.append_page(vbox2, gtk.Label('control'))
        self.notebook.append_page(logscroll, gtk.Label('log'))
        #self.notebook.append_page(vbox3, gtk.Label('info'))

        # graph widget and toolbar
        self.widget_box = gtk.Table(2,2)
        self.set_toolbar_orientation(self.toolbar_orientation)

        # central hbox
        self.central_hbox = gtk.HPaned()
        self.central_hbox.pack1(self.notebook, shrink=False)
        self.central_hbox.pack2(self.widget_box)

        # main vbox
        vbox = gtk.VBox()
        window.add(vbox)
        vbox.pack_start(self.central_hbox, True)
        vbox.pack_start(hbox_statusbars, False)

        self.set_focus(self.widget)

        self.show_all()

        # connect key press events

        self.connect('key-press-event', self.on_key_press_event)

    def init_toolbar(self):
        self.toolbar.append_item('', #Toggle info
                    'Toggle visibility of info panel (t)',
                    'Toggle visibility of info panel (t)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_info_24.png')),
                    self.on_toggleinfo,
                    user_data=None)
        self.toolbar.append_item(None,None,None,gtk.SeparatorToolItem(),None,None)
        self.toolbar.append_item('', #Connect
                    'Connect application (c)',
                    'Connect application (c)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/temanejo_icon_24.png')),
                    self.on_connect,
                    user_data=None)
        self.toolbar.append_item('',#Preferences
                    'Preferences (p)',
                    'Preferences (p)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_preferences_24.png')),
                    self.on_settings,
                    user_data=None)
        self.toolbar.append_item(None,None,None,gtk.SeparatorToolItem(),None,None)
        self.toolbar.append_item('', #Refresh
                    'Refresh the view (r)',
                    'Refresh the view (r)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_refresh_24.png')),
                    self.on_refresh,
                    user_data=None)
        self.toolbar.append_item('',#Zoom in
                    'Zoom in (+, pageUp)',
                    'Zoom in (+, pageUp)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_zoom_in_24.png')),
                    self.widget.on_zoom_in,
                    user_data=None)
        self.toolbar.append_item('',#Zoom out
                    'Zoom out (-, pageDown)',
                    'Zoom out (-, pageDown)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_zoom_out_24.png')),
                    self.widget.on_zoom_out,
                    user_data=None)
        self.toolbar.append_item('',#Best fit
                    'Zoom to fit (Escape)',
                    'Zoom to fit (Escape)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_zoom_fit_24.png')),
                    self.widget.on_zoom_fit,
                    user_data=None)
        self.toolbar.append_item('',#Default zoom
                    'Default zoom level (z)',
                    'Default zoom level (z)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_zoom_default_24.png')),
                    self.widget.on_zoom_100,
                    user_data=None)
        sep = gtk.SeparatorToolItem()
        sep.set_sensitive(False)
        self.toolbar.append_item(None,None,None,sep,None,None)
        self.toolbar.append_item('',#Run
                    'Run program to next pause event (space)',
                    'Run program to next pause event (space)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_run_24.png')),
                    self.on_run,
                    user_data=None)
        self.toolbar.append_item('',#Run n steps
                    'Run program and ignore several pause events. Change the'
                    'number of events in the preferences. (Ctrl+space)',
                    'Run program and ignore several pause events. Change the'
                    'number of events in the preferences. (Ctrl+space)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_runfast_24.png')),
                    self.on_run_nsteps,
                    user_data=None)
        self.toolbar.append_item('',#Debug
                    'Debug (d)',
                    'Debug (d)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_debug_24.png')),
                    self.on_debug,
                    user_data=None)
        sep = gtk.SeparatorToolItem()
        sep.set_expand(True)
        sep.set_draw(False)
        self.toolbar.append_item(None,None,None,sep,None,None)
        self.toolbar.append_item('',#Export
                    'Export Graph in different file formats (e)',
                    'Export Graph in different file formats (e)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_export_24.png')),
                    self.on_export,
                    user_data=None)
        self.toolbar.append_item('',#About
                    'About Temanejo (i)',
                    'About Temanejo (i)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_about_24.png')),
                    self.on_about,
                    user_data=None)
        self.toolbar.append_item('',#Quit
                    'Quit Temanejo (Ctrl+q)',
                    'Quit Temanejo (Ctrl+q)',
                    gtk.image_new_from_file(os.path.join(
                        temanejo_module_base_dir,
                        'resources/tem_exit_24.png')),
                    self.on_quit,
                    user_data=None)
        if self.toolbar_orientation == "horizontal":
            self.toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        else:
            self.toolbar.set_orientation(gtk.ORIENTATION_VERTICAL)

    def set_toolbar_orientation(self, orientation):
        if orientation != self.toolbar_orientation:
            self.toolbar_orientation = orientation

        if self.toolbar in self.widget_box.get_children():
            self.widget_box.remove(self.toolbar)
            self.widget_box.remove(self.widget)

        if self.toolbar_orientation == 'horizontal':
            self.toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
            self.widget_box.attach(self.toolbar, 0, 2, 0, 1, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.SHRINK)
            self.widget_box.attach(self.widget, 0, 2, 1, 2, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        else:
            self.toolbar.set_orientation(gtk.ORIENTATION_VERTICAL)
            self.widget_box.attach(self.toolbar, 0, 1, 0, 2, xoptions=gtk.SHRINK, yoptions=gtk.EXPAND|gtk.FILL)
            self.widget_box.attach(self.widget, 1, 2, 0, 2, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)


    def set_viewer_mode(self, viewer_mode):
        self.viewer_mode = viewer_mode


    def on_key_press_event(self, widget, event):

        #### DEBUG
        #~ keyname = gtk.gdk.keyval_name(event.keyval)
        #~ print "Key %s (%d) was pressed" % (keyname, event.keyval)
        #~ if event.state & gtk.gdk.CONTROL_MASK:
            #~ print "Control was being held down"
        #~ if event.state & gtk.gdk.MOD1_MASK:
            #~ print "Alt was being held down"
        #~ if event.state & gtk.gdk.SHIFT_MASK:
            #~ print "Shift was being held down"

        ######
        if (event.keyval == gtk.keysyms.space and not
                (event.state & gtk.gdk.CONTROL_MASK)):
            self.on_run(widget)
            return True
        if (event.keyval == gtk.keysyms.space and
                (event.state & gtk.gdk.CONTROL_MASK)):
            self.on_run_nsteps(widget)
            return True
        if event.keyval == gtk.keysyms.t:
            self.on_toggleinfo(widget)
            return True
        if event.keyval == gtk.keysyms.r:
            self.on_refresh(widget)
            return True
        if event.keyval == gtk.keysyms.e:
            self.on_export(widget)
            return True
        if event.keyval == gtk.keysyms.d:
            self.on_debug(widget)
            return True
        if event.keyval == gtk.keysyms.i:
            self.on_about(widget)
            return True
        if event.keyval == gtk.keysyms.f:
            if self.is_fullscreen is False:
                self.is_fullscreen = True
                self.fullscreen()
            else:
                self.is_fullscreen = False
                self.unfullscreen()
            return True
        if event.keyval == gtk.keysyms.g:
            self.notebook.set_current_page(1)
            self.entry_goto_task.grab_focus()
            return True
        if event.keyval == gtk.keysyms.c:
            self.on_connect(widget)
            return True
        if event.keyval == gtk.keysyms.z:
            self.widget.on_zoom_100(widget)
            return True
        if event.keyval == gtk.keysyms.Escape:
            self.widget.on_zoom_fit(widget)
            return True
        if not self.notebook.is_focus():
            if event.keyval == gtk.keysyms.Left:
                self.widget.x -= (self.widget.POS_INCREMENT /
                                  self.widget.zoom_ratio)
                self.widget.queue_draw()
                return True
            if event.keyval == gtk.keysyms.Right:
                self.widget.x += (self.widget.POS_INCREMENT /
                                  self.widget.zoom_ratio)
                self.widget.queue_draw()
                return True
            if event.keyval == gtk.keysyms.Up:
                self.widget.y -= (self.widget.POS_INCREMENT /
                                  self.widget.zoom_ratio)
                self.widget.queue_draw()
                return True
            if event.keyval == gtk.keysyms.Down:
                self.widget.y += (self.widget.POS_INCREMENT /
                                  self.widget.zoom_ratio)
                self.widget.queue_draw()
                return True
        if (event.keyval == gtk.keysyms.Page_Down or
                event.keyval == gtk.keysyms.minus):
            self.widget.on_zoom_out(widget)
            self.widget.queue_draw()
            return True
        if (event.keyval == gtk.keysyms.Page_Up or
                event.keyval == gtk.keysyms.plus):
            self.widget.on_zoom_in(widget)
            self.widget.queue_draw()
            return True
        if event.keyval == gtk.keysyms.p:
            self.on_settings(widget)
            return True
        if (event.keyval == gtk.keysyms.q and
                event.state & gtk.gdk.CONTROL_MASK):
            self.on_quit(widget)
            return True

        return False

    def on_controlcheckbutton(self, checkbutton, data=None):
        if data == 'anyEvent':
            if checkbutton.get_active():  # is checked
                self.check_preruntask.set_sensitive(False)
                self.check_removetask.set_sensitive(False)
                self.check_waiton.set_sensitive(False)
                self.check_barrier.set_sensitive(False)
                if self.comm is not None:
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_EVENT_NULL'), 1])
            else:  # is not checked
                self.check_preruntask.set_sensitive(True)
                self.check_removetask.set_sensitive(True)
                self.check_waiton.set_sensitive(True)
                self.check_barrier.set_sensitive(True)
                if self.comm is not None:
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_EVENT_NULL'), 0])
        if self.comm is not None:
            if data == 'preruntask':
                if checkbutton.get_active():
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_PRERUNTASK'), 1])
                else:
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_PRERUNTASK'), 0])
            elif data == 'removetask':
                if checkbutton.get_active():
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_REMOVETASK'), 1])
                else:
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_REMOVETASK'), 0])
            elif data == 'waiton':
                if checkbutton.get_active():
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_WAITON'), 1])
                else:
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_WAITON'), 0])
            elif data == 'barrier':
                if checkbutton.get_active():
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_BARRIER'), 1])
                else:
                    self.comm.send_request(
                        0, 'AYU_PAUSEONEVENT',
                        [event_t.index('AYU_BARRIER'), 0])
            elif data == 'stopontask':
                if self.check_stopontask.get_active():
                    if self.label_stopontask.get_text() != '-none-':
                        taskId = int(self.label_stopontask.get_text())
                        self.comm.send_request(0, 'AYU_PAUSEONTASK',
                                               [taskId, 1])
                    else:
                        self.comm.send_request(0, 'AYU_PAUSEONTASK',
                                               [0, 0])

    def on_nodecollist(self, widget, event):

        self.liststore_nodecol.clear()
#        iterat = self.liststore_nodecol.get_iter_first()
#        while self.liststore_nodecol.remove(iterat):
#            pass

        cols = self.treeview_nodecol.get_columns()
        t = self.get_active_text(self.combo_nodecollist)
        if t == 'function':
            self.widget.graphview.node_colours = t

            self.tvnc_col0.set_title('function')
            self.tvnc_col6.set_title('total')

            if self.tvnc_col1 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col1, 1)
            if self.tvnc_col2 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col2, 2)
            if self.tvnc_col3 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col3, 3)
            if self.tvnc_col4 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col4, 4)
#            if self.tvnc_col5 not in cols:
#                self.treeview_nodecol.insert_column(self.tvnc_col5, 5)

            # fill function list
            for (fId, fName) in propFunc.items():
                #rgb = paletteFractions_to_rgb(palette_rgb(fId, 1.0))
                rgb = paletteFractions_to_rgb(self.widget.graphview.palette_nodes[self.widget.graphview.hash_id(propFunc.index(fId))])
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_nodecol.append(
                    [fName, color_str, '0', '0', '0', '0', '0', '0'])

            self.liststore_nodecol.append(
                ['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'thread':
            self.widget.graphview.node_colours = t

            self.tvnc_col0.set_title('thread')
            self.tvnc_col6.set_title('total')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col2, 1)
            if self.tvnc_col3 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col3, 2)
            if self.tvnc_col4 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col4, 3)
#            if self.tvnc_col5 not in cols:
#                self.treeview_nodecol.insert_column(self.tvnc_col5, 4)

            # fill thread list
            for thId in range(client_conf.nthreads+1):
                rgb = paletteFractions_to_rgb(self.widget.graphview.palette_nodes[self.widget.graphview.hash_id(thId)])
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_nodecol.append(
                    ['thread %d' % thId, color_str,
                     '0', '0', '0', '0', '0', '0'])
            self.liststore_nodecol.append(
                ['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task status':
            self.widget.graphview.node_colours = 'status'

            self.tvnc_col0.set_title('task status')
            self.tvnc_col6.set_title('#')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)

            # fill status list
            self.liststore_nodecol.append(
                ['not queued', '#ff0000', '0', '0', '0', '0', '0', '0'])
            self.liststore_nodecol.append(
                ['queued', '#ffff00', '0', '0', '0', '0', '0', '0'])
            self.liststore_nodecol.append(
                ['running', '#00ff00', '0', '0', '0', '0', '0', '0'])
            self.liststore_nodecol.append(
                ['finished', '#ffffff', '0', '0', '0', '0', '0', '0'])
            self.liststore_nodecol.append(
                ['failed', '#888888', '0', '0', '0', '0', '0', '0'])
            self.liststore_nodecol.append(
                ['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task duration':
            self.widget.graphview.node_colours = 'task_duration'
            self.tvnc_col0.set_title('task duration')
            self.tvnc_col6.set_title('nanosecs')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)

            self.widget.graphview.get_dtminmax()
            # fill status list
            if self.widget.graphview.dt_min <= self.widget.graphview.dt_max:
                fs = palette_rgb_g2r(
                    0.001, math.log(
                        self.widget.graphview.dt_max -
                        self.widget.graphview.dt_min+1) + .001)
                rgb = paletteFractions_to_rgb(fs)
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_nodecol.append(
                    ['min', color_str, '0', '0', '0', '0', '0',
                     str(self.widget.graphview.dt_min)])
                for i in range(3, 12):
                    i10 = 10**i
                    if (i10 > self.widget.graphview.dt_min and
                            i10 < self.widget.graphview.dt_max):
                        fs = palette_rgb_g2r(
                            math.log(i10-self.widget.graphview.dt_min+1) +
                            .001,
                            math.log(self.widget.graphview.dt_max -
                                     self.widget.graphview.dt_min+1) +
                            .001)
                        rgb = paletteFractions_to_rgb(fs)
                        color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                        self.liststore_nodecol.append(
                            ['', color_str, '0', '0', '0', '0', '0', str(i10)])

                rgb = paletteFractions_to_rgb(palette_rgb_g2r(1, 1))
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_nodecol.append(
                    ['max', color_str, '0', '0', '0', '0', '0',
                     str(self.widget.graphview.dt_max)])

        elif t == 'distance to marked':
            self.widget.graphview.node_colours = 'distance_to_marked'
            self.tvnc_col0.set_title('distance to marked')
            self.tvnc_col6.set_title('#')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)

            self.widget.graphview.calc_all_distances(
                self.widget.graphview.marked_node)

            # fill colour list
            if (self.widget.graphview.marked_node is not None
                    and self.widget.graphview.dist_max != 0):
                rgb = paletteFractions_to_rgb(
                    palette_rgb_g2r(1, self.widget.graphview.dist_max))
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_nodecol.append(
                    ['min', color_str, '0', '0', '0', '0', '0', '1'])
                for i in range(5, self.widget.graphview.dist_max, 5):
                    if i <= self.widget.graphview.dist_max:
                        rgb = paletteFractions_to_rgb(palette_rgb_g2r(
                            i, self.widget.graphview.dist_max))
                        color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                        self.liststore_nodecol.append(
                            ['', color_str, '0', '0', '0', '0', '0', str(i)])

                rgb = paletteFractions_to_rgb(palette_rgb_g2r(1, 1))
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_nodecol.append(
                    ['max', color_str, '0', '0', '0', '0', '0',
                     str(self.widget.graphview.dist_max)])

        elif t == 'nothing':
            self.widget.graphview.node_colours = 'nothing'
            self.liststore_nodecol.clear()

            self.tvnc_col0.set_title('no')
            self.tvnc_col6.set_title('colours')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)
            else:
                self.liststore_nodecol.append(
                    ['no data', 'white', '0', '0', '0', '0', '0', '0'])

        self.update_ui()

    def on_margcollist(self, widget, event):

        self.liststore_margcol.clear()
#        iterat = self.liststore_margcol.get_iter_first()
#        while self.liststore_margcol.remove(iterat):
#            pass

        cols = self.treeview_margcol.get_columns()
        t = self.get_active_text(self.combo_margcollist)
        if t == 'function':
            self.widget.graphview.margins_mean = t

            self.tvmc_col0.set_title('function')
            self.tvmc_col6.set_title('total')

            if self.tvmc_col1 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col1, 1)
            if self.tvmc_col2 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col2, 2)
            if self.tvmc_col3 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col3, 3)
            if self.tvmc_col4 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col4, 4)
#            if self.tvmc_col5 not in cols:
#                self.treeview_margcol.insert_column(self.tvmc_col5, 5)

            # fill function list
            for (fId, fName) in propFunc.items():
                rgb = paletteFractions_to_rgb(self.widget.graphview.palette_nodes[self.widget.graphview.hash_id(propFunc.index(fId))])
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_margcol.append(
                    [fName, color_str, '0', '0', '0', '0', '0', '0'])

            self.liststore_margcol.append(
                ['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'thread':
            self.widget.graphview.margins_mean = t

            self.tvmc_col0.set_title('thread')
            self.tvmc_col6.set_title('total')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col2, 1)
            if self.tvmc_col3 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col3, 2)
            if self.tvmc_col4 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col4, 3)
#            if self.tvmc_col5 not in cols:
#                self.treeview_margcol.insert_column(self.tvmc_col5, 4)

            # fill thread list
            for thId in range(client_conf.nthreads+1):
                rgb = paletteFractions_to_rgb(self.widget.graphview.palette_nodes[self.widget.graphview.hash_id(thId)])
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_margcol.append(
                    ['thread %d' % thId, color_str,
                     '0', '0', '0', '0', '0', '0'])
            self.liststore_margcol.append(
                ['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task status':
            self.widget.graphview.margins_mean = 'status'

            self.tvmc_col0.set_title('task status')
            self.tvmc_col6.set_title('#')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col2)
            if self.tvmc_col3 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col3)
            if self.tvmc_col4 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col4)
#            if self.tvmc_col5 in cols:
#                self.treeview_margcol.remove_column(self.tvmc_col5)

            # fill status list
            self.liststore_margcol.append(
                ['not queued', 'red', '0', '0', '0', '0', '0', '0'])
            self.liststore_margcol.append(
                ['queued', 'yellow', '0', '0', '0', '0', '0', '0'])
            self.liststore_margcol.append(
                ['running', 'green', '0', '0', '0', '0', '0', '0'])
            self.liststore_margcol.append(
                ['finished', 'white', '0', '0', '0', '0', '0', '0'])
            self.liststore_margcol.append(
                ['failed', 'gray', '0', '0', '0', '0', '0', '0'])
            self.liststore_margcol.append(
                ['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task duration':
            self.widget.graphview.margins_mean = 'task_duration'
            self.tvmc_col0.set_title('task duration')
            self.tvmc_col6.set_title('nanosecs')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col2)
            if self.tvmc_col3 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col3)
            if self.tvmc_col4 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col4)
#            if self.tvmc_col5 in cols:
#                self.treeview_margcol.remove_column(self.tvmc_col5)

            self.widget.graphview.get_dtminmax()
            # fill status list
            if self.widget.graphview.dt_min <= self.widget.graphview.dt_max:
                fs = palette_rgb_g2r(
                    0.001, math.log(
                        self.widget.graphview.dt_max -
                        self.widget.graphview.dt_min+1) + .001)
                rgb = paletteFractions_to_rgb(fs)
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_margcol.append(
                    ['min', color_str, '0', '0', '0', '0', '0',
                     str(self.widget.graphview.dt_min)])
                for i in range(3, 12):
                    i10 = 10**i
                    if (i10 > self.widget.graphview.dt_min and
                            i10 < self.widget.graphview.dt_max):
                        fs = palette_rgb_g2r(
                            math.log(i10-self.widget.graphview.dt_min+1)+.001,
                            math.log(self.widget.graphview.dt_max -
                                     self.widget.graphview.dt_min+1)+.001)
                        rgb = paletteFractions_to_rgb(fs)
                        color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                        self.liststore_margcol.append(
                            ['', color_str, '0', '0', '0', '0', '0', str(i10)])

                rgb = paletteFractions_to_rgb(palette_rgb_g2r(1, 1))
                color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                self.liststore_margcol.append(
                    ['max', color_str, '0', '0', '0', '0', '0',
                     str(self.widget.graphview.dt_max)])
            else:
                self.liststore_margcol.append(
                    ['no data', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'nothing':
            self.widget.graphview.margins_mean = 'nothing'

            self.tvmc_col0.set_title('no')
            self.tvmc_col6.set_title('margins')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col2)
            if self.tvmc_col3 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col3)
            if self.tvmc_col4 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col4)
#            if self.tvmc_col5 in cols:
#                self.treeview_margcol.remove_column(self.tvmc_col5)

        self.update_ui()

    def on_shapelist(self, widget, event):

        self.liststore_shape.clear()
#        iterat=self.liststore_shape.get_iter_first()
#        while self.liststore_shape.remove(iterat):
#            pass

        cols = self.treeview_shape.get_columns()
        t = self.get_active_text(self.combo_shapelist)
        if t == 'function':
            self.widget.graphview.shapes_mean = 'function'

            self.tvns_col0.set_title('shape')
            self.tvns_col7.set_title('function')
            self.tvns_col6.set_title('total')

            if self.tvns_col1 not in cols:
                self.treeview_shape.insert_column(self.tvns_col1, 2)
            if self.tvns_col2 not in cols:
                self.treeview_shape.insert_column(self.tvns_col2, 3)
            if self.tvns_col3 not in cols:
                self.treeview_shape.insert_column(self.tvns_col3, 4)
            if self.tvns_col4 not in cols:
                self.treeview_shape.insert_column(self.tvns_col4, 5)
#            if self.tvns_col5 not in cols:
#                self.treeview_shape.insert_column(self.tvns_col5, 6)

            # fill function list
            for (fId, fName) in propFunc.items():
                if fId < len(self.pb_list)-1:
                    self.liststore_shape.append([self.pb_list[fId],
                                                 fName,
                                                 '0', '0', '0', '0', '0', '0'])
                else:
                    self.liststore_shape.append(
                        [self.pb_list[len(self.pb_list)-1],
                         fName, '0', '0', '0', '0', '0', '0'])

            self.liststore_shape.append(
                [None, 'total', '0', '0', '0', '0', '0', '0'])

        elif t == 'thread':
            self.widget.graphview.shapes_mean = 'thread'

            self.tvns_col0.set_title('shape')
            self.tvns_col7.set_title('thread')
            self.tvns_col6.set_title('total')

            if self.tvns_col1 in cols:
                self.treeview_shape.remove_column(self.tvns_col1)
            if self.tvns_col2 not in cols:
                self.treeview_shape.insert_column(self.tvns_col2, 2)
            if self.tvns_col3 not in cols:
                self.treeview_shape.insert_column(self.tvns_col3, 3)
            if self.tvns_col4 not in cols:
                self.treeview_shape.insert_column(self.tvns_col4, 4)
#            if self.tvns_col5 not in cols:
#                self.treeview_shape.insert_column(self.tvns_col5, 5)

            # fill thread list
            for thId in range(client_conf.nthreads):
                self.liststore_shape.append(
                    [self.pb_list[thId], 'thread %d' % thId,
                     '0', '0', '0', '0', '0', '0'])

            self.liststore_shape.append(
                [self.pb_list[len(self.shape_list)], 'comm thread',
                 '0', '0', '0', '0', '0', '0'])

            self.liststore_shape.append(
                [None, 'total', '0', '0', '0', '0', '0', '0'])

        elif t == 'nothing':
            self.widget.graphview.shapes_mean = 'nothing'

            self.tvns_col0.set_title('no')
            self.tvns_col6.set_title('shapes')
            self.tvns_col7.set_title('')

            if self.tvns_col1 in cols:
                self.treeview_shape.remove_column(self.tvns_col1)
            if self.tvns_col2 in cols:
                self.treeview_shape.remove_column(self.tvns_col2)
            if self.tvns_col3 in cols:
                self.treeview_shape.remove_column(self.tvns_col3)
            if self.tvns_col4 in cols:
                self.treeview_shape.remove_column(self.tvns_col4)
#            if self.tvns_col5 in cols:
#                self.treeview_shape.remove_column(self.tvns_col5)

        if (widget != "update_ui"):
            self.update_ui()

    def on_toggleinfo(self, widget):
        if self.notebook.get_visible():
            self.notebook.set_visible(False)
        else:
            self.notebook.set_visible(True)

    def on_debugfunction(self, widget, event):
        funcId = self.combo_debugfunction.get_active()-1
        if funcId >= 0:
            debugfunction_name = self.get_active_text(self.combo_debugfunction)
            if self.execut is not None:
                self.start_debugger(self.executable,
                               self.execut.pid,
                               break_at_function=debugfunction_name)
            else:
                logging.error('debugging with gdb is not possible for remotely'
                              '\nconnected applications.')

    def on_clearmarked(self, widget, event):
        for n in self.graph:
            self.graph._highlight[n] = 0
        self.widget.graphview.marked_node = None
        self.widget.queue_draw()

    def on_setnumthreads(self, widget, event, key=None):
        if self.comm is not None:
            self.comm.send_request(0, 'AYU_SETNUMTHREADS',
                                   [self.spin_numthreads.get_value_as_int()])

    def on_gototask(self, widget, event, key=None):
        taskId_text = self.entry_goto_task.get_text()
        if taskId_text.isdigit():
            taskId = int(self.entry_goto_task.get_text())
        else:
            return

        nodeId = self.graph.get_nodeId(taskId)

        if nodeId is None:
            return

        self.widget.x = self.widget.graphview.layout[nodeId][0]
        self.widget.y = -self.widget.graphview.layout[nodeId][1]
        self.widget.animate_to(self.widget.x, self.widget.y)

    def on_debug(self, action):
        if self.execut is not None:
            self.start_debugger(self.executable, self.execut.pid)
        else:
            if client_conf.remote_pid is not None:
                self.start_debugger("", client_conf.remote_pid)
            else:
                self.print_error("PID of remote application not known. Cannot run debugger remotely.")

    def start_debugger(self, executable, pid, break_at_function=None):
        # locally define somthing similar to Unix' which command
        def which(program):
            import os
            def is_exe(fpath):
                return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

            fpath, fname = os.path.split(program)
            if fpath:
                if is_exe(program):
                    return program
            else:
                for path in os.environ["PATH"].split(os.pathsep):
                    path = path.strip('"')
                    exe_file = os.path.join(path, program)
                    if is_exe(exe_file):
                        return exe_file

            return None

        # check for debugger
        if which(self.ext_debugger) is None:
            logging.error(self.ext_debugger + ' is not installed.')
            return

        gdb_cmd = None

        if break_at_function is not None:
            if self.ext_debugger == 'gdb':
                gdb_cmd = ['break ' + debugfunction_name, 'c']
                #logging.info("gdb_cmd is %s" % gdb_cmd)
                tmpfile = tempfile.mkstemp()
                # write commands for gdb to temporary file
                # tmpfile[0] is the handle (a number)
                # tmpfile[1] is the name with full pathtmpfile = tempfile.mkstemp()
                os.write(tmpfile[0], '\n'.join(gdb_cmd)+'\n')
                os.close(tmpfile[0])
            elif self.ext_debugger == 'ddt':
                # TODO
                logging.warning("setting breakpoints at specific functions is not yet implemented.\n"
                                "Please chose gdb as the external debugger.")
                pass

        # create debugger command line
        # the minor differences seem to be necessary!
        if self.ext_debugger == 'gdb':
            debugger_command_line = 'gdb --tui %s %d' % (executable, pid)
        elif self.ext_debugger == 'ddt':
            if not executable:
                debugger_command_line = 'ddt --attach=%s:%d' % (self.connect_host, pid)
            else:
                debugger_command_line = 'ddt --attach=%s:%d %s ' % (self.connect_host, pid, executable)

        known_terminals = {
            "xterm" : ['xterm', '-geometry', '80x40',
                        '-title', '\"', self.ext_debugger, ' started by Temanejo\"',
                        '-e', debugger_command_line],
            'xfce4-terminal' : ['xfce4-terminal', '--geometry', '80x40',
                        '--title', '\"', self.ext_debugger, ' started by Temanejo\"',
                        '-e', debugger_command_line],
            'rxvt' : ['rxvt', '-geometry', '80x40',
                        '-title', '\"', self.ext_debugger, ' started by Temanejo\"',
                        '-e', debugger_command_line],
            'urxvt' : ['urxvt', '-geometry', '80x40',
                        '-title', '\"', self.ext_debugger, ' started by Temanejo\"',
                        '-e', debugger_command_line],
            'rxvt-unicode' : ['rxvt-unicode', '-geometry', '80x40',
                        '-title', '\"', self.ext_debugger, ' started by Temanejo\"',
                        '-e', debugger_command_line]
        }

        try:
            terminal = os.environ['TERM']
        except:
            logging.warning('terminal %s not found. Falling back to xterm.' % terminal)
            terminal = 'xterm'
        if not terminal in known_terminals:
            logging.warning('terminal %s not known. Falling back to xterm.' % terminal)
            terminal = 'xterm'

        if which(terminal) is None:
            logging.error('terminal %s is not installed.' % terminal)
            return

        cmd = known_terminals[terminal]

        if gdb_cmd is not None:
            cmd += ['-x', tmpfile[1]]

        logging.info('executing external debugger:\n%s' % ' '.join(cmd))

        # execute gdb
        msg = ''
        try:
            self.debugger = subprocess.Popen(' '.join(cmd), shell=True)  # , stdin=subprocess.PIPE)
        except  (OSError, msg):
            logging.error(msg)
            logging.error(
                'Opening subprocess for external debugger failed:\n\t%s', ' '.join(cmd))
            return

        # remove temporary file
        if gdb_cmd is not None:
            time.sleep(3)
            os.unlink(tmpfile[1])

    def on_export(self, action):
        export_dlg = ExportDialog(self)
        response = export_dlg.run()
        if response == gtk.RESPONSE_OK:
            format = export_dlg.get_format()
            filename = export_dlg.get_filename()
            quality = export_dlg.get_quality()
            framerate = export_dlg.get_framerate()
            export_dlg.destroy()

            if format == 'text':
                self.export_graph_text(filename)
            elif format == 'png':
                self.export_graph_image(filename, format)
            elif format == 'jpeg':
                self.export_graph_image(filename, format, quality)
            elif format == 'dot':
                try:
                    networkx.write_dot(self.graph, filename)
                except AttributeError:
                    loggin.error('Writing a dot file is not supported by your '
                                 'networkx installation.')
            elif format == 'png_ser':
                self.export_graph_image(
                    filename.replace('NNNNN', ' % 05d' % 0), 'png')
                # for each unfinished tasks:
                for i in range(1, self.graph.order()+1-self.status_counter["F"]):
                    self.on_run('export')
                    self.export_graph_image(
                        filename.replace('NNNNN', ' % 05d' % i), 'png')

            if format == 'gif':
                # save temporary png files:
                temp_filename_base = 'temanejo_TEMP%s' %\
                    time.strftime('_%Y%m%d-%H%M%S.png', time.localtime())
                self.export_graph_image(temp_filename_base+'-%05d' % 0, 'png')
                # for each unfinished tasks:
                for i in range(1, self.graph.order()+1-self.status_counter["F"]):
                    self.on_run('export')
                    self.export_graph_image(
                        temp_filename_base+'-%05d' % i, 'png')
                # convert to animation
                res = os.system('convert -delay %d %s* %s' %
                                (100//framerate,
                                 temp_filename_base,
                                 filename))
                if res == 127:
                    logging.error('convert (ImageMagick) is not installed')
                elif res != 0:
                    logging.error('convert exited with error code %d', res)
                else:
                    os.system('rm -f ' + temp_filename_base + '*')
            if format == 'avi':
                # save temporary jpeg files:
                temp_filename_base = 'temanejo_TEMP%s' %\
                    time.strftime('_%Y%m%d-%H%M%S.jpg', time.localtime())
                self.export_graph_image(
                    temp_filename_base+'-%05d' % 0, 'jpeg', quality)
                # for each unfinished tasks:
                for i in range(1, self.graph.order()+1-self.status_counter["F"]):
                    self.on_run('export')
                    self.export_graph_image(
                        temp_filename_base+'-%05d' % i, 'jpeg', quality)
                # convert to animation
                res = os.system(
                    'mencoder "mf://' + temp_filename_base + '*" -mf fps='
                    + str(framerate) + ' -mf type=jpeg -o ' + filename
                    + ' -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=1800')

                if res == 127:
                    logging.error('mencoder is not installed')
                elif res != 0:
                    logging.error('mencoder exited with error code %d', res)
                else:
                    os.system('rm -f ' + temp_filename_base + '*')
        else:
            export_dlg.destroy()

    def export_graph_text(self, filename):
        if filename is None or filename == '':
            logging.error('empty filename')
            return

        file_basename = filename.rpartition('.')[0]
        f = open(file_basename + '.nodes', 'w')
        f.write('#taskId\tfuncId\tfuncname\tduration\n')

        for n in self.graph.nodes():
            taskId = self.graph.get_taskId(n)
            funcId = self.graph.get_funcId(taskId)
            funcname = self.graph.get_funcname(funcId)
            if self.graph._status[n] != 3:
                dt = -1
            else:
                dt = self.graph.node[n]['ts_f']-self.graph.node[n]['ts_r']
            f.write('%s\t%s\t%s\t%s\n' % (taskId, funcId, funcname, dt))

        f.close

        f = open(file_basename + '.edges', 'w')
        f.write('#taskId\ttaskId\tmemaddr\n')

        for e in self.graph.edges():
            e_0 = e[0]
            e_1 = e[1]
            for ee in self.graph[e_0][e_1]:
                t = hex(self.graph[e_0][e_1][ee]['orig_addr'])
                f.write('%d\t%d\t%s\n' % (e_0, e_1, t))

        f.close

    def export_graph_image(self, filename, format, quality=None):
        rect = self.widget.get_allocation()
        pixbuf = gtk.gdk.Pixbuf(
            gtk.gdk.COLORSPACE_RGB, True, 8, rect.width, rect.height)
        pixbuf.get_from_drawable(self.widget.window,
                                 gtk.gdk.colormap_get_system(),
                                 0, 0, 0, 0,
                                 rect.width, rect.height)
        logging.info('writing %s to %s', format, filename)
        if quality is not None:
            pixbuf.save(filename, format, {"quality": str(quality)})
        else:
            pixbuf.save(filename, format)

    def cleanup(self):
        # graph
        self.graph.clear()

        # last event and taskId for status bar
        self.last_event = 'AYU_EVENT_NULL'
        self.last_check_event = 'AYU_EVENT_NULL'
        self.last_taskId = client_conf.no_task_id

        # widget
        self.widget.first_drawing_event = True
        self.widget.graphview.clear()

        # communticator thread
        if self.comm is not None:
            self.comm.terminate()
            self.comm = None

        # thread for execution of the application
        if self.execut is not None:
            os.system('kill ' + str(self.execut.pid))
            self.execut = None

        # number of registered functions
        self.func_count = 0

        # barrier counter
        self.barrier_count = 0

        # counters
        self.func_counter.clear()                #  use a dict
        keys = self.task_states[:-1]+["total",]  #  with keys graph.task_states-"X"+"total"
        for key in keys:
            self.func_counter[key] = dict()      # init to empty dict

        self.thread_counter.clear()               # use a dict
        keys = self.task_states[1:-1]+["total",]  #  with keys graph.task_states-"NQ"-"X"+"total"
        for key in keys:
            self.thread_counter[key] = dict()     # init to empty dict

        self.status_counter.clear()              # use a dict
        keys = self.task_states[:-1]             # with keys graph.task_states -"X"
        for key in keys:
            self.status_counter[key] = 0         # init to zero

        # node colour list
        self.liststore_nodecol.clear()
        self.combo_nodecollist.set_active(-1)
        self.combo_nodecollist.set_active(
            self.node_color_property_list.index(self.widget.graphview.node_colours))

        # margin colour list
        self.liststore_margcol.clear()
        self.combo_margcollist.set_active(-1)
        self.combo_margcollist.set_active(
            self.margin_color_property_list.index(self.widget.graphview.margins_mean))

        # node shape list
        self.liststore_shape.clear()
        self.combo_shapelist.set_active(-1)
        self.combo_shapelist.set_active(
            self.node_shape_property_list.index(self.widget.graphview.shapes_mean))

        # checkboxes etc.
        self.check_preselecttask.set_active(False)
        self.check_anyEvent.set_active(False)
        self.check_preruntask.set_active(True)
        self.check_removetask.set_active(False)
        self.check_waiton.set_active(False)
        self.check_barrier.set_active(False)
        self.check_stopontask.set_active(False)
#        self.check_debugfunction.set_active(False)
        self.label_stopontask.set_text('-none-')

        listmodel = self.combo_debugfunction.get_model()
        iterat = listmodel.get_iter_first()
        while listmodel.remove(iterat):
            pass
        listmodel.append(['-none-'])
        self.combo_debugfunction.set_active(0)

        self.widget.queue_draw()

    def get_active_text(self, combobox):
        model = combobox.get_model()
        active = combobox.get_active()
        if active < 0:
            return None
        return model[active][0]

    def on_connect(self, action):
        self.warning.remove_all(self.warning_lastmsg_cid)
        self.warning_eventbox.modify_bg(gtk.STATE_NORMAL, None)
        self.warning.remove_all(self.warning_lastmsg_cid)

        # decide whether to connect or not ----------
        do_connect = False
        if self.auto_connect is True:
            do_connect = True
        else:
            # connection Dialog ----------------------
            connect_dlg = ConnectDialog(self)
            response = connect_dlg.run()

            if response == gtk.RESPONSE_ACCEPT:
                do_connect = True
            else:
                connect_dlg.destroy()
                return

        # do connect ---------------------------------
        if do_connect is True:
            # cleanup --------------------------------
            self.cleanup()

            if self.auto_connect is False:
                # get dialogue input
                if connect_dlg.get_local_remote() == 0:  # local
                    self.executable = connect_dlg.get_exec()
                    self.executable_args = connect_dlg.get_exec_args()
                    preload_lib = connect_dlg.get_lib_check()
                    if preload_lib is True:
                        self.library = connect_dlg.get_lib()
                    else:
                        self.library = None
                    configuration = connect_dlg.get_cfg()
                    self.connect_host = 'localhost'
                    self.connect_port = connect_dlg.get_port()
                    cpus = connect_dlg.get_cpus()
                    logging.info('executable:           %s', self.executable)
                    logging.info('executable arguments: %s', self.executable_args)
                    logging.info('Ayudame library:      %s', self.library)
                    logging.info('SMPSS configuration:  %s', configuration)
                    logging.info('hostname:             %s', self.connect_host)
                    logging.info('port:                 %s', self.connect_port)
                    logging.info('number of cpus:       %s', cpus)

                else:  # remote
                    self.executable = None
                    self.executable_args = ''
                    self.library = None
                    configuration = None
                    self.connect_host = connect_dlg.get_host()
                    self.connect_port = connect_dlg.get_port()
                    cpus = None
                    logging.info('hostname:            %s', self.connect_host)
                    logging.info('port:                %s', self.connect_port)

                connect_dlg.destroy()

            else:  # auto_connect == True
                self.executable = None
                self.executable_args = ''
                self.library = None
                configuration = None
                cpus = None
                logging.info('hostname:            %s', self.connect_host)
                logging.info('port:                %s', self.connect_port)

            # initialise thread counters
            if cpus is not None:
                client_conf.nthreads = cpus
            else:
                client_conf.nthreads = THREADS_DISPLAY
            self.spin_numthreads.set_value(client_conf.nthreads)

            # set comm thread id
            self.widget.graphview.commthreadId = client_conf.nthreads

            # HACK: the following assumes threads are numbered consecutive
            #       and there is an additinal "-1" (meaning non-specified?)
            for i in range(client_conf.nthreads+1):
                self.thread_counter["total"][i] = 0
                self.thread_counter["Q"][i] = 0
                self.thread_counter["R"][i] = 0
                self.thread_counter["F"][i] = 0
#                self.thread_counter["X"][i] = 0
            self.thread_counter["Q"][-1] = 0
            self.thread_counter["R"][-1] = 0
            self.thread_counter["F"][-1] = 0
            self.thread_counter["total"][-1] = 0

            # node colour list
            self.liststore_nodecol.clear()
            self.combo_nodecollist.set_active(-1)
            self.combo_nodecollist.set_active(
                self.node_color_property_list.index(self.widget.graphview.node_colours))

            # margin colour list
            self.liststore_margcol.clear()
            self.combo_margcollist.set_active(-1)
            self.combo_margcollist.set_active(
                self.margin_color_property_list.index(self.widget.graphview.margins_mean))

            # node shape list
            self.liststore_shape.clear()
            self.combo_shapelist.set_active(-1)
            self.combo_shapelist.set_active(
                self.node_shape_property_list.index(self.widget.graphview.shapes_mean))

            # -------------

        # ----------------------------------------
        # run executable -------------------------
        if self.executable is not None:
            if self.executable == '':
                self.print_warning('Application not specified.')

            elif not os.path.isfile(self.executable):
                self.print_warning('Application %s not found.' % self.executable)
                return
            elif not os.access(self.executable, os.X_OK):
                self.print_warning('File %s is not executable.' % self.executable)
                return
            else:
                if (preload_lib is True
                        and self.library is None):
                    self.print_warning('Ayudame library not specified.')
                    return
                if '/' not in self.executable:
                    self.executable = './' + self.executable

                # ayudame specific environment variables

                env_dict = os.environ

                loglevel_tem = logging.getLogger().getEffectiveLevel()
                if loglevel_tem == logging.CRITICAL:
                    loglevel_ayu = -1 # absolute silence
                elif loglevel_tem == logging.ERROR:
                    loglevel_ayu = 0 # default
                elif loglevel_tem == logging.WARNING:
                    loglevel_ayu = 1 # default
                elif loglevel_tem == logging.INFO:
                    loglevel_ayu = 2 # default
                elif loglevel_tem == logging.DEBUG:
                    loglevel_ayu = 3 # default
                env_dict['AYU_VERBOSE'] = str(loglevel_ayu)
                env_dict['AYU_PORT'] = str(self.connect_port)
                # StarSs specific environment variables
                env_dict['CSS_NUM_CPUS'] = str(cpus)
                if configuration is not None:
                    env_dict['CSS_CONFIG_FILE'] = configuration
                if preload_lib is True:
                    env_dict['LD_PRELOAD'] = self.library
                # Nanos++ specific environment variables
                # TODO: There are a lot more special vars like
                #       NX_SCHEDULE, ... to handle
                # check if user has set OmpSs instrumentation
                need_instrumentation = True
                if "NX_ARGS" not in env_dict:
                    env_dict["NX_ARGS"]=""
                if "NX_INSTRUMENTATION" in env_dict:
                    need_instrumentation = False
                if "NX_ARGS" in env_dict and (env_dict["NX_ARGS"].find("--instrumentation") != -1 ):
                    need_instrumentation = False
                if need_instrumentation:
                    env_dict["NX_ARGS"]= env_dict["NX_ARGS"]  + " --instrumentation ayudame"
                if "NX_ARGS" in env_dict and \
                    ( "--smp-workers" not in env_dict["NX_ARGS"] ):
                    env_dict["NX_ARGS"] += " --smp-workers " + str(cpus)
                #env_dict['OMP_NUM_THREADS'] = str(cpus)
                # StarPU specific environment variables
                env_dict['STARPU_NCPU'] = str(cpus)

                exec_cmd = [self.executable]
                exec_cmd += shlex.split(self.executable_args)

                msg = ''
                try:
                    self.execut = subprocess.Popen(exec_cmd, env=env_dict)
                except (OSError, msg):
                    self.print_error('OSError: %s' % msg)
                    return
                end = self.logbuffer.get_end_iter()
                self.logbuffer.insert(
                    end,
                    '\nExecuting %s with PID %d' % (self.exec_cmd,
                                                    self.execut.pid))

                logging.info(
                    'executing \'%s\' with PID %d',
                    string.join(exec_cmd), self.execut.pid)
                time.sleep(.5)

        # ----------------------------------------

        # set up client socket -------------------
        logging.info('creating socket ...')
        try:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except (socket.error, msg):
            self.sock = None
            self.print_error('creating socket failed: %s' % msg)
            return

        connect_count = 0
        while(1):
            msg = ''
            try:
                self.sock.connect((self.connect_host, self.connect_port))
            except (socket.error, msg):
                self.print_warning('connecting to host %s:%d failed: %s' %
                                    (self.connect_host, self.connect_port, msg),
                                   to_dialogue=False)
                self.print_warning('Trying again... %s times' % (3-connect_count),
                                   to_statusbar=False,
                                   to_dialogue=False)
                time.sleep(self.retry_interval)
                connect_count += 1
                if connect_count > self.max_retries:
                    self.print_error(
                        'Connection to host could not be established.')
                    self.sock.close()
                    self.sock = None
                    self.auto_connect = False
                    self.on_connect('on_connect')
                    return
            else:
                logging.info('Connection to server established.')
                break
        # ----------------------------------------

        # launch communicater thread

        self.comm = Communicator(self.sock, self)
        self.comm.deamon = True
        self.comm.start()
        # ----------------------------------------
        self.auto_connect = False

        # wait until application is up and running
        tmp_max_retries = 100
        while self.allow_requests is False:
            time.sleep(.1)
            tmp_max_retries -= 1
            if tmp_max_retries == 0:
                self.print_error("Application refused to become ready for receiving requests.")

        # if Temanejo is in viewer mode, one "step request" has to be sent
        # because Ayudame automatically pauses on AYU_INIT event from runtime.

        if self.viewer_mode == True:
            self.comm.send_request(0, 'AYU_STEP', 10, ignore_viewer_mode=True)

    def print_warning(self, message, to_logging=True, to_statusbar=True, to_dialogue=True):
        if to_logging is True:
            logging.warning(message)
        if to_statusbar is True:
            self.warning_lastmsg_id = self.warning.push(
                self.warning_lastmsg_cid, "WARNING: %s" % message)
            self.warning_eventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(1., 1., 0.))

        if to_dialogue is True:
            md = gtk.MessageDialog(
                 self,
                 gtk.DIALOG_DESTROY_WITH_PARENT,
                 gtk.MESSAGE_WARNING,
                 gtk.BUTTONS_CLOSE,
                 message)
            md.run()
            md.destroy()

    def print_error(self, message, to_logging=True, to_statusbar=True, to_dialogue=True):
        if to_logging is True:
            logging.error(message)
        if to_statusbar is True:
            self.warning_lastmsg_id = self.warning.push(
                self.warning_lastmsg_cid, "ERROR: %s" % message)
            self.warning_eventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(1., 0., 0.))

        if to_dialogue is True:
            md = gtk.MessageDialog(
                 self,
                 gtk.DIALOG_DESTROY_WITH_PARENT,
                 gtk.MESSAGE_ERROR,
                 gtk.BUTTONS_CLOSE,
                 message)
            md.run()
            md.destroy()

    def on_refresh(self, action):
        self.widget.graphview.is_invalid = True
        self.update_ui('all')

    def on_run(self, action):
        if self.comm is None:
            logging.critical('No communicator!')
            return
        self.actiongroup.get_action('Run_fast').set_sensitive(True)
        logging.debug('about to send request AYU_STEP')
        self.comm.send_request(0, 'AYU_STEP', 1)
        logging.debug('sent request AYU_STEP')
        time.sleep(.1)
        self.update_ui('all')

    def on_run_nsteps(self, action):
        if self.comm is None:
            return
        self.comm.send_request(0, 'AYU_STEP', self.n_steps)
        time.sleep(.1*math.sqrt(self.n_steps))
        self.update_ui('all')

    def on_about(self, action):
        dlg = AboutDialog()
        dlg.run()
        dlg.destroy()

    def on_settings(self, action):
        graphview = self.widget.graphview
        dlg = SettingsDialog(self)
        response = dlg.run()
        time.sleep(.2)
        if response == gtk.RESPONSE_OK or \
                response == gtk.RESPONSE_APPLY:
            self.widget.autozoom = dlg.get_check_autozoom()
            graphview.show_markers = dlg.get_check_markers()
            graphview.addr_key = dlg.get_edge_colours()
            graphview.draw_edge_labels = dlg.get_edge_labels()
            graphview.margin_width = dlg.get_spin_margin_width()
            graphview.edge_width = dlg.get_spin_edge_width()
            graphview.node_size = dlg.get_spin_node_size()
            if graphview.layout_direction != dlg.get_graph_direction():
                graphview.layout_direction = dlg.get_graph_direction()
                self.widget.zoom_once = True
                graphview.is_invalid = True
            if self.n_steps != dlg.get_spin_nsteps():
                self.n_steps = dlg.get_spin_nsteps()
                toolbutton = self.uimanager.get_widget('/ToolBar/Run_fast')
                toolbutton.set_label('Run '+str(self.n_steps)+' tasks')
            if graphview.use_layout != dlg.get_graph_layout():
                graphview.use_layout = dlg.get_graph_layout()
                self.widget.zoom_once = True
                graphview.is_invalid = True
            if (graphview.global_tasks_in_first_row !=
                    dlg.get_global_tasks_first_row()):
                graphview.global_tasks_in_first_row = \
                    dlg.get_global_tasks_first_row()
                self.widget.zoom_once = True
                graphview.is_invalid = True
            if self.toolbar_orientation != dlg.get_toolbar_orientation():
                self.toolbar_orientation = dlg.get_toolbar_orientation()
                # refresh graph widget and toolbar
                self.set_toolbar_orientation(self.toolbar_orientation)

            self.ext_debugger = dlg.get_extdebugger()

            self.widget.queue_draw()

        dlg.destroy()

    # def on_help(self, action):
    #     dlg=HelpDialog()
    #     dlg.run()
    #     dlg.destroy()

    def on_quit(self, action, data=None):
        logging.info('Stopping communication thread.')
        if self.comm is not None:
            self.comm.terminate()
            self.sock.shutdown(socket.SHUT_RDWR)
            self.sock.close()

        logging.info('Stopping application thread.')
        if (self.execut is not None and
                self.viewer_mode is False):
#            self.execut.send_signal(signal.SIGTERM) # for python ver. >= 2.6
            os.system('kill ' + str(self.execut.pid))

        logging.info('### Exiting ###')
        gtk.main_quit()

        return False

    def push_statusbar(self, msg=None):
        if msg is None:
            msg = 'last event: %s for taskId %d' % \
                (self.last_event, self.last_taskId)

        self.statusbar.pop(self.statusbar_lastmsg_cid)
        self.statusbar_lastmsg_id = self.statusbar.push(
            self.statusbar_lastmsg_cid, msg)

        self.statusbar2.pop(self.statusbar2_lastmsg_cid)
        self.statusbar2_lastmsg_id = self.statusbar2.push(
            self.statusbar2_lastmsg_cid,
            '%d nodes, %d edges, %d addresses, %d original addresses' %
            (self.graph.order(),
             self.graph.size(),
             len(self.graph.addresses),
             len(self.graph.orig_addresses)))

    def highlight_checkbox(self, event=None):
        if event is None:
            event = self.last_check_event
        # all to default
        self.eventbox_preruntask.modify_style(self.check_default_style)
        self.eventbox_removetask.modify_style(self.check_default_style)
        self.eventbox_waiton.modify_style(self.check_default_style)
        self.eventbox_barrier.modify_style(self.check_default_style)

        if event == 'AYU_PRERUNTASK':
            self.eventbox_preruntask.modify_bg(
                gtk.STATE_NORMAL, gtk.gdk.Color(65535, 0, 0))
        elif event == 'AYU_REMOVETASK':
            self.eventbox_removetask.modify_bg(
                gtk.STATE_NORMAL, gtk.gdk.Color(65535, 0, 0))
        elif event == 'AYU_WAITON':
            self.eventbox_waiton.modify_bg(
                gtk.STATE_NORMAL, gtk.gdk.Color(65535, 0, 0))
        elif event == 'AYU_BARRIER':
            self.eventbox_barrier.modify_bg(
                gtk.STATE_NORMAL, gtk.gdk.Color(65535, 0, 0))

        #self.eventbox_preruntask.queue_draw()
        #self.eventbox_removetask.queue_draw()
        #self.eventbox_waiton.queue_draw()
        #self.eventbox_barrier.queue_draw()

    def update_ui(self, widget='all'):
        # widget is one of 'all', 'log', 'statusbar', 'counters', 'graphwidget'

        graphview = self.widget.graphview

        # in trivial cases where communicator or graph are inexistent do nothing
        if (self.comm is None or
                self.comm.working is True or
                self.graph.order() == 0):
            return

        if widget == 'log' or widget == 'all':
            text = self.prelog
            self.prelog = ''
            end = self.logbuffer.get_end_iter()
            self.logbuffer.insert(end, text)
            self.logview.scroll_mark_onscreen(self.logbuffer.get_insert())

#            self.logview.queue_draw()

#        if widget == 'checkboxes' or widget == 'all':
            # checkboxes
#            self.highlight_checkbox()
        if widget == 'statusbar' or widget == 'all':
            # status bar
            self.push_statusbar()

        if widget == 'counters' or widget == 'all':

            # make sure, the number of threads is taken into account
            self.on_shapelist("update_ui", "update")
            # node colour counters
            if graphview.node_colours == 'function':
                # function counter table
                i = 0
                for (fId, count) in self.func_counter["total"].items():
                    ls_iter = self.liststore_nodecol.get_iter(i)
                    i += 1
                    self.liststore_nodecol.set_value(
                        ls_iter, 2, str(self.func_counter["NQ"][fId]))
                    self.liststore_nodecol.set_value(
                        ls_iter, 3, str(self.func_counter["Q"][fId]))
                    self.liststore_nodecol.set_value(
                        ls_iter, 4, str(self.func_counter["R"][fId]))
                    self.liststore_nodecol.set_value(
                        ls_iter, 5, str(self.func_counter["F"][fId]))
#                    self.liststore_nodecol.set_value(
#                       ls_iter, 6, str(self.func_counter["X"][fId]))
                    self.liststore_nodecol.set_value(ls_iter, 7, str(count))
                ls_iter = self.liststore_nodecol.get_iter(
                    len(self.func_counter["total"]))
                self.liststore_nodecol.set_value(
                    ls_iter, 2, str(sum(self.func_counter["NQ"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 3, str(sum(self.func_counter["Q"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 4, str(sum(self.func_counter["R"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 5, str(sum(self.func_counter["F"].values())))
#                self.liststore_nodecol.set_value(
#                   ls_iter, 6, str(sum(self.func_counter["X"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 7, str(sum(self.func_counter["total"].values())))

            elif graphview.node_colours == 'thread':
                for thId in range(client_conf.nthreads+1):
                    ls_iter = self.liststore_nodecol.get_iter(thId)
                    self.liststore_nodecol.set_value(
                        ls_iter, 3, str(self.thread_counter["Q"][thId]))
                    self.liststore_nodecol.set_value(
                        ls_iter, 4, str(self.thread_counter["R"][thId]))
                    self.liststore_nodecol.set_value(
                        ls_iter, 5, str(self.thread_counter["F"][thId]))
#                    self.liststore_nodecol.set_value(
#                       ls_iter, 6, str(self.thread_counter["X"][thId]))
                    self.liststore_nodecol.set_value(
                        ls_iter, 7, str(self.thread_counter["total"][thId]))
                ls_iter = self.liststore_nodecol.get_iter(client_conf.nthreads+1)
                self.liststore_nodecol.set_value(
                    ls_iter, 3, str(sum(self.thread_counter["Q"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 4, str(sum(self.thread_counter["R"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 5, str(sum(self.thread_counter["F"].values())))
#                self.liststore_nodecol.set_value(
#                   ls_iter, 6, str(sum(self.thread_counter["X"].values())))
                self.liststore_nodecol.set_value(
                    ls_iter, 7, str(sum(self.thread_counter["total"].values())))

            elif graphview.node_colours == 'status':
                self.liststore_nodecol.set_value(
                    self.liststore_nodecol.get_iter(0), 7,
                    str(self.status_counter["NQ"]))
                self.liststore_nodecol.set_value(
                    self.liststore_nodecol.get_iter(1), 7,
                    str(self.status_counter["Q"]))
                self.liststore_nodecol.set_value(
                    self.liststore_nodecol.get_iter(2), 7,
                    str(self.status_counter["R"]))
                self.liststore_nodecol.set_value(
                    self.liststore_nodecol.get_iter(3), 7,
                    str(self.status_counter["F"]))
#                self.liststore_nodecol.set_value(
#                   self.liststore_nodecol.get_iter(4), 7,
#                   str(self.status_counter["X"]))
                self.liststore_nodecol.set_value(
                    self.liststore_nodecol.get_iter(5), 7,
                    str(self.status_counter["NQ"] +
                        self.status_counter["Q"] +
                        self.status_counter["R"] +
                        self.status_counter["F"]))
#                       +self.status_counter["X"]))

            elif graphview.node_colours == 'task_duration':
                graphview.get_dtminmax()
                # fill colour list
                if graphview.dt_min <= graphview.dt_max:
                    self.liststore_nodecol.clear()
                    fs = palette_rgb_g2r(
                        0.001,
                        math.log(graphview.dt_max -
                                 graphview.dt_min + 1) + .001)
                    rgb = paletteFractions_to_rgb(fs)
                    color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                    self.liststore_nodecol.append(
                        ['min', color_str, '0', '0', '0', '0', '0',
                         str(graphview.dt_min)])
                    for i in range(3, 12):
                        i10 = 10**i
                        if (i10 > graphview.dt_min and
                                i10 < graphview.dt_max):
                            fs = palette_rgb_g2r(
                                math.log(i10 -
                                         graphview.dt_min + 1)
                                + .001,
                                math.log(graphview.dt_max -
                                         graphview.dt_min + 1)
                                + .001)
                            rgb = paletteFractions_to_rgb(fs)
                            color_str = '#%02x%02x%02x' % (rgb[0],
                                                           rgb[1],
                                                           rgb[2])
                            self.liststore_nodecol.append(
                                ['', color_str, '0', '0', '0', '0', '0',
                                 str(i10)])

                    rgb = paletteFractions_to_rgb(palette_rgb_g2r(1, 1))
                    color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                    self.liststore_nodecol.append(
                        ['max', color_str, '0', '0', '0', '0', '0',
                         str(graphview.dt_max)])
                else:
                    pass
                    #self.liststore_nodecol.append(
                    #    ['no data', 'white', '0', '0', '0', '0', '0'])

            elif graphview.node_colours == 'distance_to_marked':
                graphview.calc_all_distances(
                    graphview.marked_node)

                # fill status list
                if (graphview.marked_node is not None and
                        graphview.dist_max != 0):
                    self.liststore_nodecol.clear()
                    fs = palette_rgb_g2r(
                        1, graphview.dist_max)
                    rgb = paletteFractions_to_rgb(fs)
                    color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                    self.liststore_nodecol.append(
                        ['min', color_str, '0', '0', '0', '0', '0', '1'])
                    for i in range(5, graphview.dist_max, 5):
                        if i <= graphview.dist_max:
                            fs = palette_rgb_g2r(
                                i, graphview.dist_max)
                            rgb = paletteFractions_to_rgb(fs)
                            color_str = '#%02x%02x%02x' % (rgb[0],
                                                           rgb[1],
                                                           rgb[2])
                            self.liststore_nodecol.append(
                                ['', color_str, '0', '0', '0', '0', '0',
                                 str(i)])

                    rgb = paletteFractions_to_rgb(palette_rgb_g2r(1, 1))
                    color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                    self.liststore_nodecol.append(
                        ['max', color_str, '0', '0', '0', '0', '0',
                         str(graphview.dist_max)])

            # margin colour counters
            if graphview.margins_mean == 'function':
                # function counter table
                i = 0
                for (fId, count) in self.func_counter["total"].items():
                    ls_iter = self.liststore_margcol.get_iter(i)
                    i += 1
                    self.liststore_margcol.set_value(
                        ls_iter, 2, str(self.func_counter["NQ"][fId]))
                    self.liststore_margcol.set_value(
                        ls_iter, 3, str(self.func_counter["Q"][fId]))
                    self.liststore_margcol.set_value(
                        ls_iter, 4, str(self.func_counter["R"][fId]))
                    self.liststore_margcol.set_value(
                        ls_iter, 5, str(self.func_counter["F"][fId]))
#                    self.liststore_margcol.set_value(
#                       ls_iter, 6, str(self.func_counter["X"][fId]))
                    self.liststore_margcol.set_value(ls_iter, 7, str(count))
                ls_iter = self.liststore_margcol.get_iter(
                    len(self.func_counter["total"]))
                self.liststore_margcol.set_value(
                    ls_iter, 2, str(sum(self.func_counter["NQ"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 3, str(sum(self.func_counter["Q"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 4, str(sum(self.func_counter["R"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 5, str(sum(self.func_counter["F"].values())))
#                self.liststore_margcol.set_value(
#                   ls_iter, 6, str(sum(self.func_counter["X"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 7, str(sum(self.func_counter["total"].values())))

            elif graphview.margins_mean == 'thread':
                for thId in range(client_conf.nthreads+1):
                    ls_iter = self.liststore_margcol.get_iter(thId)
                    self.liststore_margcol.set_value(
                        ls_iter, 3, str(self.thread_counter["Q"][thId]))
                    self.liststore_margcol.set_value(
                        ls_iter, 4, str(self.thread_counter["R"][thId]))
                    self.liststore_margcol.set_value(
                        ls_iter, 5, str(self.thread_counter["F"][thId]))
#                    self.liststore_margcol.set_value(
#                       ls_iter, 6, str(self.thread_counter["X"][thId]))
                    self.liststore_margcol.set_value(
                        ls_iter, 7, str(self.thread_counter["total"][thId]))
                ls_iter = self.liststore_margcol.get_iter(client_conf.nthreads+1)
                self.liststore_margcol.set_value(
                    ls_iter, 3, str(sum(self.thread_counter["Q"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 4, str(sum(self.thread_counter["R"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 5, str(sum(self.thread_counter["F"].values())))
#                self.liststore_margcol.set_value(
#                   ls_iter, 6, str(sum(self.thread_counter["X"].values())))
                self.liststore_margcol.set_value(
                    ls_iter, 7, str(sum(self.thread_counter["total"].values())))

            elif graphview.margins_mean == 'status':
                self.liststore_margcol.set_value(
                    self.liststore_margcol.get_iter(0), 7,
                    str(self.status_counter["NQ"]))
                self.liststore_margcol.set_value(
                    self.liststore_margcol.get_iter(1), 7,
                    str(self.status_counter["Q"]))
                self.liststore_margcol.set_value(
                    self.liststore_margcol.get_iter(2), 7,
                    str(self.status_counter["R"]))
                self.liststore_margcol.set_value(
                    self.liststore_margcol.get_iter(3), 7,
                    str(self.status_counter["F"]))
#                self.liststore_margcol.set_value(
#                   self.liststore_margcol.get_iter(4), 7,
#                   str(self.status_counter["X"]))
                self.liststore_margcol.set_value(
                    self.liststore_margcol.get_iter(5), 7,
                    str(self.status_counter["NQ"] +
                        self.status_counter["Q"] +
                        self.status_counter["R"] +
                        self.status_counter["F"]))
#                       +self.status_counter["X"]))

            elif graphview.margins_mean == 'task_duration':
                graphview.get_dtminmax()
                # fill status list
                if graphview.dt_min <= graphview.dt_max:
                    self.liststore_margcol.clear()
                    fs = palette_rgb_g2r(
                        0.001,
                        math.log(graphview.dt_max -
                                 graphview.dt_min + 1) + .001)
                    rgb = paletteFractions_to_rgb(fs)
                    color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                    self.liststore_margcol.append(
                        ['min', color_str, '0', '0', '0', '0', '0',
                         str(graphview.dt_min)])
                    for i in range(3, 12):
                        i10 = 10**i
                        if (i10 > graphview.dt_min and
                                i10 < graphview.dt_max):
                            fs = palette_rgb_g2r(
                                math.log(i10 -
                                         graphview.dt_min + 1)
                                + .001,
                                math.log(graphview.dt_max -
                                         graphview.dt_min + 1)
                                + .001)
                            rgb = paletteFractions_to_rgb(fs)
                            color_str = '#%02x%02x%02x' % (rgb[0],
                                                           rgb[1],
                                                           rgb[2])
                            self.liststore_margcol.append(
                                ['', color_str, '0', '0', '0', '0', '0',
                                 str(i10)])

                    rgb = paletteFractions_to_rgb(palette_rgb_g2r(1, 1))
                    color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
                    self.liststore_margcol.append(
                        ['max', color_str, '0', '0', '0', '0', '0',
                         str(graphview.dt_max)])
                else:
                    pass
                    #self.liststore_margcol.append(
                    #   ['no data', 'white', '0', '0', '0', '0', '0'])

            # node shape counters
            if graphview.shapes_mean == 'function':
                # function counter table
                i = 0
                for (fId, count) in self.func_counter["total"].items():
                    ls_iter = self.liststore_shape.get_iter(i)
                    i += 1
                    self.liststore_shape.set_value(
                        ls_iter, 2, str(self.func_counter["NQ"][fId]))
                    self.liststore_shape.set_value(
                        ls_iter, 3, str(self.func_counter["Q"][fId]))
                    self.liststore_shape.set_value(
                        ls_iter, 4, str(self.func_counter["R"][fId]))
                    self.liststore_shape.set_value(
                        ls_iter, 5, str(self.func_counter["F"][fId]))
#                    self.liststore_shape.set_value(
#                       ls_iter, 6, str(self.func_counter["X"][fId]))
                    self.liststore_shape.set_value(ls_iter, 7, str(count))
                ls_iter = self.liststore_shape.get_iter(len(self.func_counter["total"]))
                self.liststore_shape.set_value(
                    ls_iter, 2, str(sum(self.func_counter["NQ"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 3, str(sum(self.func_counter["Q"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 4, str(sum(self.func_counter["R"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 5, str(sum(self.func_counter["F"].values())))
#                self.liststore_shape.set_value(
#                   ls_iter, 6, str(sum(self.func_counter["X"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 7, str(sum(self.func_counter["total"].values())))

            elif graphview.shapes_mean == 'thread':
                for thId in range(client_conf.nthreads+1):
                    ls_iter = self.liststore_shape.get_iter(thId)
#                    ls_iter = self.liststore_shape.get_iter(thId)
                    self.liststore_shape.set_value(
                        ls_iter, 3, str(self.thread_counter["Q"][thId]))
                    self.liststore_shape.set_value(
                        ls_iter, 4, str(self.thread_counter["R"][thId]))
                    self.liststore_shape.set_value(
                        ls_iter, 5, str(self.thread_counter["F"][thId]))
#                    self.liststore_shape.set_value(
#                       ls_iter, 6, str(self.thread_counter["X"][thId]))
                    self.liststore_shape.set_value(
                        ls_iter, 7, str(self.thread_counter["total"][thId]))
                ls_iter = self.liststore_shape.get_iter(client_conf.nthreads+1)
                self.liststore_shape.set_value(
                    ls_iter, 3, str(sum(self.thread_counter["Q"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 4, str(sum(self.thread_counter["R"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 5, str(sum(self.thread_counter["F"].values())))
#                self.liststore_shape.set_value(
#                   ls_iter, 6, str(sum(self.thread_counter["X"].values())))
                self.liststore_shape.set_value(
                    ls_iter, 7, str(sum(self.thread_counter["total"].values())))

        if widget == 'graphwidget' or widget == 'all':
        #if self.widget.autozoom or self.widget.zoom_once:
        #    self.widget.zoom_to_fit()
        #else:
        #    # draw!
            self.widget.queue_draw()

    def register_function(self, fId, fName):
        logging.info("register_function, %s %s " % (fId, fName))
        self.graph.set_funcname(fId, fName)

        # fill function list
        if self.widget.graphview.node_colours == 'function':
            rgb = paletteFractions_to_rgb(self.widget.graphview.palette_nodes[self.widget.graphview.hash_id(propFunc.index(fId))])
            color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
            self.liststore_nodecol.insert(
                len(propFunc)-1,
                [fName, color_str, '0', '0', '0', '0', '0', '0'])

        if self.widget.graphview.margins_mean == 'function':
            rgb = paletteFractions_to_rgb(self.widget.graphview.palette_nodes[self.widget.graphview.hash_id(propFunc.index(fId))])
            color_str = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
            self.liststore_margcol.insert(
                len(propFunc)-1,
                [fName, color_str, '0', '0', '0', '0', '0', '0'])

        if self.widget.graphview.shapes_mean == 'function':
            # TODO!
            # check for number_of_functions > len(self.shape_list)
            self.liststore_shape.insert(
                len(propFunc)-1,
                [self.pb_list[fId], fName, '0', '0', '0', '0', '0', '0'])

        # number of registered functions
        self.func_count += 1
        # initiate counters
        self.func_counter["total"][fId] = 0
        self.func_counter["NQ"][fId] = 0
        self.func_counter["Q"][fId] = 0
        self.func_counter["R"][fId] = 0
        self.func_counter["F"][fId] = 0
#        self.func_counter["X"][fId] = 0
        # fill combo list
        self.combo_debugfunction.append_text(fName)

    def get_ayudame_libraries(self, exec_dir, module_dir):
        # search for libayudame.so in the order:
        # (0. option -L is handled elsewhere, so ignored here)
        # 1. $AYUDAME_LIB (is a file)
        # 2. $exec_dir/../lib/libayudame.so
        # 3. $module_dir/lib/libayudame.so

        candidates = [
            os.environ["AYUDAME_LIB"] if "AYUDAME_LIB" in os.environ else False,
            os.path.join(exec_dir, '../lib/libayudame.so'),
            os.path.join(module_dir, './lib/libayudame.so')
        ]
        check_candidates = lambda c: c and os.path.isfile(c)

        libraries = [c for c in candidates if check_candidates(c)]
        return libraries

    def test_get_ayudame_library_no_lib_present(self):
        assert self.get_ayudame_libraries("/no/dir/", "/no/dir/") == []

#******


#****c* Temanejo/ConnectDialog
# NAME
#   ConnectDialog
#   - dialogue for connecting to an Ayudame server
# DESCRIPTION
#   The class implements a dialogue with which the user can choose an
#   executable, the location of the libayudame.so, a configuration file,
#   port number and number of cores. On pressing the connect button the
#   executable will be executed and pause when reaching the initialisation
#   routine of the applications parallel runtime.
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
#   callback_check_lib
#   callback_choose_cfg
#   callback_choose_exec
#   callback_choose_lib
#   get_cfg
#   get_cpus
#   get_exec
#   get_exec_args
#   get_host
#   get_lib
#   get_lib_check
#   get_local_remote
#   get_port
# SOURCE


class ConnectDialog(gtk.Dialog):
    def __init__(self, parent):
        gtk.Dialog.__init__(self, title='Connect to Ayudame...', parent=self,
                            flags=(gtk.DIALOG_MODAL |
                                   gtk.DIALOG_DESTROY_WITH_PARENT),
                            buttons= (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                      gtk.STOCK_CONNECT, gtk.RESPONSE_ACCEPT))

        self.set_default_response(gtk.RESPONSE_ACCEPT)

        # notebook tab for local connection
        vbox0 = gtk.VBox()

        frame = gtk.Frame('Specify the executable:')
        self.entry_exec = gtk.Entry(max=0)
        if parent.executable is None or len(parent.executable) == 0:
            self.entry_exec.set_text('#enter executable here#')
        else:
            self.entry_exec.set_text(parent.executable)
        self.entry_exec.set_width_chars(64)
        self.entry_exec.set_activates_default(True)
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect(
            'clicked', self.callback_choose_exec, 'choose executable')
        hbox = gtk.HBox(False, 5)
        hbox.pack_start(self.entry_exec, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify command line options for executable:')
        self.entry_exec_args = gtk.Entry(max=0)
        # TODO: why should self.executable_args be a list or tuple, check this!
        #       for now we make sure it is.
        if not isinstance(parent.executable_args, (tuple, list)):
            executable_args = [parent.executable_args,]
        else:
            executable_args = parent.executable_args
        self.entry_exec_args.set_text(" ".join(executable_args))
        hbox = gtk.HBox(False, 5)
        hbox.pack_start(self.entry_exec_args, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify libayudame.so:')
        self.check_lib = gtk.CheckButton()
        self.check_lib.set_active(True)
        self.check_lib.connect(
            'toggled', self.callback_check_lib, 'use library')
        self.entry_lib = gtk.Entry(max=0)
        if parent.library is None or len(parent.library) == 0:
            self.entry_lib.set_text('#enter libayudame.so here#')
        else:
            self.entry_lib.set_text(parent.library)
        self.entry_lib.set_width_chars(64)
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect('clicked', self.callback_choose_lib, 'choose library')
        hbox = gtk.HBox(False, 5)
        hbox.pack_start(self.check_lib, True, True, 5)
        hbox.pack_start(self.entry_lib, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify SMPSs configuration file:')
        self.entry_cfg = gtk.Entry(max=0)
        self.entry_cfg.set_text('#enter configuration file here#')
        self.entry_cfg.set_width_chars(64)
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect(
            'clicked', self.callback_choose_cfg, 'choose configuration file')
        hbox = gtk.HBox(False, 5)
        hbox.pack_start(self.entry_cfg, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify port:')
        self.entry_port0 = gtk.Entry(max=0)
        if parent.connect_port is not None and parent.connect_port != 0:
            port = parent.connect_port
        else:
            try:
                port = os.environ['AYU_PORT']
            except:
                port = int(time.time() % 100)+7700
                logging.info('AYU_PORT is not defined, setting port to %d', port)

        self.entry_port0.set_text(str(port))
        frame.add(self.entry_port0)

        hbox = gtk.HBox(False, 5)
        hbox.pack_start(frame, True, True, 0)

        frame = gtk.Frame('Specify number of CPUs:')
        self.entry_cpus = gtk.Entry(max=0)
        try:
            cpus = client_conf.nthreads
        except:
            cpus = 2
            logging.info('CSS_NUM_CPUS is not defined, setting number of cpus to %d', cpus)
        self.entry_cpus.set_text(str(cpus))
        frame.add(self.entry_cpus)

        hbox.pack_start(frame, True, True, 0)
        vbox0.pack_start(hbox, True, True, 5)

        # notebook tab for remote connection
        vbox1 = gtk.VBox()
        hbox = gtk.HBox(False, 5)

        frame = gtk.Frame('Specify host:')
        self.entry_host = gtk.Entry(max=0)
        if parent.connect_host is not None and len(parent.connect_host) != 0:
            host = parent.connect_host
        else:
            host = 'localhost'
        self.entry_host.set_text(host)
        frame.add(self.entry_host)
        hbox.pack_start(frame, True, True, 0)

        frame = gtk.Frame('Specify port:')
        self.entry_port1 = gtk.Entry(max=0)
        self.entry_port1.set_text(str(port))
        frame.add(self.entry_port1)
        hbox.pack_start(frame, True, True, 0)

        vbox1.pack_start(hbox, False, True, 5)

        # fill notebook
        self.notebook = gtk.Notebook()
        self.notebook.append_page(vbox0, gtk.Label('local connection'))
        self.notebook.append_page(vbox1, gtk.Label('remote connection'))

        # # MPI frame
        # frame_MPI=gtk.Frame('MPI settings')
        # self.check_MPI=gtk.CheckButton('execute with mpiexec')
        # adjust = gtk.Adjustment(1, 1, 99, 1)
        # self.spin_MPI=gtk.SpinButton(adjust, 1, 2)
        # hbox=gtk.HBox()
        # hbox.pack_start(self.check_MPI, True, True)
        # hbox.pack_start(self.spin_MPI, False, False)
        # frame_MPI.add(hbox)

        # global layout
        self.vbox.pack_start(self.notebook, True, True, 5)
        # self.vbox.pack_start(frame_MPI, True, True, 5)

        # global adjustments
        self.set_border_width(10)
        self.show_all()

    def callback_check_lib(self, widget, callback_data):
        self.entry_lib.set_sensitive(self.check_lib.get_active())

    def callback_choose_exec(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Specify the executable',
                                           action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                           buttons=(gtk.STOCK_CANCEL,
                                                    gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,
                                                    gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_exec.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def callback_choose_lib(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Specify libayudame.so',
                                           action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                           buttons=(gtk.STOCK_CANCEL,
                                                    gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,
                                                    gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_lib.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def callback_choose_cfg(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(
            title='Specify the configuration file',
            action=gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons=(gtk.STOCK_CANCEL,
                     gtk.RESPONSE_CANCEL,
                     gtk.STOCK_OPEN,
                     gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_cfg.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def get_exec(self):
        response = self.entry_exec.get_text()
        if response == '' or response[0] == '#':
            return ''
        else:
            return response

    def get_exec_args(self):
        return self.entry_exec_args.get_text()

    def get_lib(self):
        response = self.entry_lib.get_text()
        if response == '' or response[0] == '#':
            return None
        else:
            return response

    def get_lib_check(self):
        response = self.check_lib.get_active()
        return response

    def get_cfg(self):
        response = self.entry_cfg.get_text()
        if response == '' or response[0] == '#':
            return None
        else:
            return response

    def get_port(self):
        if self.notebook.get_current_page() == 0:  # local
            response = self.entry_port0.get_text()
        else:  # remote
            response = self.entry_port1.get_text()
        if len(response) == 0:
            return 7795
        else:
            return int(response)

    def get_host(self):
        response = self.entry_host.get_text()
        if len(response) == 0 or response[0] == '#':
            return 'localhost'
        else:
            return response

    def get_cpus(self):
        response = self.entry_cpus.get_text()
        if len(response) == 0:
            return 2
        else:
            return int(response)

    def get_local_remote(self):
        return self.notebook.get_current_page()  # 0:local, 1:remote

#******

#****c* Temanejo/ExportDialog
# NAME
#   ExportDialog
#   - dialogue for choosing the export file format
# DESCRIPTION
#   With this Dialog one can choose the export file format and file name
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
#   callback_choose_file
#   callback_radio_format
#   get_filename
#   get_format
#   get framerate
#   get quality
# SOURCE


class ExportDialog(gtk.Dialog):
    def __init__(self, parent):
        gtk.Dialog.__init__(self, title='Export', parent=self,
                            flags=(gtk.DIALOG_MODAL |
                                   gtk.DIALOG_DESTROY_WITH_PARENT),
                            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK, gtk.RESPONSE_OK))

        self.jpg_quality = 90
        self.framerate = 4

        # image export
        frame_img = gtk.Frame('Image export format')
        frame_img.set_border_width(10)
        vbox = gtk.VBox()

        self.radio_png = gtk.RadioButton(
            None, 'png', use_underline=False)
        self.radio_jpeg = gtk.RadioButton(
            self.radio_png, 'jpeg', use_underline=False)
        self.radio_dot = gtk.RadioButton(
            self.radio_png, 'dot', use_underline=False)
        self.radio_png.connect(
            'clicked', self.callback_radio_format, 'png')
        self.radio_jpeg.connect(
            'clicked', self.callback_radio_format, 'jpeg')

        vbox.pack_start(self.radio_png)
        vbox.pack_start(self.radio_jpeg)
        frame_img.add(vbox)

        # text export
        frame_text = gtk.Frame('Text export format')
        frame_text.set_border_width(10)
        vbox = gtk.VBox()

        self.radio_nodes_edges = gtk.RadioButton(
            self.radio_png, '.nodes & .edges', use_underline=False)
        self.radio_dot = gtk.RadioButton(
            self.radio_png, 'dot', use_underline=False)
        self.radio_nodes_edges.connect(
            'clicked', self.callback_radio_format, 'text')
        self.radio_dot.connect(
            'clicked', self.callback_radio_format, 'dot')

        vbox.pack_start(self.radio_nodes_edges)
        vbox.pack_start(self.radio_dot)
        frame_text.add(vbox)

        # video export
        frame_video = gtk.Frame('Multiple image/video export format')
        frame_video.set_border_width(10)
        vbox = gtk.VBox()

        self.radio_png_ser = gtk.RadioButton(
            self.radio_png, 'png series', use_underline=False)
        self.radio_avi = gtk.RadioButton(
            self.radio_png, 'avi', use_underline=False)
        self.radio_gif = gtk.RadioButton(
            self.radio_png, 'gif', use_underline=False)
        self.radio_png_ser.connect(
            'clicked', self.callback_radio_format, 'png_ser')
        self.radio_avi.connect(
            'clicked', self.callback_radio_format, 'avi')
        self.radio_gif.connect(
            'clicked', self.callback_radio_format, 'gif')

        vbox.pack_start(self.radio_png_ser)
        vbox.pack_start(self.radio_avi)
        vbox.pack_start(self.radio_gif)
        frame_video.add(vbox)

        # settings
        frame_settings = gtk.Frame('Settings')
        frame_settings.set_border_width(10)
        hbox = gtk.HBox()
        label1 = gtk.Label('jpeg quality:')
        adjust = gtk.Adjustment(self.jpg_quality, 10, 100, 5)
        self.spin_quality = gtk.SpinButton(adjust, 0, 0)
        label2 = gtk.Label('animation frames/s:')
        adjust = gtk.Adjustment(self.framerate, 1, 30, 1)
        self.spin_framerate = gtk.SpinButton(adjust, 0, 0)

        hbox.pack_start(label1, False, False, 0)
        hbox.pack_start(self.spin_quality, True, False, 0)
        hbox.pack_start(label2, False, False, 0)
        hbox.pack_start(self.spin_framerate, True, False, 0)
        frame_settings.add(hbox)

        # file name
        frame_file = gtk.Frame('File name')
        frame_file.set_border_width(10)
        hbox = gtk.HBox(False, 5)
        self.entry_file = gtk.Entry()
        self.entry_file.set_size_request(500, -1)
        if parent.executable is None:
            base = 'temanejo'
        else:
            base = parent.executable
        self.entry_file.set_text(base+time.strftime(
            '_%Y%m%d-%H%M%S.png', time.localtime()))
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect('clicked', self.callback_choose_file, 'choose file')

        hbox.pack_start(self.entry_file, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame_file.add(hbox)

        # global layout
        hbox = gtk.HBox()
        hbox.pack_start(frame_text)
        hbox.pack_start(frame_img)
        hbox.pack_start(frame_video)
        self.vbox.pack_start(hbox, True, True, 0)
        self.vbox.pack_start(frame_settings, True, True, 0)
        self.vbox.pack_start(frame_file, True, True, 5)

        self.show_all()

        self.format = 'png'

    def callback_radio_format(self, widget, callback_data):
        self.format = callback_data
        if callback_data == 'text':
            ext = '[nodes|edges]'
        else:
            ext = self.format
        filename = self.entry_file.get_text()
        if callback_data == 'png_ser':
            filename = filename.rpartition('.')[0] + '_NNNNN.png'
        else:
            if filename.find('NNNNN') == -1:
                filename = filename.rpartition('.')[0] + '.' + ext
            else:
                filename = filename.replace('_NNNNN', '')
                filename = filename.rpartition('.')[0] + '.' + ext

        self.entry_file.set_text(filename)

    def callback_choose_file(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Choose a file',
                                           action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                           buttons=(gtk.STOCK_CANCEL,
                                                    gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,
                                                    gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_file.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def get_format(self):
        return self.format

    def get_filename(self):
        return self.entry_file.get_text()

    def get_framerate(self):
        return self.spin_framerate.get_value_as_int()

    def get_quality(self):
        return self.spin_quality.get_value_as_int()

#******


#****c* Temanejo/SettingsDialog
# NAME
#   SettingsDialog
#   - dialogue for setting the user options of Temanejo
# DESCRIPTION
#   This Dialog contains switches for adjusting the behaviour of Temanejo
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
#   get_check_autozoom
#   get_check_key
#   get_check_markers
#   get_edge_colours
#   get_edge_labels
#   get_global_tasks_first_row
#   get_graph_direction
#   get_graph_layout
#   get_spin_edge_width
#   get_spin_margin_width
#   get_spin_node_size
#   get_spin_nsteps
#   on_graph_layout
# SOURCE

class SettingsDialog(gtk.Dialog):
    def __init__(self, parent):
        gtk.Dialog.__init__(self, title='Settings', parent=self,
                            flags=gtk.DIALOG_MODAL |
                            gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK, gtk.RESPONSE_OK))

        vbox_ui_left = gtk.VBox()
        vbox_ui_right = gtk.VBox()
        vbox_ne = gtk.VBox()
        vbox_gen = gtk.VBox()
        vbox_tweaks = gtk.VBox()

        # General interface
        frame_general = gtk.Frame('Interface')
        #frame_general.set_tooltip_text('General interface settings')
        frame_general.add(vbox_gen)
        vbox_ui_right.pack_start(frame_general)

        # how many steps to run
        label = gtk.Label('Run n steps')
        label.set_alignment(0.0, 0.5)
        label.set_size_request(150, -1)
        adjust = gtk.Adjustment(parent.n_steps, 1, 1000, 1)
        self.spin_nsteps = gtk.SpinButton(adjust, 1, 0)
        hbox = gtk.HBox()
        hbox.pack_start(label, True, True)
        hbox.pack_start(self.spin_nsteps, False, False)
        #hbox.set_tooltip_text(
        #   'How many events to advance when running several steps')
        align = gtk.Alignment()
        align.add(hbox)
        align.set_padding(8, 8, 8, 8)
        vbox_gen.pack_start(align)

        # Whether to zoom automatically
        self.check_autozoom = gtk.CheckButton('Autozoom', False)
        self.check_autozoom.set_active(parent.widget.autozoom)
        #self.check_autozoom.set_tooltip_text(
        #    'Whether to zoom automatically when the graph changes.\nNote that'
        #    'the zoom buttons will have no effect when this is checked.')
        align = gtk.Alignment()
        align.add(self.check_autozoom)
        align.set_padding(8, 8, 8, 8)
        vbox_gen.pack_start(align)

        # show markers
        self.check_markers = gtk.CheckButton('Show markers', True)
        self.check_markers.set_active(parent.widget.graphview.show_markers)
        #self.check_markers.set_tooltip_text(
        #    'Whether to show blocked and priority markers.')
        align = gtk.Alignment()
        align.add(self.check_markers)
        align.set_padding(8, 8, 8, 8)
        vbox_gen.pack_start(align)

        # nodes and edges
        frame_nodes_edges = gtk.Frame('Node appearance')
        #frame_nodes_edges.set_tooltip_text(
        #    'How to display nodes and their margins\n')

        # Size of nodes
        label = gtk.Label('Size of nodes')
        label.set_alignment(0.0, 0.5)
        label.set_size_request(150, -1)
        adjust = gtk.Adjustment(parent.widget.graphview.node_size, 1, 99, 1)
        self.spin_node_size = gtk.SpinButton(adjust, 1, 0)
        hbox = gtk.HBox()
        hbox.pack_start(label, True, True)
        hbox.pack_start(self.spin_node_size, False, False)
        align = gtk.Alignment()
        align.add(hbox)
        align.set_padding(8, 8, 8, 8)
        vbox_ne.pack_start(align)

        # Thickness of coloured margins
        label = gtk.Label('Margin thickness')
        label.set_alignment(0.0, 0.5)
        label.set_size_request(150, -1)
        adjust = gtk.Adjustment(parent.widget.graphview.margin_width, 1, 99, 1)
        self.spin_margin_width = gtk.SpinButton(adjust, 1, 0)
        hbox = gtk.HBox()
        hbox.pack_start(label, True, True)
        hbox.pack_start(self.spin_margin_width, False, False)
        align = gtk.Alignment()
        align.add(hbox)
        align.set_padding(8, 8, 8, 8)
        vbox_ne.pack_start(align)

        # Thickness of edges
        label = gtk.Label('Edge thickness')
        label.set_alignment(0.0, 0.5)
        label.set_size_request(150, -1)
        adjust = gtk.Adjustment(parent.widget.graphview.edge_width, .1, 10, .1)
        self.spin_edge_width = gtk.SpinButton(adjust, .1, 1)
        hbox = gtk.HBox()
        hbox.pack_start(label, True, True)
        hbox.pack_start(self.spin_edge_width, False, False)
        align = gtk.Alignment()
        align.add(hbox)
        align.set_padding(8, 8, 8, 8)
        vbox_ne.pack_start(align)

        frame_nodes_edges.add(vbox_ne)
        vbox_ui_left.pack_start(frame_nodes_edges)

        # edge colours
        frame_edgecol = gtk.Frame('Edge colours')
        #frame_edgecol.set_tooltip_text('What the edge colours indicate')
        self.radio_ecorigaddr = gtk.RadioButton(
            None, 'Original dependency address', False)
        self.radio_ecnoth = gtk.RadioButton(
            self.radio_ecorigaddr, 'Nothing', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_ecorigaddr)
        self.radio_ecaddr = gtk.RadioButton(
            self.radio_ecorigaddr, 'Dependency address after renaming', False)
        self.radio_ecaddr.set_sensitive(client_conf.have_renaming)
        vbox.pack_start(self.radio_ecaddr)
        vbox.pack_start(self.radio_ecnoth)
        ec = parent.widget.graphview.addr_key

        if ec == 'addr':
            self.radio_ecaddr.set_active(True)
        if ec == 'orig_addr':
            self.radio_ecorigaddr.set_active(True)
        if ec == 'nothing':
            self.radio_ecnoth.set_active(True)

        frame_edgecol.add(vbox)
        vbox_ui_left.pack_start(frame_edgecol)

        # edge labels
        frame_edgelab = gtk.Frame('Edge labels')
        self.check_edgelab = gtk.CheckButton('draw edge labels', False)
        self.check_edgelab.set_active(parent.widget.graphview.draw_edge_labels)
        frame_edgelab.add(self.check_edgelab)
        vbox_ui_left.pack_start(frame_edgelab)

        # graph direction
        frame_graph_dir = gtk.Frame('Graph direction')
        #frame_graph_dir.set_tooltip_text('How to arrange the graph')

        self.radio_t2b = gtk.RadioButton(
            None, 'Top to bottom', False)
        self.radio_l2r = gtk.RadioButton(
            self.radio_t2b, 'Left to right', False)
        self.radio_b2t = gtk.RadioButton(
            self.radio_t2b, 'Bottom to top', False)
        self.radio_r2l = gtk.RadioButton(
            self.radio_t2b, 'Right to left', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_t2b)
        vbox.pack_start(self.radio_l2r)
        vbox.pack_start(self.radio_b2t)
        vbox.pack_start(self.radio_r2l)
        direction = parent.widget.graphview.layout_direction
        if direction == 't2b':
            self.radio_t2b.set_active(True)
        elif direction == 'l2r':
            self.radio_l2r.set_active(True)
        elif direction == 'b2t':
            self.radio_b2t.set_active(True)
        elif direction == 'r2l':
            self.radio_r2l.set_active(True)
        frame_graph_dir.add(vbox)
        vbox_ui_right.pack_start(frame_graph_dir)

        # global layout of user interface tab
        hbox = gtk.HBox()
        hbox.pack_start(vbox_ui_left, padding=5)
        hbox.pack_start(vbox_ui_right, padding=5)

        # tweaks tab

        # layout routine
        frame_layout = gtk.Frame('Graph layout')

        self.radio_temanejo = gtk.RadioButton(
            None, 'Use Temanejo\'s routine', False)
        self.radio_pygraphviz = gtk.RadioButton(
            self.radio_temanejo, 'Use pygraphviz (NetworkX)', False)
        if parent.pygraphviz_layout_present is False:
            self.radio_pygraphviz.set_sensitive(False)
        self.radio_linear = gtk.RadioButton(
            self.radio_temanejo,
            'Use extended linear visualisation (beta)',
            False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_temanejo)
        vbox.pack_start(self.radio_pygraphviz)
        vbox.pack_start(self.radio_linear)

        if parent.widget.graphview.use_layout == 'temanejo':
            self.radio_temanejo.set_active(True)
        elif parent.widget.graphview.use_layout == 'pygraphviz':
            self.radio_pygraphviz.set_active(True)
        elif parent.widget.graphview.use_layout == 'linear':
            self.radio_linear.set_active(True)
        frame_layout.add(vbox)

        vbox_tweaks.pack_start(frame_layout, expand=False, fill=False)

        self.radio_temanejo.connect(
            'clicked', self.on_graph_layout, 'temanejo')
        self.radio_pygraphviz.connect(
            'clicked', self.on_graph_layout, 'pygraphviz')
        self.radio_linear.connect(
            'clicked', self.on_graph_layout, 'linear')

        # global tasks
        frame_globaltasks = gtk.Frame('Global tasks')

        self.radio_globaltasks_everywhere = gtk.RadioButton(
            None, 'Put global tasks close to their dependencies', False)
        self.radio_globaltasks_firstrow = gtk.RadioButton(
            self.radio_globaltasks_everywhere,
            'Put global tasks in the first row',
            False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_globaltasks_everywhere)
        vbox.pack_start(self.radio_globaltasks_firstrow)

        if parent.widget.graphview.global_tasks_in_first_row is True:
            self.radio_globaltasks_firstrow.set_active(True)
        else:
            self.radio_globaltasks_everywhere.set_active(True)
        frame_globaltasks.add(vbox)

        if self.radio_pygraphviz.get_active():
            self.radio_globaltasks_everywhere.set_sensitive(False)
            self.radio_globaltasks_firstrow.set_sensitive(False)

        vbox_tweaks.pack_start(frame_globaltasks, expand=False, fill=False)

        # toolbar orientation
        frame_toolbarorientation = gtk.Frame('Toolbar orientation')

        self.radio_toolbar_horizontal = gtk.RadioButton(
            None, 'Draw toolbar horizontally', False)
        self.radio_toolbar_vertical = gtk.RadioButton(
            self.radio_toolbar_horizontal,
            'Draw toolbar vertically',
            False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_toolbar_horizontal)
        vbox.pack_start(self.radio_toolbar_vertical)

        if parent.toolbar_orientation == "horizontal":
            self.radio_toolbar_horizontal.set_active(True)
        else:
            self.radio_toolbar_vertical.set_active(True)
        frame_toolbarorientation.add(vbox)

        vbox_tweaks.pack_start(frame_toolbarorientation, expand=False, fill=False)

        # which external debugger
        frame_extdebugger = gtk.Frame('External debugger')

        self.radio_extdebugger_gdb = gtk.RadioButton(
            None, 'gdb', False)
        self.radio_extdebugger_ddt = gtk.RadioButton(
            self.radio_extdebugger_gdb,
            'ddt', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_extdebugger_gdb)
        vbox.pack_start(self.radio_extdebugger_ddt)

        if parent.ext_debugger == "gdb":
            self.radio_extdebugger_gdb.set_active(True)
        else:
            self.radio_extdebugger_ddt.set_active(True)
        frame_extdebugger.add(vbox)


        vbox_tweaks.pack_start(frame_extdebugger, expand=False, fill=False)


        # build notebook

        notebook = gtk.Notebook()
        notebook.append_page(hbox, gtk.Label('User Interface'))
        notebook.append_page(vbox_tweaks, gtk.Label('Tweaks'))

        #~ align=gtk.Alignment()
        #~ align.add(hbox)
        #~ align.set_padding(8, 8, 8, 8)
        #~ frame.add(align)
        self.vbox.pack_start(notebook, True, True, 0)
        self.show_all()

    def on_graph_layout(self, widget, data=None):
        if data == 'temanejo':
            self.radio_globaltasks_everywhere.set_sensitive(True)
            self.radio_globaltasks_firstrow.set_sensitive(True)
        else:
            self.radio_globaltasks_everywhere.set_sensitive(False)
            self.radio_globaltasks_firstrow.set_sensitive(False)

    def get_check_key(self):
        return self.check_key.get_active()

    def get_check_autozoom(self):
        return self.check_autozoom.get_active()

    def get_check_markers(self):
        return self.check_markers.get_active()

    def get_spin_margin_width(self):
        return self.spin_margin_width.get_value_as_int()

    def get_spin_edge_width(self):
        return self.spin_edge_width.get_value()

    def get_spin_node_size(self):
        return self.spin_node_size.get_value_as_int()

    def get_spin_nsteps(self):
        return self.spin_nsteps.get_value_as_int()

    def get_graph_direction(self):
        if self.radio_t2b.get_active():
            return 't2b'
        if self.radio_l2r.get_active():
            return 'l2r'
        if self.radio_b2t.get_active():
            return 'b2t'
        if self.radio_r2l.get_active():
            return 'r2l'

    def get_edge_colours(self):
        if self.radio_ecaddr.get_active():
            return 'addr'
        if self.radio_ecorigaddr.get_active():
            return 'orig_addr'
        if self.radio_ecnoth.get_active():
            return 'nothing'

    def get_edge_labels(self):
        return self.check_edgelab.get_active()

    def get_graph_layout(self):
        if self.radio_temanejo.get_active():
            return 'temanejo'
        elif self.radio_pygraphviz.get_active():
            return 'pygraphviz'
        elif self.radio_linear.get_active():
            return 'linear'

    def get_global_tasks_first_row(self):
        return self.radio_globaltasks_firstrow.get_active()

    def get_toolbar_orientation(self):
        if self.radio_toolbar_horizontal.get_active() is True:
            return "horizontal"
        else:
            return "vertical"

    def get_extdebugger(self):
        if self.radio_extdebugger_gdb.get_active() is True:
            return "gdb"
        else:
            return "ddt"

#******

#****c* Temanejo/AboutDialog
# NAME
#   AboutDialog
#   - dialogue displaying information about Temanejo
# DESCRIPTION
#   This Dialog displays information about Temanejo.
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
# SOURCE


class AboutDialog(gtk.Dialog):
    def __init__(self):
        gtk.Dialog.__init__(self, title='About Temanejo...', parent=self,
                            flags=gtk.DIALOG_MODAL |
                            gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        label = gtk.Label()
        frame = gtk.Frame()
        label.set_text('Temanejo version ' + __version__ + '\n'
                       '-------------------------------------------\n\n'
                       '(c) 2010-2015 HLRS, University of Stuttgart\n'
                       'published underr the terms of the BSD license.\n\n'
                       'authors: Steffen Brinkmann et al <temanejo@hlrs.de>\n\n'
                       'Please refer to the LICENSE file for detailed '
                       'information.')
        align = gtk.Alignment()
        frame.set_border_width(10)

        image = gtk.Image()
        image.set_from_file(os.path.join(
            temanejo_module_base_dir, 'resources/temanejo_icon.png'))

        hbox = gtk.HBox()
        hbox.pack_start(image, padding=5)
        hbox.pack_start(label, padding=5)
        align.add(hbox)
        align.set_padding(20, 20, 10, 10)
        frame.add(align)
        self.vbox.pack_start(frame, True, True, 0)
        self.show_all()
#******

#****c* Temanejo/HelpDialog
# NAME
#   HelpDialog
#   - dialogue displaying a short help screen about Temanejo
# DESCRIPTION
#   This Dialog displays some helping information about Temanejo.
#   For an extensive documentation refer to the files in Tools/doc/
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
# SOURCE

# class HelpDialog(gtk.Dialog):
#     def __init__(self):
#         gtk.Dialog.__init__(self, title='Help', parent=self,
#                             flags=gtk.DIALOG_MODAL |
#                                  gtk.DIALOG_DESTROY_WITH_PARENT,
#                             buttons= (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
#         table=gtk.Table(4, 2)
#         image=gtk.Image()
#         image.set_from_file(
#            os.path.join(temanejo_module_base_dir, 'task_NQ.png'))
#         label = gtk.Label('task which is not queued')
#         label.set_alignment(0.0, 0.5)
#         label.set_width_chars(25)
#         table.attach(image, 0, 1, 0, 1,xpadding=5,ypadding=3)
#         table.attach(label, 1, 2, 0, 1)

#         image=gtk.Image()
#         image.set_from_file(os.path.join(
#            temanejo_module_base_dir, 'task_Q.png'))
#         label = gtk.Label('task which is queued')
#         label.set_alignment(0.0, 0.5)
#         label.set_width_chars(25)
#         table.attach(image, 0, 1, 1, 2,xpadding=5,ypadding=3)
#         table.attach(label, 1, 2, 1, 2)

#         image=gtk.Image()
#         image.set_from_file(
#            os.path.join(temanejo_module_base_dir, 'task_R.png'))
#         label = gtk.Label('task which is running')
#         label.set_alignment(0.0, 0.5)
#         label.set_width_chars(25)
#         table.attach(image, 0, 1, 2, 3,xpadding=5,ypadding=3)
#         table.attach(label, 1, 2, 2, 3)

#         image=gtk.Image()
#         image.set_from_file(
#            os.path.join(temanejo_module_base_dir, 'task_F.png'))
#         label = gtk.Label('task which has finished')
#         label.set_alignment(0.0, 0.5)
#         label.set_width_chars(25)
#         table.attach(image, 0, 1, 3, 4,xpadding=5,ypadding=3)
#         table.attach(label, 1, 2, 3, 4)

#         align=gtk.Alignment()
#         align.set_padding(5, 5, 5, 5)
#         align.add(table)

#         frame=gtk.Frame('Graph key')
#         frame.set_border_width(10)
#         frame.add(align)

#         self.vbox.pack_start(frame, True, True, 0)
#         self.show_all()
#******

#****c* Temanejo/Communicator
# NAME
#   Communicator
#   - thread class for receiving messages from Ayudame
# DESCRIPTION
#   This class opens a thread for receiving messages from Ayudame. It
#   implements a socket client. Furthermore it processes the received
#   messages.
# DERIVED FROM
#   Thread
# METHODS
#   __init__
#   print_and_log
#   run
#   send_request
#   terminate
# SOURCE


class Communicator(Thread):
    def __init__(self, sock, par):
        Thread.__init__(self)
        self.stopevent = Event()
        self.working = False # is only true after an event has been received and is being processed
        self.sizeofdata = 8  # sizeof(uint64_t)
        self.block_size = 8 * self.sizeofdata
        self.buf_size = self.block_size  # * 100 # BUG: != 1 makes it unstable
        self.func_name_buf_size = 128
        self.s = sock
        self.parent = par
        self.event_count = 0
        self.log_dt = time.time()
        self.neg_thread_id_warning_displayed = False

        # send breakpoint requests
        self.parent.allow_requests = True
        if self.parent.check_anyEvent.get_active():
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_EVENT_NULL'), 1])
        else:
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_EVENT_NULL'), 0])

        if self.parent.check_preruntask.get_active():
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_PRERUNTASK'), 1])
        else:
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_PRERUNTASK'), 0])

        if self.parent.check_removetask.get_active():
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_REMOVETASK'), 1])
        else:
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_REMOVETASK'), 0])

        if self.parent.check_waiton.get_active():
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_WAITON'), 1])
        else:
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_WAITON'), 0])

        if self.parent.check_barrier.get_active():
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_BARRIER'), 1])
        else:
            self.send_request(
                0, 'AYU_PAUSEONEVENT', [event_t.index('AYU_BARRIER'), 0])
        self.parent.allow_requests = False

    def run(self):

        # this needs to go elsewhere!
        def transfer_counters(task_id, current, next, next_thread_id):
            '''This method update counters due to a change in task state.
            Several saveguards are implemented: current and next (task state)
            can be None, indicating that there is no such state. Also
            waiton tasks are recognized through their function ID and ignored.

            This method needs to be called before graph.set_status and graph.set_threadId,
            as it will read the current values from the graph.

            This method needs the variables graph (a temanejo_graph.Graph) and
            parent (a TemanejoWindow) to be defined.'''

            # in any way, just to make sure, the widget is declared invalid to
            # force refreshing it.
            #parent.widget.is_invalid = True


            # exit if wait_on task; not counted; actually, this should not happen at all
            _func_id = graph.get_funcId(taskid)
            if  _func_id == client_conf.waiton_func_id:
                return

            # get current thread_id
            _current_thread_id = graph.get_threadId(task_id)

            # increase counters for next state
            if not next==None:  # can't think of a case where this is true, but anyway
                parent.func_counter["total"][_func_id] += 1
                parent.func_counter[next][_func_id] += 1
                parent.status_counter[next] += 1
                if not next_state == "NQ": # thread counter not avail for "NQ"
                    parent.thread_counter[next][next_thread_id] += 1
                    parent.thread_counter["total"][next_thread_id] += 1

            # decrement counters for current state; ignore for new tasks (old==None)
            if not current==None:
                parent.func_counter["total"][_func_id] -= 1
                parent.func_counter[current][_func_id] -= 1
                parent.status_counter[current] -= 1
                if not current == "NQ": # counter not avail for "NQ"
                    parent.thread_counter[current][_current_thread_id] -= 1
                    parent.thread_counter["total"][_current_thread_id] -= 1

            # we are done
            return


        parent = self.parent
        graph = parent.graph
        while not self.stopevent.isSet():
            # receive messages
            self.working = False
            n = ''
            n += self.s.recv(self.buf_size)
            self.working = True
            # -------------------

            # process messages
            buf_count = 0
            while (len(n) >= self.block_size and
                    buf_count + self.block_size <= len(n)):

                # '>' converts from big-endian to local endianity
                buf = struct.unpack(
                    '>qqqqqqqq', n[buf_count:buf_count+self.block_size])
                if len(buf) == 0:
                    logging.critical('struct.unpack failed. Exiting.')
                    return
                buf_count += self.block_size

                logging.debug(buf)

                runtime_id = buf[0]  # runtime Id
                taskid = buf[1]  # task Id
                event = buf[2]  # event Id
                ts = buf[7]  # timestamp
                parent.last_event = event_t[event]
                parent.last_taskId = taskid

                if event == event_t.index('AYU_ADDTASK'):
                    funcId = buf[3]
                    crit = buf[4]
                    self.print_and_log(event_t[event], taskid, funcId)

                    ###########################################################
                    # TODO: this is for receiveing filename and filenumber
                    # from Ayudame. As of today, 2013-11-08, no runtime
                    # instrumentation supports that. Also, Temanejo does not
                    # display this information. This feature is not documented.
                    # This may not be commented because the next statement
                    # relies on the existence of srcfilename_buf and
                    # line_number.
                    len_filename = buf[5]
                    line_number = buf[6]
                    srcfilename_buf = ''
                    if len_filename != 0:
                        # read source file name
                        n += self.s.recv(len_filename)
                        fmt_string = str(len_filename)+'s'
                        srcfilename_buf = struct.unpack(
                            fmt_string, n[buf_count:buf_count+len_filename])[0]
                        # advance buffer counter
                        buf_count += len_filename
                    #
                    ###########################################################

                    graph.add_task(taskid,
                                   funcId,
                                   crit,
                                   parent.barrier_count,
                                   ts,
                                   srcfilename_buf,
                                   line_number)
                    if funcId not in propFunc.ids():
                        parent.register_function(
                            funcId, 'function %d' % funcId)
                    current_state = None
                    next_state = "NQ"
                    transfer_counters(taskid, current_state, next_state, next_thread_id=None)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_ADDDEPENDENCY'):
                    self.print_and_log(event_t[event], taskid, buf[3])
                    graph.add_dep(taskid, buf[3], buf[4], buf[5])

                elif event == event_t.index('AYU_ADDTASKTOQUEUE'):
                    thread_id = buf[3]
                    if thread_id < 0:
                        if self.neg_thread_id_warning_displayed == False:
                            msg = 'Thread id for task %d was %d.\n' % (taskid, thread_id) + \
                                  'Negative thread ids are treated as 0.\n' + \
                                  'This message is only displayed once.'
                            logging.warning(msg)
                            message_dialog_async('WARNING',msg)
                            self.neg_thread_id_warning_displayed = True
                        thread_id = 0
                    self.print_and_log(event_t[event], taskid, thread_id)
                    current_state = parent.task_states[graph.get_status(taskid)]
                    next_state = "Q"
                    transfer_counters(taskid, current_state, next_state, thread_id)
                    graph.set_threadId(taskid, thread_id)
                    graph.set_status(taskid, parent.task_states.index(next_state), ts)  # queued
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_PRESELECTTASK'):
                    pass
#                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_PRERUNTASK'):
                    thread_id = buf[3]
                    if thread_id < 0:
                        if self.neg_thread_id_warning_displayed == False:
                            msg = 'Thread id for task %d was %d.\n' % (taskid, thread_id) + \
                                  'Negative thread ids are treated as 0.\n' + \
                                  'This message is only displayed once.'
                            logging.warning(msg)
                            message_dialog_async('WARNING',msg)
                            self.neg_thread_id_warning_displayed = True
                        thread_id = 0
                    self.print_and_log(event_t[event], taskid, thread_id)
                    current_state = parent.task_states[graph.get_status(taskid)]
                    next_state = "R"
                    transfer_counters(taskid, current_state, next_state, thread_id)
                    graph.set_status(taskid, parent.task_states.index(next_state), ts)  # running
                    graph.set_threadId(taskid, thread_id)
                    parent.last_check_event = event_t[event]
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_RUNTASK'):
                    current_state = parent.task_states[graph.get_status(taskid)]
                    next_state = "R"
                    transfer_counters(taskid, current_state, next_state, next_thread_id=graph.get_threadId(taskid))
                    graph.set_status(taskid, parent.task_states.index(next_state), ts)  # running
                    parent.update_ui('all')

                elif event == event_t.index('AYU_POSTRUNTASK'):
                    self.print_and_log(event_t[event], taskid)
                    current_state = parent.task_states[graph.get_status(taskid)]
                    next_state = "F"
                    transfer_counters(taskid, current_state, next_state, next_thread_id=graph.get_threadId(taskid))
                    graph.set_status(taskid, parent.task_states.index(next_state), ts)  # finished
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_REMOVETASK'):
                    self.print_and_log(event_t[event], taskid)
                    current_state = parent.task_states[graph.get_status(taskid)]
                    next_state = "F"
                    transfer_counters(taskid, current_state, next_state, next_thread_id=graph.get_threadId(taskid))
                    graph.set_status(taskid, parent.task_states.index(next_state), ts)  # finished
                    if graph.get_funcId(taskid) != client_conf.waiton_func_id:  # TODO: need this if?
                        if graph.get_nodeId(taskid) in graph.dependentNodes:
                            for t in graph.dependentNodes[graph.get_nodeId(taskid)]:
                                parent.comm.send_request(
                                    0, 'AYU_BLOCKTASK', [graph.get_taskId(t), 0])
                                if t in graph.blockedNodes:
                                    graph.blockedNodes.remove(t)
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_RUNTASKFAILED'):
                    self.print_and_log(event_t[event], taskid)
                    current_state = parent.task_states[graph.get_status(taskid)]
                    next_state = "F"  #     next_state = "X"
                    transfer_counters(taskid, current_state, next_state, next_thread_id=graph.get_threadId(taskid))
                    graph.set_status(taskid, parent.task_states.index(next_state), ts)  # finished
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_WAITON'):
                    self.print_and_log(event_t[event], taskid)
                    graph.add_task(taskid, client_conf.waiton_func_id, 0, parent.barrier_count, ts)
                    parent.last_check_event = event_t[event]
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_BARRIER'):
                    self.print_and_log(event_t[event], taskid)
                    parent.barrier_count += 1
                    graph.total_barrier_count += 1
                    parent.last_check_event = event_t[event]
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_INIT'):
                    nthreads = buf[3]
                    self.print_and_log(event_t[event], taskid, nthreads)
                    for i in range(client_conf.nthreads+1, nthreads+1):
                        parent.thread_counter["total"][i] = 0
                        parent.thread_counter["Q"][i] = 0
                        parent.thread_counter["R"][i] = 0
                        parent.thread_counter["F"][i] = 0
                    client_conf.nthreads = nthreads
                    parent.allow_requests = True
                    parent.update_ui('all')

                elif event == event_t.index('AYU_PREINIT'):
                    if 0 != buf[1]: 
                        client_conf.remote_pid = buf[1]
                    else: 
                        client_conf.remote_pid = None
                    self.print_and_log(event_t[event], taskid)
                    client_conf.init(runtime_id)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_REGISTERFUNCTION'):
                    len_funcname = buf[3]
                    func_id = buf[4]
                    # read function name
                    n += self.s.recv(len_funcname)
                    fmt_string = str(len_funcname)+'s'
                    funcname_buf = struct.unpack(
                        fmt_string, n[buf_count:buf_count+len_funcname])[0]
                    # print and log
                    self.print_and_log(
                        event_t[event], taskid, func_id, funcname_buf)
                    # advance buffer counter
                    buf_count += len_funcname

                    # register function
                    #print "event AYU_REGISTERFUNCTION: ", func_id, funcname_buf
                    parent.register_function(func_id, funcname_buf)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_ADDHIDDENTASK'):
                    self.print_and_log(event_t[event], taskid)
                    graph.add_task(taskid, client_conf.waiton_func_id, 0, parent.barrier_count, ts)
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_ADDWAITONTASK'):
                    self.print_and_log(event_t[event], taskid)
                    graph.add_task(taskid, client_conf.waiton_func_id, 0, parent.barrier_count, ts)
                    parent.last_check_event = event_t[event]
                    parent.widget.window.invalidate_rect(parent.widget.get_allocation(), False)
                    parent.update_ui('all')

                elif event == event_t.index('AYU_FINISH'):
                    self.print_and_log(event_t[event], taskid)
                    parent.allow_requests = False
                    self.working = False
                    time.sleep(.1)
                    parent.update_ui('all')

                    msg = ''
                    try:
                        #self.s.shutdown(socket.SHUT_RDWR)
                        self.s.close()
                    except (socket.error, msg):
                        logging.error('closing connection failed: %s', msg)
                    return

                else:
                    logging.error('unknown event (%d) received for taskId: %d',
                                  event, taskid)
                    return
            # -----------------------------------------------

    def print_and_log(self, event, taskid, a=None, b=None):

        if event == 'AYU_ADDDEPENDENCY':
            t = '\n%s,\t taskId %i,\t depId: %i' % (event, taskid, a)
            logging.debug('%s received for taskId: %d , depId: %d',
                          event, taskid, a)

        elif (event == 'AYU_PRERUNTASK' or
                event == 'AYU_ADDTASKTOQUEUE'):
            t = '\n%s,\t taskId %i,\t threadId: %i' % (event, taskid, a)
            logging.debug('%s received for taskId: %d , threadId: %d',
                          event, taskid, a)

        elif event == 'AYU_ADDTASK':
            t = '\n%s,\t taskId %i,\t funcId: %i' % (event, taskid, a)
            logging.debug('%s received for taskId: %d , funcId: %d',
                          event, taskid, a)

        elif event == 'AYU_REMOVETASK':
            t = '\n%s,\t taskId %i' % (event, taskid)
            logging.debug('%s received for taskId: %d', event, taskid)

        elif event == 'AYU_REGISTERFUNCTION':
            t = '\n%s,\t taskId %i,\t funcId %i, function name: %s' %\
                (event, taskid, a, b)
            logging.debug(
                '%s received for taskId: %d , funcId %d , function name: %s',
                event, taskid, a, b)

        elif event == 'AYU_ADDWAITONTASK':
            t = '\n%s,\t taskId %i' % (event, taskid)
            logging.warning('%s received for taskId: %d\n'
                            'The event AYU_ADDWAITONTASK is deprecated. '
                            'Please use AYU_WAITON', event, taskid)

        elif event == 'AYU_ADDHIDDENTASK':
            t = '\n%s,\t taskId %i' % (event, taskid)
            logging.warning('%s received for taskId: %d\n'
                            'The event AYU_ADDWAITONTASK is deprecated.\n'
                            'If you think that you need it please contact the'
                            'developers.', event, taskid)

        elif event == 'AYU_FINISH':
            t = '\n%s,\t taskId %i\n### Exiting ###' % (event, taskid)
            logging.debug('%s received for taskId: %d', event, taskid)

        elif event == 'AYU_INIT':
            t = '\n%s,\t nthreads %i\n' % (event, a)
            logging.debug('%s received for taskId: %d', event, taskid)

        else:
            t = '\n%s,\t taskId: %i' % (event, taskid)
            logging.debug('%s received for taskId: %d', event, taskid)

        self.parent.prelog += t

#        self.parent.update_ui('statusbar')
#        self.parent.update_ui('log')

    def send_request(self, t_id, request, data=None, ignore_viewer_mode=False):
        if self.parent.viewer_mode == True and ignore_viewer_mode == False:
            logging.info('Viewer mode enabled: %s not sent.',
                            request)
            return

        if self.parent.host_allows_requests is False:
            logging.warning('Host doesn\'t allow requests: %s not sent.',
                            request)
            return
        if self.parent.allow_requests is False:
            logging.warning('Application has not started yet or already '
                            'finished. request: %s not sent',
                            request)
            return
        d = [0, 0, 0, 0, 0, 0]
        if data is not None:
            if isinstance(data, list):
                for i in range(len(data)):
                    d[i] = data[i]
                    logging.debug('about to sent: request: %s , data[0]: %d',
                                  request, data[0])
            else:
                d[0] = data
                logging.debug('about to sent: request: %s , data: %d',
                              request, data)
        buf = struct.pack('>qqqqqqqq', t_id, request_t.index(request),
                          d[0], d[1], d[2], d[3], d[4], d[5])

        msg = ''
        try:
            self.s.send(buf)
            logging.debug('sent request: %s', request)
        except (socket.error, msg):
            logging.error('sending request %s to host failed: %s',
                          request, msg)

    def terminate(self):
        self.stopevent.set()

#******

#****f* Temanejo/Temanejo_main
# NAME
#   Temanejo_main
#   - the main application class
# DESCRIPTION
#   This class is the main application class. It implements the main function
#   and a parser for command line options. The main function creates a
#   TemanejoWindow and runs gtk.main()
# METHODS
#   init_optparser
#   main
# SOURCE


class Temanejo_main:

    def main(self):

        sys.setcheckinterval(1000)

        # initiaise  option parser
        parser = self.init_optparser()

        # stop prsing when first unknown option occurs
        # this will probably be the application and everythin after it
        # is an application argument
        parser.disable_interspersed_args()

        # intitiate window
        win = TemanejoWindow()
        win.connect('destroy', win.on_quit)
        win.connect('delete_event', win.on_quit)

        # process options
        (options, args) = parser.parse_args(sys.argv[1:])

        win.set_title(__progname__+' '+options.title)
        win.widget.graphview.layout_direction = options.layoutdir
        win.widget.autozoom = options.autozoom
        win.widget.graphview.addr_key = options.edge_col
        win.widget.graphview.node_colours = options.node_col
        win.widget.graphview.margins_mean = options.margin_col
        win.widget.graphview.shapes_mean = options.shapes
        win.widget.graphview.global_tasks_in_first_row = options.global_tasks

        win.auto_connect = options.auto_connect

        if options.host_port != '':
            if options.host_port.find(':') != -1:
                win.connect_host = options.host_port.split(':')[0]
                if options.host_port.split(':')[1] != '':
                    win.connect_port = int(options.host_port.split(':')[1])
                else:
                    win.connect_port = 0
                win.auto_connect = True
            else:
                logging.error('after option -c, a string containing a colon, '
                              'which separates the host (even if left blank)\n'
                              'from the port, must be specified.')
                exit(1)

        if win.connect_host == '':
            win.connect_host = 'localhost'
        if win.connect_port == 0:
            env_dict = os.environ
            if 'AYU_PORT' in env_dict:
                win.connect_port = int(env_dict['AYU_PORT'])

        win.max_retries = options.max_retries
        win.retry_interval = options.retry_interval

        if len(options.lib_file):
            win.library = options.lib_file

        if options.wide == True:
            win.set_toolbar_orientation('vertical')

        if options.viewer_mode == True:
            win.set_viewer_mode(True)

        # by default only errors and critical is displayed
        logging.basicConfig(
            format=__progname__ + '-%(levelname)s: %(message)s',
            level=logging.ERROR)

        if options.verbose == 1:
            logging.getLogger().setLevel(logging.WARNING)
        if options.verbose == 2:
            logging.getLogger().setLevel(logging.INFO)
        if options.verbose > 2:
            logging.getLogger().setLevel(logging.DEBUG)

        if options.quiet:
            # only display critical errors
            logging.basicConfig(level=logging.CRITICAL)

        # check if pygraphviz is present
        import imp
        try:
            imp.find_module('pygraphviz')
            win.pygraphviz_layout_present = True
            if options.pygraphviz is True:
                win.widget.graphview.use_layout = 'pygraphviz'
        except:
            pass

        # run program
        logging.info('Start session, %s', time.ctime())

        if len(args) >= 1:
            win.executable = args[0]
        if len(args) > 1:
            win.executable_args = args[1:]
        for i in range(len(win.executable_args)):
            if win.executable_args[i].find(' ') != -1:
                win.executable_args[i] = '"'+win.executable_args[i]+'"'

        win.on_connect(None)

        gtk.gdk.threads_init()
        gtk.gdk.threads_enter()
        gtk.main()
        gtk.gdk.threads_leave()
        logging.shutdown()

    def init_optparser(self):

        parser = optparse.OptionParser(
            usage='%prog [options] [app [app_args]]',
            description='Temanejo connects to a task-parallel application '
            'which has been instrumented with calls to the Ayudame library. '
            'Supported runtimes include SMPSs, OmpSs, StarPU, and CppSs. '
            'In the simplest case, Temanejo will start the parallel '
            'application "app" and pass it its arguments "app_args". '
            'Temanejo will interpret the first argument it does not support '
            'as the application name. It is '
            'also possible to set up remote connections. Please refer to the '
            'README file and the documentation in the doc/ directory.',
            epilog='Temanejo is published under the terms of the BSD license. '
            '(C) HLRS, University of Stuttgart. For further information '
            'please refer to the '
            'LICENSE file.',
            version='%%prog %s' % __version__)

        parser.add_option(
            '-v', '--verbose',
            action='count', dest='verbose', default=0,
            help='Switch on verbose output, which prints warnings. Use -vv '
            '(info) and -vvv (debug) for increasingly epic information.')

        parser.add_option(
            '-q', '--quiet',
            action='store_true', dest='quiet', default=False,
            help='Switch off any output except critical error messages. This '
            'overwrites -v, -vv and -vvv.')

        group_conn = optparse.OptionGroup(parser, "Connection Options")

        group_conn.add_option(
            '-c', '--connect',
            action='store', dest='host_port', default='', metavar='HOST:PORT',
            help='Specify parameters for remote connection of form '
            '"host:port". Using this option will cause Temanejo to omit the '
            'initial connect dialogue. The option argument is required. One '
            'of the arguments (host or port) can be omitted. So if you want '
            'to connect to localhost at port 111, type "-c :111". If you want '
            'to connect to localhost at port $AYU_PORT type "-c :" or use the '
            '"-C" option.')

        group_conn.add_option(
            '-C', '--auto-connect',
            action='store_true', dest='auto_connect', default=False,
            help='This option will cause Temanejo to omit the initial connect '
            'dialogue and connect directly to localhost at port $AYU_PORT.')

        group_conn.add_option(
            '-l', '--library',
            action='store', dest='lib_file', default='',
            help='Specify library file to communicate with the application. '
            'By default, libayudame.so is expected in temanejo_dir/lib/.')

        group_conn.add_option(
            '', '--connection-retry',
            action='store', type='int', dest='max_retries', default=5,
            help='Specify many times Temanejo should retry to connect to the '
            'socket. [default=5]')

        group_conn.add_option(
            '', '--retry-interval',
            action='store', type='float', dest='retry_interval', default=1.0,
            help='Specify how many seconds Temanejo should wait between each '
            'connection attempt. Accepts floating point numbers.  [default=1.0]')

        group_conn.add_option(
            '', '--viewer',
            action='store_true', dest='viewer_mode', default=False,
            help='Enable viewer mode. Temanejo will not issue any requests when'
            ' enabled. Also no breakpoints will be set and buttons which control'
            ' the application are disabled. [default=False]')

        group_disp = optparse.OptionGroup(parser, "Display Options")

        group_disp.add_option('-t', '--title', dest='title', default='',
                          help='Window title (appended to "Temanejo:").')

        group_disp.add_option(
            '-d', '--layout-direction',
            type='choice', choices=('t2b', 'l2r', 'b2t', 'r2l'),
            dest='layoutdir', default='t2b',
            help='How to draw the graph: top to bottom (\'t2b\'), left to '
            'right (\'l2r\'), Bottom to top (\'b2t\') or right to left '
            '(\'r2l\') [default: %default].')

        group_disp.add_option(
            '-Z', '--autozoom',
            action='store_true', dest='autozoom', default=True,
            help='Switch on autozooming of the graph.')

        group_disp.add_option(
            '-z', '--no-autozoom',
            action='store_false', dest='autozoom', default=True,
            help='Switch off autozooming of the graph [default].')

        group_disp.add_option(
            '-n', '--node-colour',
            type='choice', choices=('nothing', 'status', 'thread', 'function',
                                    'task_duration'),
            dest='node_col', default='function',
            help='Choose what the node colour means. One of "nothing" (no '
            'colours drawn), "status" (colour indicates task status), '
            '"thread" (thread Id), "function" [default] (function Id), '
            '"task_dur" (task duration).')

        group_disp.add_option(
            '-m', '--margin-colour',
            type='choice', choices=('nothing', 'status', 'thread', 'function',
                                    'task_duration'),
            dest='margin_col', default='status',
            help='Choose what the margin colour means. One of "nothing" (no '
            'margins drawn), "status" [default] (margin indicates task '
            'status), "thread" (thread Id), "function" (function Id), '
            '"task_dur" (task duration).')

        group_disp.add_option(
            '-e', '--edge-colour',
            type='choice', choices=('nothing', 'addr', 'orig_addr'),
            dest='edge_col', default='orig_addr',
            help='Choose what the edge colour means. One of "nothing" (black '
            'edges), "orig_addr" [default] (original address of dependency), '
            '"addr" (address of dependency after renaming).')

        group_disp.add_option(
            '-s', '--shapes',
            type='choice', choices=('nothing', 'thread', 'function'),
            dest='shapes', default='thread',
            help='Choose what the node shape means. One of "nothing" (no '
            'margins drawn), "thread" [default] (thread Id), "function" '
            '(function Id).')

        group_disp.add_option(
            '', '--pygraphviz',
            action='store_true', dest='pygraphviz', default=False,
            help='Specify whether to use pygraphviz for the graph layout. '
            'Pygraphviz is part of the networkx package, but may cause '
            'problems on some systems. Barriers are not going to be displayed.')

        group_disp.add_option(
            '-w', '--wide',
            action='store_true', dest='wide', default=False,
            help='Optimise the graphical user interface for wide screens.'
                'Presently, this option will only set the toolbar orientation'
                'to vertical.')

        group_disp.add_option(
            '', '--global-tasks-in-first-row',
            action='store_true', dest='global_tasks', default=False,
            help='Specify whether global tasks should be drawn in first line.')

        parser.add_option_group(group_conn)
        parser.add_option_group(group_disp)

        return parser


if __name__ == '__main__':

    temanejo_app = Temanejo_main()
    #cProfile.run('temanejo_app.main()', 'profile.out', 'time')
    temanejo_app.main()

#******
