#****h* Temanejo/temanejo_graphWidget
# NAME
#   temanejo_graphWidget.py
#   - widget for displaying and interacting wtih task dependency graph
# DESCRIPTION
#   temanejo_graphWidget.py is intented to be imported from Temanejo and
#   implements a widget derived from gtk.DrawingArea which can display and
#   interact with a graph.
#   For that purpose this module implements several classes which represent
#   zoom, animation and drag events.
# PORTABILITY
#   The following standard python modules are imported:
#   * cairo
#   * colorsys
#   * gtk
#   * gobject
#   * logging
#   * math
#   * networkx
#   * time
#   The following local modules are imported:
#   * statusbars
#   * temanejo_client_config as client_conf
# AUTHOR
#    Steffen Brinkmann et al, HLRS <temanejo@hlrs.de>
# COPYRIGHT
#    (C) 2013-2015 HLRS, University of Stuttgart
#    temanejo_graphWidget.py is published under the terms of the BSD license.
#    temanejo_graphWidget.py is based on xdot.py, ver.0.4 (C) 2008 Jose Fonseca
#    which is published under the terms of the GNU Lesser General Public
#    License
#******

from __future__ import division
from compatibility import old_div


DEBUG = True

import cairo
import colorsys
import gtk
import gobject
import logging
import math
import networkx
import time

import statusbars   # for globally accessible statusbars
import temanejo_client_config as client_conf # for client dependent configuration
import properties
from hashes import hashto10bits

#****c* temanejo_graphWidget/Animation
# NAME
#   Animation
#   - Base class for animation events
# DESCRIPTION
#   This is the base class for moving and zooming events.
# DERIVED FROM
#   object
# DERIVED BY
#   NoAnimation
#   LinearAnimation
# METHODS
#   __init__
#   start
#   stop
#   tick
# SOURCE


class Animation(object):

    step = 0.03  # seconds

    def __init__(self, graph_widget):
        self.graph_widget = graph_widget
        self.timeout_id = None

    def start(self):
        self.timeout_id = gobject.timeout_add(int(self.step * 1000), self.tick)

    def stop(self):
        self.graph_widget.animation = NoAnimation(self.graph_widget)
        if self.timeout_id is not None:
            gobject.source_remove(self.timeout_id)
            self.timeout_id = None

    def tick(self):
        self.stop()

#******

#****c* temanejo_graphWidget/NoAnimation
# NAME
#   NoAnimation
#   - Class for an animation which is not happening
# DESCRIPTION
#   This class implements a "no-"animation. It will pass on start() and stop().
# DERIVED FROM
#   Animation
# METHODS
#   start
#   stop
# SOURCE


class NoAnimation(Animation):

    def start(self):
        pass

    def stop(self):
        pass

#******

#****c* temanejo_graphWidget/LinearAnimation
# NAME
#   LinearAnimation
#   - Class for a linear animation. Base class for a move event
# DESCRIPTION
#   This class implements a "no-"animation. It will pass on start() and stop()
# DERIVED FROM
#   Animation
# DERIVED BY
#   MoveToAnimation
# METHODS
#   animate
#   start
#   tick
# SOURCE


class LinearAnimation(Animation):

    duration = 0.6

    def start(self):
        self.started = time.time()
        Animation.start(self)

    def tick(self):
        t = (time.time() - self.started)/self.duration
        self.animate(max(0, min(t, 1)))
        return (t < 1)

    def animate(self, t):
        pass

#******

#****c* temanejo_graphWidget/MoveToAnimation
# NAME
#   MoveToAnimation
#   - Class for a move event. Base class for a zoom event
# DESCRIPTION
#   This class implenents a move event.
# DERIVED FROM
#   LinearAnimation
# DERIVED BY
#   ZoomToAnimation
# METHODS
#   __init__
#   animate
# SOURCE


class MoveToAnimation(LinearAnimation):

    def __init__(self, graph_widget, target_x, target_y):
        Animation.__init__(self, graph_widget)
        self.source_x = graph_widget.x
        self.source_y = graph_widget.y
        self.target_x = target_x
        self.target_y = target_y

    def animate(self, t):
        sx, sy = self.source_x, self.source_y
        tx, ty = self.target_x, self.target_y
        self.graph_widget.x = tx * t + sx * (1-t)
        self.graph_widget.y = ty * t + sy * (1-t)
        if self.graph_widget.x == tx:
            Animation.stop(self)
        self.graph_widget.queue_draw()


#******

#****c* temanejo_graphWidget/ZoomToAnimation
# NAME
#   ZoomToAnimation
#   - Class for a zoom event.
# DESCRIPTION
#   This class implenents a zoom event.
# DERIVED FROM
#   MoveToAnimation
# METHODS
#   __init__
#   animate
# SOURCE


class ZoomToAnimation(MoveToAnimation):

    def __init__(self, graph_widget, target_x, target_y):
        MoveToAnimation.__init__(self, graph_widget, target_x, target_y)
        self.source_zoom = graph_widget.zoom_ratio
        self.target_zoom = self.source_zoom
        self.extra_zoom = 0

        middle_zoom = 0.5 * (self.source_zoom + self.target_zoom)

        distance = math.hypot(self.source_x - self.target_x,
                              self.source_y - self.target_y)
        rect = self.graph_widget.get_allocation()
        visible = old_div(min(rect.width, rect.height), self.graph_widget.zoom_ratio, expect_float=True)
        visible *= 0.9
        if distance > 0:
            desired_middle_zoom = visible / distance
            self.extra_zoom = min(0, 4 * (desired_middle_zoom - middle_zoom))

    def animate(self, t):
        a, b, c = self.source_zoom, self.extra_zoom, self.target_zoom
        self.graph_widget.zoom_ratio = c*t + b*t*(1-t) + a*(1-t)
        self.graph_widget.zoom_to_fit_on_resize = False
        MoveToAnimation.animate(self, t)

#******

#****c* temanejo_graphWidget/DragAction
# NAME
#   DragAction
#   - Base class for dragging events.
# DESCRIPTION
#   This class is the base class for dragging events.
# DERIVED FROM
#   object
# DERIVED BY
#   NullAction
#   PanAction
#   ZoomAction
#   ZoomAreaAction
# METHODS
#   __init__
#   abort
#   drag
#   draw
#   on_button_press
#   on_button_release
#   on_motion_notify
#   start
#   stop
# SOURCE


class DragAction(object):

    def __init__(self, graph_widget):
        self.graph_widget = graph_widget

    def on_button_press(self, event):
        self.startmousex = self.prevmousex = event.x
        self.startmousey = self.prevmousey = event.y
        self.start()

    def on_motion_notify(self, event):
        if event.is_hint:
            x, y, state = event.window.get_pointer()
        else:
            x, y, state = event.x, event.y, event.state
        deltax = self.prevmousex - x
        deltay = self.prevmousey - y
        self.drag(deltax, deltay)
        self.prevmousex = x
        self.prevmousey = y

    def on_button_release(self, event):
        self.stopmousex = event.x
        self.stopmousey = event.y
        self.stop()

    def draw(self, cr):
        pass

    def start(self):
        pass

    def drag(self, deltax, deltay):
        pass

    def stop(self):
        pass

    def abort(self):
        pass


#******

#****c* temanejo_graphWidget/NullAction
# NAME
#   NullAction
#   - implements a dragging event which is not happening
# DESCRIPTION
#   This class represents a "no-drag" event.
# DERIVED FROM
#   DragAction
# METHODS
#   on_motion_notify
# SOURCE


class NullAction(DragAction):
    def on_motion_notify(self, event):
        pass

#******

#****c* temanejo_graphWidget/PanAction
# NAME
#   PanAction
#   - implements a pnning event
# DESCRIPTION
#   This class represents a panning event when dragging the graph with the
#   mouse.
# DERIVED FROM
#   DragAction
# METHODS
#   drag
#   start
#   stop
# SOURCE


class PanAction(DragAction):

    def start(self):
        self.graph_widget.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))

    def drag(self, deltax, deltay):
        self.graph_widget.x += old_div(deltax, self.graph_widget.zoom_ratio, expect_float=True)
        self.graph_widget.y += old_div(deltay, self.graph_widget.zoom_ratio, expect_float=True)
        self.graph_widget.queue_draw()

    def stop(self):
        self.graph_widget.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.ARROW))

    abort = stop


#******

#****c* temanejo_graphWidget/ZoomAction
# NAME
#   ZoomAction
#   - implements a zooming event
# DESCRIPTION
#   This class represents a zooming event in and out of tha graph.
# DERIVED FROM
#   DragAction
# METHODS
#   drag
#   stop
# SOURCE


class ZoomAction(DragAction):

    def drag(self, deltax, deltay):
        self.graph_widget.zoom_ratio *= 1.005 ** (deltax + deltay)
        self.graph_widget.zoom_to_fit_on_resize = False
        self.graph_widget.queue_draw()

    def stop(self):
        self.graph_widget.queue_draw()

#******

#****c* temanejo_graphWidget/ZoomAreaAction
# NAME
#   ZoomAreaAction
#   - implements a zooming event into an area
# DESCRIPTION
#   This class represents a zooming event. The widget will zoom into a
#   rectangle marked with the mouse pointer.
# DERIVED FROM
#   DragAction
# METHODS
#   abort
#   drag
#   draw
#   stop
# SOURCE


class ZoomAreaAction(DragAction):

    def drag(self, deltax, deltay):
        self.graph_widget.queue_draw()

    def draw(self, cr):
        cr.save()
        cr.set_source_rgba(.5, .5, 1.0, 0.25)
        cr.rectangle(self.startmousex, self.startmousey,
                     self.prevmousex - self.startmousex,
                     self.prevmousey - self.startmousey)
        cr.fill()
        cr.set_source_rgba(.5, .5, 1.0, 1.0)
        cr.set_line_width(1)
        cr.rectangle(self.startmousex - .5, self.startmousey - .5,
                     self.prevmousex - self.startmousex + 1,
                     self.prevmousey - self.startmousey + 1)
        cr.stroke()
        cr.restore()

    def stop(self):
        x1, y1 = self.graph_widget.window2graph(self.startmousex,
                                                self.startmousey)
        x2, y2 = self.graph_widget.window2graph(self.stopmousex,
                                                self.stopmousey)
        self.graph_widget.zoom_to_area(x1, y1, x2, y2)

    def abort(self):
        self.graph_widget.queue_draw()


#******

#****c* temanejo_graphWidget/Graph_Widget
# NAME
#   Graph_Widget
#   - a widget for displaying a *Ss dependenca graph
# DESCRIPTION
#   This class implements a widget which allows to display and interact with
#   a *Ss dependency graph
# DERIVED FROM
#   gtk.DrawingArea
# METHODS
#   __init__
#   animate_to
#   do_expose_event
#   __get_current_pos # not used
#   __get_drag_action
#   __get_jump # not used
#   __is_click
#   __limit_zoomratio
#   on_area_button_press
#   on_area_button_release
#   on_area_motion_notify
#   on_area_scroll_event
#   on_area_size_allocate
#   on_context_menu_activate
#   on_zoom_100
#   on_zoom_fit
#   on_zoom_in
#   on_zoom_out
#   set_current_pos # not used
#   window2graph
#   __zoom_image
#   zoom_to_area
#   __zoom_to_fit
# ATTRIBUTES
#   POS_INCREMENT
#   ZOOM_INCREMENT
#   ZOOM_TO_FIT_MARGIN
#   __gsignals__
# SOURCE


class GraphWidget(gtk.DrawingArea):
    """PyGTK widget that draws graphs."""

    __gsignals__ = {
        'expose-event': 'override',
        'clicked': (gobject.SIGNAL_RUN_LAST,
                    gobject.TYPE_NONE,
                    (gobject.TYPE_STRING, gtk.gdk.Event))
    }

    def __init__(self, g, w):
        gtk.DrawingArea.__init__(self)

        # register context id for messages to status bar
        # only one msg will be active for this cid
        statusbars.warning.zoom_cid = statusbars.warning.get_context_id("overzoomed")

        # temanejo main window
        self.main_window = w

        # graph object:
        self.graph = g
        self.last_clicked_nodeId = 0

        # graph view object
        self.graphview = GraphView(g)

        # some user interface settings:
        self.autozoom = False
        self.zoom_once = True

        self.set_flags(gtk.CAN_FOCUS)

        self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK)
        self.connect("button-press-event", self.on_area_button_press)
        self.connect("button-release-event", self.on_area_button_release)
        self.add_events(gtk.gdk.POINTER_MOTION_MASK |
                        gtk.gdk.POINTER_MOTION_HINT_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK)
        self.connect("motion-notify-event", self.on_area_motion_notify)
        self.connect("scroll-event", self.on_area_scroll_event)
        self.connect("size-allocate", self.on_area_size_allocate)

        #self.connect('key-press-event', self.on_key_press_event)

        self.x, self.y = 0.0, 0.0
        self.zoom_ratio = .4
        self.zoom_to_fit_on_resize = False
        self.animation = NoAnimation(self)
        self.drag_action = NullAction(self)
        self.presstime = None

        self.info_menu_item = None
        self.context_menu = gtk.Menu()
        menu_item = gtk.MenuItem(None)  # separator
        self.context_menu.append(menu_item)
        self.menu_item_mark = gtk.MenuItem('Mark this task')
        self.menu_item_mark.connect_object(
            'activate', self.on_context_menu_activate, 'mark_task')
        self.context_menu.append(self.menu_item_mark)
        menu_item = gtk.MenuItem('Show neighbours of this task')
        menu_item.connect_object(
            'activate', self.on_context_menu_activate, 'show_neighbours')
        self.context_menu.append(menu_item)
        # menu_item = gtk.MenuItem('Show shortest path to marked task')
        # menu_item.connect_object(
        #    'activate', self.on_context_menu_activate, 'show_shortest')
        # self.context_menu.append(menu_item)
        self.menu_item_block = gtk.MenuItem('Block this task')
        self.menu_item_block.connect_object(
            'activate', self.on_context_menu_activate, 'block_task')
        self.context_menu.append(self.menu_item_block)
        self.menu_item_block.set_sensitive(client_conf.have_blocktask)
        self.menu_item_depend = gtk.MenuItem(
            'Make this task dependent on marked task')
        self.menu_item_depend.connect_object(
            'activate', self.on_context_menu_activate, 'depend_task')
        self.context_menu.append(self.menu_item_depend)
        self.menu_item_depend.set_sensitive(client_conf.have_makedeps)
        self.menu_item_stop = gtk.MenuItem('Stop when this task is reached')
        self.menu_item_stop.connect_object(
            'activate', self.on_context_menu_activate, 'pause_on_task')
        self.context_menu.append(self.menu_item_stop)
        self.menu_item_prio = gtk.MenuItem('Prioritise this task')
        self.menu_item_prio.connect_object(
            'activate', self.on_context_menu_activate, 'execute_task')
        self.context_menu.append(self.menu_item_prio)
        self.menu_item_prio.set_sensitive(client_conf.have_priorizetask)

        self.context_menu.set_title('Title')
        self.context_menu.show_all()

        self.shall_draw_key = True

    def on_context_menu_activate(self, action):
        graph = self.graph
        graphview = self.graphview
        nodeId = self.last_clicked_nodeId
        taskId = graph.get_taskId(nodeId)
        if action is 'block_task':
            if nodeId in graph.blockedNodes:
                self.main_window.comm.send_request(0, 'AYU_BLOCKTASK',
                                                   [taskId, 0])
                graph.blockedNodes.remove(nodeId)
            else:
                self.main_window.comm.send_request(0, 'AYU_BLOCKTASK',
                                                   [taskId, 1])
                graph.blockedNodes.append(nodeId)
            time.sleep(.1)
            self.queue_draw()

        elif action is 'depend_task':
            # there is a marked task and
            # clicked node is not marked itself and
            # marked node is not finished yet and
            # clicked task is not running yet:
            if (graphview.marked_node is not None and
                    graphview.marked_node != nodeId and
                    graph._status[graphview.marked_node] < 3 and
                    graph._status[nodeId] < 2):
                self.main_window.comm.send_request(0, 'AYU_BLOCKTASK',
                                                   [taskId, 1])
                graph.blockedNodes.append(nodeId)
                if graphview.marked_node in graph.dependentNodes:
                    graph.dependentNodes[graphview.marked_node].append(nodeId)
                else:
                    graph.dependentNodes[graphview.marked_node] = [nodeId]
            self.queue_draw()

        elif action is 'pause_on_task':
            self.main_window.label_stopontask.set_text(str(taskId))
            if not self.main_window.check_stopontask.get_active():
                self.main_window.check_stopontask.set_active(True)
            else:
                self.main_window.on_controlcheckbutton(
                    self.main_window.check_stopontask, data='stopontask')

        elif action is 'execute_task':
            if self.last_clicked_nodeId in graph.criticalNodes:
                self.main_window.comm.send_request(0, 'AYU_PRIORITISETASK',
                                                   [taskId, 0])
                graph.criticalNodes.remove(nodeId)
            else:
                self.main_window.comm.send_request(0, 'AYU_PRIORITISETASK',
                                                   [taskId, 1])
                graph.criticalNodes.append(nodeId)

        elif action is 'show_neighbours':
            # unhighlight everything
            for n in graph:
                graph._highlight[n] = 0
            # highlight the clicked node
            graph._highlight[nodeId] = 1
            graphview.marked_node = nodeId
            # highlight the neighbours
            for n in graph.predecessors(nodeId):
                graph._highlight[n] = 2
            for n in graph.successors(nodeId):
                graph._highlight[n] = 2
            #self.animate_to(self.x_tmp, self.y_tmp)
            self.queue_draw()

        elif action is 'mark_task':
            # unhighlight everything
            for n in graph:
                graph._highlight[n] = 0

            if graphview.marked_node == nodeId:
                graphview.marked_node = None
                graphview.distance_to_marked = {}
                graphview.dist_max = 0
            else:
                # highlight the clicked node
                graph._highlight[nodeId] = 1
                graphview.marked_node = nodeId
                graphview.dist_max = 0
                graphview.calc_all_distances(nodeId)
                #self.animate_to(self.x_tmp, self.y_tmp)
            self.queue_draw()

        #~ elif action is 'show_shortest':
            #~ # unhighlight everything
            #~ for n in graph:
                #~ graph.highlight[n] = 0
            #~ # highlight the clicked node
            #~ graph.highlight[nodeId] = 1
            #~ graphview.marked_node = nodeId
            #~ #self.animate_to(self.x_tmp, self.y_tmp)

    def do_expose_event(self, event):
#        self.window.freeze_updates()
        cr = self.window.cairo_create()
        # set a clip region for the expose event
        cr.rectangle(
            event.area.x, event.area.y,
            event.area.width, event.area.height
        )
        cr.clip()

        cr.set_source_rgba(1.0, 1.0, 1.0, 1.0)
        cr.paint()

        cr.save()
        rect = self.get_allocation()
        cr.translate(0.5*rect.width, 0.5*rect.height)
        cr.scale(self.zoom_ratio, self.zoom_ratio)
        cr.translate(-self.x, -self.y)

        if (isinstance(self.animation, NoAnimation) and
                isinstance(self.drag_action, NullAction)):
            self.graphview.draw_only_nodes = False
        else:
            self.graphview.draw_only_nodes = True

        # autozoom
        if (self.autozoom is True and
                self.graphview.width > 1 and
                self.graph.is_invalid is True):
            self.zoom_once = True

        if DEBUG:
            self.graphview.draw(cr)
        else:
            try:
                self.graphview.draw(cr)
            except:
                logging.warning('drawing graph failed.')
                pass

        if hasattr(self.main_window, 'update_ui'):
            if self.graphview.force_redraw is True:
                self.graphview.force_redraw is False
                self.main_window.update_ui('counters')
                self.main_window.update_ui('log')
                self.main_window.update_ui('statusbar')

        # autozoom
        #if self.autozoom and self.graphview.width > 1:
        #    self.__zoom_to_fit()
        # zoom once (after ui changes, at startup)
        if self.zoom_once and self.graphview.width > 1:
            self.zoom_once = False
            self.__zoom_to_fit()

        cr.restore()

        self.drag_action.draw(cr)

        return False

#    def __get_current_pos(self):
#        return self.x, self.y

#    def __set_current_pos(self, x, y):
#        self.x = x
#        self.y = y
##        self.queue_draw()

    def __limit_zoomratio(self, zoom_ratio):
        # this is a hack.
        # better to encapsulate everything zooming related in a class
        if not hasattr(self, "_overzoomed"):
            self._overzoomed = False
        
        warning_msg = "Warning: Zoom level is too high or low!"
        overzooming = False  # set true if this current zoom hits limits

        # are we overzooming?
        min_zoom = 0.03  # nodes very small for zoom_ratio < 0.03
        max_zoom = 20    # nodes are bigger than screen for zoom_ratio > 20
        limited = max(min(max_zoom, zoom_ratio), min_zoom)
        overzooming = True if limited != zoom_ratio else False
        
        # logging
        if (overzooming ^ self._overzoomed):  # changed state, need to change warnings
            statusbar = statusbars.warning
            if overzooming:     # changed from ok-zoom to overzoomed
                statusbar.push(statusbar.zoom_cid, warning_msg)
            else:
                statusbar.pop(statusbar.zoom_cid)

        self._overzoomed = overzooming   # save state

        # honor zoom limit?
        # we should also test if the reason for zooming was an
        # explicit user request (then honor it) or implicitely
        # due to autozoom. For now, assume we are autozooming
        # without explicit user request.
        auto_zooming = True
        if (self.autozoom and auto_zooming):
            return limited
        else:      # user knows what he is doing
            return zoom_ratio

    def __zoom_image(self, zoom_ratio, center=False, pos=None):

        zoom_ratio = self.__limit_zoomratio(zoom_ratio)
        
        if self.graphview.width != 0:
            if center:
                self.x = self.graphview.x_c
                self.y = -self.graphview.y_c

            elif pos is not None:
                rect = self.get_allocation()
                x, y = pos
                x -= 0.5*rect.width
                y -= 0.5*rect.height
                self.x += old_div(x, self.zoom_ratio, expect_float=True) - old_div(x, zoom_ratio, expect_float=True)
                self.y += old_div(y, self.zoom_ratio, expect_float=True) - old_div(y, zoom_ratio, expect_float=True)
            self.zoom_ratio = zoom_ratio
            self.zoom_to_fit_on_resize = False

        self.queue_draw()

    def zoom_to_area(self, x1, y1, x2, y2):
        rect = self.get_allocation()
        width = abs(x1 - x2)
        height = abs(y1 - y2)
        if width == 0 or height == 0:
            return
        self.zoom_ratio = min(
            float(rect.width)/float(width),
            float(rect.height)/float(height)
        )
        self.zoom_to_fit_on_resize = False
        self.x = old_div((x1 + x2), 2, expect_float=True)
        self.y = old_div((y1 + y2), 2, expect_float=True)
        self.queue_draw()

    def __zoom_to_fit(self):
        rect = self.get_allocation()
        rect.x += self.ZOOM_TO_FIT_MARGIN
        rect.y += self.ZOOM_TO_FIT_MARGIN
        rect.width -= 2 * self.ZOOM_TO_FIT_MARGIN
        rect.height -= 2 * self.ZOOM_TO_FIT_MARGIN
        if self.graphview.width == 0 or self.graphview.height == 0:
            return
        zoom_ratio = min(
            float(rect.width)/float(self.graphview.width),
            float(rect.height)/float(self.graphview.height)
            )

        self.__zoom_image(zoom_ratio, center=True)
        self.zoom_to_fit_on_resize = True

    ZOOM_INCREMENT = 1.25
    ZOOM_TO_FIT_MARGIN = 25

    def on_zoom_in(self, action):
        self.__zoom_image(self.zoom_ratio * self.ZOOM_INCREMENT)
#        self.queue_draw()

    def on_zoom_out(self, action):
        self.__zoom_image(old_div(self.zoom_ratio, self.ZOOM_INCREMENT, expect_float=True))
#        self.queue_draw()

    def on_zoom_fit(self, action):
        self.__zoom_to_fit()
#        self.queue_draw()

    def on_zoom_100(self, action):
        self.__zoom_image(1.0)
#        self.queue_draw()

    POS_INCREMENT = 100

    #~ def on_key_press_event(self, widget, event):
        #~ if event.keyval == gtk.keysyms.Left:
            #~ self.x -= self.POS_INCREMENT/self.zoom_ratio
            #~ self.queue_draw()
            #~ return True
        #~ if event.keyval == gtk.keysyms.Right:
            #~ self.x += self.POS_INCREMENT/self.zoom_ratio
            #~ self.queue_draw()
            #~ return True
        #~ if event.keyval == gtk.keysyms.Up:
            #~ self.y -= self.POS_INCREMENT/self.zoom_ratio
            #~ self.queue_draw()
            #~ return True
        #~ if event.keyval == gtk.keysyms.Down:
            #~ self.y += self.POS_INCREMENT/self.zoom_ratio
            #~ self.queue_draw()
            #~ return True
        #~ if (event.keyval == gtk.keysyms.Page_Down or
        #~            event.keyval == gtk.keysyms.minus):
            #~ self.__zoom_image(self.zoom_ratio / self.ZOOM_INCREMENT)
            #~ self.queue_draw()
            #~ return True
        #~ if (event.keyval == gtk.keysyms.Page_Up or
        #~            event.keyval == gtk.keysyms.plus):
            #~ self.__zoom_image(self.zoom_ratio * self.ZOOM_INCREMENT)
            #~ self.queue_draw()
            #~ return True
        #~ if event.keyval == gtk.keysyms.Escape:
            #~ self.drag_action.abort()
            #~ self.drag_action = NullAction(self)
            #~ return True
#        if event.keyval == gtk.keysyms.r:
#            self.reload()
#            return True
#        if event.keyval == gtk.keysyms.q:
#            gtk.main_quit()
#            return True
        #~ return False

    def __get_drag_action(self, event):
        state = event.state
        if event.button is 1:  # left button
            if state & gtk.gdk.CONTROL_MASK:
                return ZoomAction
            else:
                return PanAction
        if event.button is 2:  # middle button
            return ZoomAreaAction
        return NullAction

    def on_area_button_press(self, area, event):
        self.animation.stop()
        self.drag_action.abort()
        action_type = self.__get_drag_action(event)
        self.drag_action = action_type(self)
        self.drag_action.on_button_press(event)
        self.presstime = time.time()
        self.pressx = event.x
        self.pressy = event.y
        return False

    def __is_click(self, event, click_fuzz=4, click_timeout=1.0):
        assert event.type == gtk.gdk.BUTTON_RELEASE
        if self.presstime is None:
            # got a button release without seeing the press?
            return False
        # XXX instead of doing this complicated logic, shouldn't we listen
        # for gtk's clicked event instead?
        deltax = self.pressx - event.x
        deltay = self.pressy - event.y
        return (time.time() < self.presstime + click_timeout
                and math.hypot(deltax, deltay) < click_fuzz)

    def on_area_button_release(self, area, event):
        self.drag_action.on_button_release(event)
        self.drag_action = NullAction(self)
        x, y = int(event.x), int(event.y)
        x, y = self.window2graph(x, y)
        if event.button == 1 and self.__is_click(event):  # left-click
            self.animate_to(x, y)
            return True
        if event.button == 3 and self.__is_click(event):  # right-click
            nodeId = self.graphview.get_nodeId_from_xy(x, y)
            self.x_tmp = x
            self.y_tmp = y
            if nodeId is not None:
                self.last_clicked_nodeId = nodeId
                taskId = self.graph.get_taskId(nodeId)
                if self.info_menu_item is not None:
                    self.context_menu.remove(self.info_menu_item)
                info_string = 'TaskId: %s\nThreadId: %s \nFunction: %s' % \
                    (taskId,
                     self.graph.get_threadId(taskId),
                     self.graph.get_funcname(self.graph.get_funcId(taskId)))
                if self.graphview.marked_node is not None:
                    dist_to_marked = self.graph.distance(
                        self.graph.get_nodeId(taskId), self.graphview.marked_node)
                    info_string += '\nDistance to marked: %s' % dist_to_marked

                self.info_menu_item = gtk.MenuItem(info_string, False)
                self.info_menu_item.set_sensitive(True)
                self.info_menu_item.show()
                self.context_menu.prepend(self.info_menu_item)

                if self.last_clicked_nodeId == self.graphview.marked_node:
                    self.menu_item_mark.get_child().set_text(
                        'Unmark this task')
                else:
                    self.menu_item_mark.get_child().set_text(
                        'Mark this task')

                if self.last_clicked_nodeId in self.graph.blockedNodes:
                    self.menu_item_block.get_child().set_text(
                        'Unblock this task')
                else:
                    self.menu_item_block.get_child().set_text(
                        'Block this task')

                # if there is a marked task and
                # clicked node is not marked itself
                if (self.graphview.marked_node is not None
                        and self.graphview.marked_node != nodeId):
                    self.menu_item_depend.set_sensitive(client_conf.have_makedeps)
                else:
                    self.menu_item_depend.set_sensitive(False)

                if self.last_clicked_nodeId in self.graph.criticalNodes:
                    self.menu_item_prio.get_child().set_text(
                        'Deprioritise this task')
                else:
                    self.menu_item_prio.get_child().set_text(
                        'Prioritise this task')

                if self.graph._status[nodeId] > 1:
                    if nodeId not in self.graph.blockedNodes:
                        self.menu_item_block.set_sensitive(False)
                    self.menu_item_stop.set_sensitive(False)
                else:
                    self.menu_item_block.set_sensitive(client_conf.have_blocktask)
                    self.menu_item_stop.set_sensitive(True)
                if self.graph._status[nodeId] > 0:
                    self.menu_item_prio.set_sensitive(False)
                else:
                    self.menu_item_prio.set_sensitive(client_conf.have_priorizetask)

                self.context_menu.popup(None, None, None, event.button, 0)
            return True
        self.queue_draw()
        return False

    def on_area_scroll_event(self, area, event):
        if event.direction == gtk.gdk.SCROLL_UP:
            self.__zoom_image(self.zoom_ratio * self.ZOOM_INCREMENT,
                            pos=(event.x, event.y))
            return True
        if event.direction == gtk.gdk.SCROLL_DOWN:
            self.__zoom_image(old_div(self.zoom_ratio, self.ZOOM_INCREMENT, expect_float=True),
                            pos=(event.x, event.y))
            return True
        return False

    def on_area_motion_notify(self, area, event):
        self.drag_action.on_motion_notify(event)
        return True

    def on_area_size_allocate(self, area, allocation):
        if self.zoom_to_fit_on_resize:
            self.__zoom_to_fit()

    def animate_to(self, x, y):
        self.animation = ZoomToAnimation(self, x, y)
        self.animation.start()

    def window2graph(self, x, y):
        rect = self.get_allocation()
        x -= 0.5*rect.width
        y -= 0.5*rect.height
        x /= self.zoom_ratio
        y /= self.zoom_ratio
        x += self.x
        y += self.y
        return x, y

#    def __get_jump(self, x, y):
#        x, y = self.window2graph(x, y)
#        return self.graph.__get_jump(x, y)

#******

#****c* temanejo_graphWidget/NIYDialog
# NAME
#   NIYDialog
#   - dialog displaying a short notice saying "Not implemented yet"
# DESCRIPTION
#   This Dialog displays a short message saying "Not implemented yet"
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
# SOURCE


class NIYDialog(gtk.Dialog):
    def __init__(self, msg=''):
        gtk.Dialog.__init__(self,
                            title='Help',
                            parent=self,
                            flags=gtk.DIALOG_MODAL |
                            gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        label = gtk.Label()
        frame = gtk.Frame()
        label.set_text(msg + ': \nThis feature is not implemented yet.')
        align = gtk.Alignment()
        frame.set_border_width(10)
        align.add(label)
        align.set_padding(20, 20, 20, 20)
        frame.add(align)
        self.vbox.pack_start(frame, True, True, 0)
        self.show_all()
#******

#****c* Temanejo_graph/GraphView
# NAME
#   Graph - class providing a view on the dependency graph of a task parallel 
#   application
# DESCRIPTION
#   GraphView implements visualisation and interaction routines for Graph.
#   It is intented to be imported from Temanejo.
# METHODS
#   __init__
#   calc_all_distances
#   clear
#   draw
#   __draw_barriers
#   __draw_dependentEdges
#   __draw_edge_lines
#   __draw_edges
#   __draw_edges_linear
#   __draw_label_background
#   __draw_labels
#   __draw_margin
#   __draw_nodes
#   __draw_shape
#   get_None
#   get_col_disttotask
#   get_col_funcId
#   get_col_status
#   get_col_taskduration
#   get_col_threadId
#   get_dtminmax
#   get_nodeId_from_xy
#   get_points_of_shape
#   get_shape_funcId
#   get_shape_none
#   get_shape_threadId
#   get_size
#   graph_layout
#   __palette_rgb
#   __palette_rgb_inf
#   __square_distance
# SOURCE


class GraphView:
    def __init__(self, graph):
        self.graph = graph  # Graph object to be viewed
        self.width = 0  # width of graph layout on screen
        self.height = 0  # height of graph layout on screen
        self.x_c = 0.0  # x coordinate of centre
        self.y_c = 0.0  # y coordinate of centre
        self.node_size = 40  # size of node (radius of enclosing circle)
        self.node_separation = 4*self.node_size # space between nodes
        self.margin_width = int(self.node_size/4. + 0.5)  # thickness of status margin
        self.edge_width = 2  # thickness of edge
        self._node_label_fontsize = int(self.node_size/2. + 0.5)
        self._edge_label_fontsize = int(self.node_size/5. + 0.5)
        self._edge_label_margin = 1
        self.multdep_separation = 4  # separation of multiple dependencies
        self.use_layout = 'temanejo'  # which type of graph layout to plot
                                      # 'temanejo', 'pygraphviz' or 'linear'
        self.layout_list = {}  # list of graphical layouts of each subgraph
        self.layout = {}  # graphical layout of the graph
        self.offset_of_subgraph = {}  # x and y offsets for each subgraph
        self.global_tasks_in_first_row = False  # wether to draw global tasks
                                                # in first row
        self.force_redraw = True  # force redrawing when returning from draw()
        self.draw_only_nodes = False  # set to True for faster drawing of big
                                      # graphs, e.g during panning
        self.is_invalid = True  # whether to draw graph again
        self.node_colours = 'function'  # what the node colour means by default
                                        # One of 'function', 'task_duration',
                                        # 'status', 'thread', 'nothing'
        self.shapes_mean = 'thread'  # what the node shape means by default.
                                     # One of 'function', 'thread', 'nothing'
        self.margins_mean = 'status'  # what the margin colour means by default
                                      # One of 'function', 'thread',
                                      # 'task_duration', 'status', 'nothing'

        self.shape_list = ['box',
                           'triangle',
                           'diamon',
                           'house',
                           'trapezium',
                           'varpentagra',
                           'pentago',
                           'hexago',
                           'invtriangle',
                           'invhouse',
                           'invtrapezium',
                           'invpentago']

    # , 'ellipse', 'parallelogram', 'invtrapezium', 'Mdiamond', 'pentagra',
    # 'Msquare', 'Mcircle', 'polygon, sides=7'

        self.commthreadId = 16  # thread id of communication thread
        self.addr_key = 'orig_addr'  # key for choosing address data.
                                     # one of 'addr', 'orig_addr', 'nothing'

        self.layout_direction = 't2b'  # direction of graph,
                                       # one of 't2b' (default), 'b2t',
                                       # 'l2r', 'r2l'
        self.palette_num_colours = 64
        # hash 64-bit ids down to 10 bits, then modulo down to palette_num_colors values
        # if id has small numerical value, assume it is sequential and leave untouched for familiar colours
        self.hash_id = lambda id: id if id<self.palette_num_colours else (hashto10bits(id) % self.palette_num_colours)
        hashes = [self.hash_id(n) for n in range(self.palette_num_colours)]
        self.palette_nodes = [self.__palette_rgb_inf(h) for h in hashes]
        self.palette_edges = [self.__palette_rgb_inf(h) for h in hashes]

        self.draw_edge_labels = True  # whether to draw edge labels
        self.show_markers = True  # whether to show blocked and prioriry
                                  # markers on nodes
        self.marked_node = None  # marked node
        self.distance_to_marked = {}  # holds the distance to the marked task
        self.subgraph_pos = []
        self.barrier_lines = []  # at which y offset to draw barrier lines
        #~ self.barrier_of_subgraph = {}  # number of barriers before subgraph
        self.row_width = [0]  # list of widths of each row
        self.row_height = [0]  # list of heights of each row
        self.row_xoff = [[0]]  # list of horizontal offsets of each subgraph
                               # in each row
        self.row_yoff = [0]  # list of vertical offsets of each row
        self.dt_min = 999999999
        self.dt_max = 0
        self.dist_max = 0
        #self.subgraph_of_node = {} # subgraph of each node

    def clear(self):

        self.width = 0
        self.height = 0
        self.x_c = 0.0
        self.y_c = 0.0

        self.layout_list = {}
        self.layout = {}
        self.offset_of_subgraph = {}
        self.force_redraw = True
        self.draw_only_nodes = False
        self.is_invalid = True
        self.marked_node = None
        self.distance_to_marked = {}
        self.subgraph_pos = []
        self.barrier_lines = []
        #~ self.barrier_of_subgraph = {}
        self.row_width = [0]
        self.row_height = [0]
        self.row_xoff = [[0]]
        self.row_yoff = [0]
        self.dt_min = 999999999
        self.dt_max = 0
        self.dist_max = 0
        self.barrier_lines = []

    def get_size(self):
        return self.width, self.height

    def draw(self, cr):
#        print "nodes: \n", self.graph.nodes()
#        print "edges: \n", self.graph.edges()

        if networkx.is_directed_acyclic_graph(self.graph) is not True:
            logging.error('graph is not acyclic! Cowardly refusing to draw graph.')
            return

        if len(self.graph) == 0:
            return

        nodes_tmp = None
        edges_tmp = None

        while self.graph.is_invalid or self.is_invalid:
            # check for ongoing changes to the graph
            l = len(self.graph)
            time.sleep(.001)
            if l != len(self.graph):
                self.force_redraw = True
                continue
            
            # temporary copies
            nodes_tmp = self.graph.nodes()
            edges_tmp = self.graph.edges()
            graph_tmp = networkx.MultiDiGraph(self.graph)
            graph_tmp2 = networkx.MultiDiGraph(graph_tmp)

            # layout
            if (self.use_layout == 'temanejo' or
                    self.use_layout == 'pygraphviz'):

                if self.use_layout == 'pygraphviz':
                    self.layout = networkx.pygraphviz_layout(graph_tmp,'dot')
                elif self.use_layout == 'temanejo':
                    #(self.layout_list,t_layout,total_cross_count,
                    #   total_edge_length,n_width,n_height) = \
                    #   self.graph_layout(graph_tmp)
                    (self.layout_list, width_list, height_list) = \
                        self.__graph_layout(graph_tmp)
                    #~
        #            print "#graph layout 5 lastet", (time.time()-t0)*1e3, "ms"
        #~
                                #~ # analysis
                                #~
                                #~ n_nodes = self.number_of_nodes()
                                #~ n_edges = self.number_of_edges()
                                # float array of outgoing edges for each node:
                                #~ out_edges = \
                                #~    [float(x) for x in
                                #~    self.out_degree().values()]
                                # # third moment of edge distribution per node:
                                #~ mu3 = sum(
                                #~   ( out_edges-pylab.mean(out_edges) )**3 )/
                                #~   len(out_edges) / n_edges**3
                                # normalised standard deviation:
                                #~ std_dev = pylab.sqrt(
                                #~   pylab.var(out_edges))/n_edges
                                # skewness = mu3 / standard_deviation**3:
                                #~ skewness = mu3 / std_dev**3
                                # median of outgoing edges per node:
                                #~ med_out = pylab.median(out_edges)
                                #~ med_var = \
                                #~   sorted(out_edges)[int(len(out_edges)*.99)]
                                #pylab.hist(out_edges)
                                #pylab.show()

                    if self.layout_list == 0 or self.layout_list is None:
                        logging.error('layout_list == 0')
                        return
                    
                    # generate layout from layout_list

                    # sort subgraphs according to their smallest task number
                    inx = range(len(self.layout_list))
                    inx.sort(lambda x, y: cmp(min(self.layout_list[x]),
                             min(self.layout_list[y])))
                    self.layout_list = [self.layout_list[i] for i in inx]
        #            self.layout_list.sort(key=min)
                    width_list = [width_list[i] for i in inx]
                    height_list = [height_list[i] for i in inx]

                    # prepare arrays and variables for final complete layout
                    self.layout = {}
                    self.barrier_lines = []
                    self.row_width = [0]
                    self.row_xoff = [[0]]  # list of horizontal offsets of
                                           # each subgraph in each row
                    self.row_yoff = [0]  # list of vertical offset of each row
                    self.subgraph_pos = []
                    self.count_consecutive_barriers = []
                    s_i = 0
                    s_x = 0
                    s_y = 0
                    last_barrier = 0
                    current_row = 0

                    
                    # loop over subgraphs
                    for s_i, subgraph in enumerate(self.layout_list):
                        # barrier between rows
                        if last_barrier < \
                                self.graph._barrier_count[subgraph.keys()[0]]:
                            self.count_consecutive_barriers.append(
                                self.graph._barrier_count[subgraph.keys()[0]] -
                                last_barrier)

                            last_barrier = \
                                self.graph._barrier_count[subgraph.keys()[0]]
                            current_row += 1
                            self.row_width.append(0)
                            self.row_height.append(0)
                            self.row_xoff.append([0])
                            s_x = 0
                            s_y += 1
                            self.row_yoff.append(
                                self.row_yoff[current_row-1] -
                                self.row_height[current_row-1] * self.node_separation - self.node_separation)
                            self.barrier_lines.append(
                                self.row_yoff[current_row]+self.node_separation)

                        self.row_width[current_row] += width_list[s_i]
                        self.row_xoff[current_row].append(
                            self.row_width[current_row])
                        self.row_height[current_row] = max(
                            self.row_height[current_row], height_list[s_i])
                        self.subgraph_pos.append((s_x, s_y))
                        s_x += 1

                    # fill the final layout
                    for s_i, subgraph in enumerate(self.layout_list):
                        for n in subgraph:
                            (s_x, s_y) = self.subgraph_pos[s_i]
                            self.layout[n] = (subgraph[n][0] +
                                              self.row_xoff[s_y][s_x] * self.node_separation,
                                              subgraph[n][1] +
                                              self.row_yoff[s_y])

                    # append final barrier, if it exists
                    if self.graph.total_barrier_count > last_barrier:
                        self.barrier_lines.append(
                            self.row_yoff[current_row]-height_list[-1] * self.node_separation - self.node_separation)
                        self.count_consecutive_barriers.append(1)

                # reorder coordinates for other than top to bottom display
                if self.layout_direction == 'b2t':
                    for n in self.layout:
                        self.layout[n] = \
                            (self.layout[n][0], -self.layout[n][1])
                elif self.layout_direction == 'r2l':
                    for n in self.layout:
                        self.layout[n] = \
                            (self.layout[n][1], self.layout[n][0])
                elif self.layout_direction == 'l2r':
                    for n in self.layout:
                        self.layout[n] = \
                            (-self.layout[n][1], self.layout[n][0])

                xmin = ymin = 9999999
                xmax = ymax = -9999999
                for l in self.layout:
                    if self.layout[l][0] < xmin:
                        xmin = self.layout[l][0]
                    if self.layout[l][0] > xmax:
                        xmax = self.layout[l][0]
                    if self.layout[l][1] < ymin:
                        ymin = self.layout[l][1]
                    if self.layout[l][1] > ymax:
                        ymax = self.layout[l][1]

                if self.use_layout == 'temanejo':
                    # if there is a final barrier line, enlarge graphview
                    if self.graph.total_barrier_count > last_barrier:
                        if self.layout_direction == 't2b':
                            ymin -= self.node_separation * 2
                        elif self.layout_direction == 'b2t':
                            ymax += self.node_separation * 2
                        elif self.layout_direction == 'l2r':
                            # needs to be more because of the label:
                            xmax += self.node_separation * 2.45
                        elif self.layout_direction == 'r2l':
                            xmin -= self.node_separation * 2

                # calculate width and height of graph view
                self.width = int(xmax - xmin +
                                 2*self.node_size + 2*self.margin_width)
                self.height = int(ymax - ymin +
                                  2*self.node_size + 2*self.margin_width)

                # calculate centre of graph view
                self.x_c = 0.5*(xmax - xmin) + xmin
                self.y_c = 0.5*(ymax - ymin) + ymin

                # if graph changed in between, do again
                if len(graph_tmp2) != len(self.graph):
                    nodes_tmp = None
                    edges_tmp = None
                    self.force_redraw = True
                    continue
                else:
                    self.graph.is_invalid = False
                    self.is_invalid = False

            elif self.use_layout == 'linear':
                self.graph.is_invalid = False
                self.is_invalid = False

        if self.use_layout == 'temanejo' or self.use_layout == 'pygraphviz':

            nodes_tmp = self.layout.keys()
            edges_tmp = self.graph.subgraph(nodes_tmp).edges()

            if self.use_layout == 'temanejo':
                barriers_tmp = self.barrier_lines

            self.__draw_nodes(cr, nodes_tmp)
            if not self.draw_only_nodes:
                self.__draw_edges(cr, edges_tmp)
                self.__draw_labels(cr, nodes_tmp, edges_tmp)
                if self.use_layout == 'temanejo':
                    self.__draw_barriers(cr, barriers_tmp)
                if len(self.graph.dependentNodes) > 0:
                    self.__draw_dependentEdges(cr)

        elif self.use_layout == 'linear':
            self.__draw_edges_linear(cr, self.graph.edges(), self.graph.nodes())

#        self.__draw_bounding_rect(cr)

### for debugging
#    def __draw_bounding_rect(self, cr):
#        cr.set_line_width(10)
#
#        cr.move_to(0, 0)
#        cr.line_to(200, 0)
#        cr.move_to(0, 0)
#        cr.line_to(0, 100)
#        cr.set_source_rgba(0.0, 1.0, 0.0, 1.0)
#        cr.stroke()
#
#        cr.move_to(self.x_c-100, -self.y_c)
#        cr.line_to(self.x_c+100, -self.y_c)
#        cr.move_to(self.x_c, -self.y_c-100)
#        cr.line_to(self.x_c, -self.y_c+100)
#        cr.rectangle(self.xmin, -self.ymin, self.width, self.height)
#        cr.set_source_rgba(0.0, 0.0, 1.0, 1.0)
#        cr.stroke()
#
#        cr.set_line_width(1)

    def __graph_layout(self, graph):

        #t0 = time.time()

        if graph.number_of_nodes() == 0:
            return None

        graph_list = list(networkx.weakly_connected_component_subgraphs(graph))
        number_of_independent_graphs = len(graph_list)

        layout_list = []
        height_list = [0] * number_of_independent_graphs
        width_list = [0] * number_of_independent_graphs

#        total_cross_count = 0
#        total_edge_length = 0

        for layout_i, g in enumerate(graph_list):
            layout_tmp = {}

            # case for one or two nodes
            if g.number_of_nodes() <= 2:
                i_n = 0
                for n in sorted(g.nodes()):
                    x = self.node_separation
                    y = -i_n * self.node_separation
                    layout_tmp[n] = (x, y)
                    #layout_list[layout_i][n] = (x + layout_xmax, y)
                    #layout_list[n] = (x + layout_xmax, y)
                    i_n += 1
                #layout_xmax = layout_xmax + self.node_separation
                width_list[layout_i] = 1
                height_list[layout_i] = g.number_of_nodes()
                layout_list.append(layout_tmp)
                continue

            # set up data arrays
            level_of_node = {}  # level for each node
            index_of_node = {}  # index for each node
            predecessors_of_node = {}  # predecessors of each node
            successors_of_node = {}  # successors of each node
            nodes_in_level = [[]]  # nodes in each level
            number_of_levels = 1  # total number of levels
            number_of_dependencies = {}  # number of dependencies for each node
            single_dep_neighbours = {}  # list of neighbours with only one
                                        # dependency for each node
            multi_dep_neighbours = {}  # list of neighbours with more than one
                                       # dependency for each node
            single_dep_nodes = []  # list of nodes with exactly one dependency
            multi_dep_nodes = []  # list of nodes with more than one dependency

            # fill predecessors_of_node, successors_of_node etc.
            for n in g.nodes_iter():
                predecessors_of_node[n] = g.predecessors(n)
                successors_of_node[n] = g.successors(n)
                number_of_dependencies[n] = len(predecessors_of_node[n]) + \
                    len(successors_of_node[n])
                single_dep_neighbours[n] = []
                multi_dep_neighbours[n] = []

                for x in predecessors_of_node[n] + successors_of_node[n]:
                    if len(g.predecessors(x)) + len(g.successors(x)) == 1:
                        single_dep_neighbours[n].append(x)
                    else:
                        multi_dep_neighbours[n].append(x)

                if number_of_dependencies[n] == 1:
                    single_dep_nodes.append(n)
                elif number_of_dependencies[n] > 1:
                    multi_dep_nodes.append(n)

            # create subgraph with multi dependency nodes only
            subgraph = g.subgraph(multi_dep_nodes)

            # arrange all nodes of subgraph in levels,
            for n in networkx.topological_sort(subgraph):
                # find maximum level of predecessors
                maxprelev = 0
                for p in subgraph.predecessors(n):
                    maxprelev = max(maxprelev, level_of_node[p])
                # put node one level below
                nlev = maxprelev + 1
                if nlev >= number_of_levels:  # nlev is a 0 based index
                    number_of_levels += 1
                    nodes_in_level.append([])
                nodes_in_level[nlev].append(n)
                level_of_node[n] = nlev

            # arrange vertically: move nodes up as far as possible
            for l in reversed(nodes_in_level):
                for n in l[:]:
                    if len(subgraph.successors(n)) == 0:
                        continue
                    # find minimum level of successors
                    minsuclev = number_of_levels
                    for s in subgraph.successors(n):
                        minsuclev = min(minsuclev, level_of_node[s])
                    if level_of_node[n] == minsuclev - 1:
                        continue
                    # put node one level above successor
                    nlev = minsuclev - 1
                    nodes_in_level[level_of_node[n]].remove(n)
                    nodes_in_level[nlev].append(n)
                    level_of_node[n] = nlev

            # arrange vertically: move nodes down as far as possible
            for l in nodes_in_level:
                for n in l[:]:
                    if len(subgraph.predecessors(n)) == 0:
                        continue
                   # find maximum level of predecessors
                    maxprelev = 0
                    for p in subgraph.predecessors(n):
                        if p in multi_dep_neighbours[n]:
                            maxprelev = max(maxprelev, level_of_node[p])
                    if level_of_node[n] == maxprelev + 1:
                        continue
                    # put node one level below
                    nlev = maxprelev + 1
                    if nlev >= number_of_levels:  # nlev is a 0 based index
                        number_of_levels += 1
                        nodes_in_level.append([])
                    nodes_in_level[level_of_node[n]].remove(n)
                    nodes_in_level[nlev].append(n)
                    level_of_node[n] = nlev

            # center levels
            max_level_len = max([len(l) for l in nodes_in_level])
            for l in nodes_in_level:
                levlen = len(l)
                l[0:] = [None]*((max_level_len-levlen)//2+1) + l + \
                    [None]*((max_level_len-levlen)//2)

            # fill index_of_node
            for l in nodes_in_level:
                for n in l:
                    if n is not None:
                        index_of_node[n] = l.index(n)

            # swap nodes
            for i in range(1):  # 6
                l_i = 0  # list index
                for l in nodes_in_level:
                    for n in l[1:]:
                        if n is None:
                            continue
                        left = l[index_of_node[n]-1]
                        suc = [x for x in subgraph.successors(n)
                               if level_of_node[x]-l_i < 2]
                        if left is None:
                            continue
                        left_suc = [x for x in subgraph.successors(left)
                                    if level_of_node[x]-l_i < 2]

                        cross_count = 0
                        cross_count_swap = 0
                        for s in suc:
                            cross_count += \
                                len([x for x in left_suc
                                     if index_of_node[x] > index_of_node[s]])
                            cross_count_swap += \
                                len([x for x in left_suc
                                     if index_of_node[x] < index_of_node[s]])

                        # swap nodes if it results in less crossings
                        if cross_count_swap < cross_count:

                            l[index_of_node[n]] = left
                            l[index_of_node[left]] = n

                            index_tmp = index_of_node[left]
                            index_of_node[left] = index_of_node[n]
                            index_of_node[n] = index_tmp

                    l_i += 1

            # swap nodes with None neighbours
            for i in range(10):  # 6
                break_flag = True
                l_i = 0  # list index
                for l in nodes_in_level:
                    for kk in range(len(l)//2):
                        break_flag = True
                        for n in l[1:-2]:
                            if n is None:
                                continue
                            i_n = l.index(n)
                            left = l[i_n-1]
                            right = l[i_n+1]
                            suc = [x for x in subgraph.successors(n)
                                   if level_of_node[x]-l_i < 2]
                            if left is not None and right is not None:
                                continue

                            mean_neighbour_index = 0  # nodes_tmp.index(n)
                            count = 0.0
                            for x in multi_dep_neighbours[n]:
                                if True:  # x in successors_of_node[n]:
                                    if abs(level_of_node[n] -
                                           level_of_node[x]) < 2:
                                        mean_neighbour_index += \
                                            nodes_in_level[
                                                level_of_node[x]].index(x)
                                        count += 1
                            if count == 0:
                                continue
                            mean_neighbour_index /= count

                            # swap nodes for being closer to
                            # mean_neighbour_index
                            if (mean_neighbour_index < i_n-.5 and
                                    left is None):
                                break_flag = False
                                l[i_n] = None
                                l[i_n-1] = n
                                index_of_node[n] = i_n - 1
                            elif (mean_neighbour_index > i_n+.5 and
                                    right is None):
                                break_flag = False
                                l[i_n] = None
                                l[i_n+1] = n
                                index_of_node[n] = i_n + 1

                        if break_flag is True:
                            break

                    l_i += 1

                if break_flag is True:
                    break

            # sort in single dependency nodes
            for l in nodes_in_level:
                for n in l[:]:
                    if n is None or n in single_dep_nodes:
                        continue
                    for p in predecessors_of_node[n]:
                        if p in single_dep_nodes:
                            i_n = l.index(n)
                            if (len(nodes_in_level[level_of_node[n] - 1])
                                    > i_n and
                                    nodes_in_level[level_of_node[n] - 1][i_n]
                                    is None):
                                nodes_in_level[level_of_node[n]-1][i_n] = p
                            else:
                                #print n,p,level_of_node[n], i_n
                                nodes_in_level[level_of_node[n]-1].insert(
                                    i_n, p)
                                nodes_in_level[level_of_node[n]].insert(
                                    i_n, None)
                                # if level_of_node[n] > 1:
                                    # nodes_in_level[
                                    #    level_of_node[n]-2].insert(i_n,None)
                            level_of_node[p] = level_of_node[n]-1

            for l in reversed(nodes_in_level):
                for n in l[:]:
                    if n is None:
                        continue
                    for s in successors_of_node[n]:
                        if s in single_dep_nodes:
                            i_n = l.index(n)
                            if level_of_node[n] >= number_of_levels-1:
                                nodes_in_level.append([])
                                number_of_levels += 1
                            #nodes_in_level[level_of_node[n]+1].insert(i_n, s)
                            if (len(nodes_in_level[level_of_node[n] + 1])
                                    > i_n and
                                    nodes_in_level[level_of_node[n] + 1][i_n]
                                    is None):
                                nodes_in_level[level_of_node[n]+1][i_n] = s
                            else:
                                nodes_in_level[level_of_node[n]+1].insert(
                                    i_n, s)
                                nodes_in_level[level_of_node[n]].insert(
                                    i_n, None)
                            level_of_node[s] = level_of_node[n]+1

            for n in reversed(single_dep_nodes):
                if len(successors_of_node[n]) != 0:
                    x = successors_of_node[n][0]
                else:  # if len(predecessors_of_node[n]) != 0:
                    x = predecessors_of_node[n][0]

                i_n = nodes_in_level[level_of_node[n]].index(n)
                i_x = nodes_in_level[level_of_node[x]].index(x)

                nodes_in_level[level_of_node[n]][i_n] = None
                if (len(nodes_in_level[level_of_node[n]]) > i_x and
                        nodes_in_level[level_of_node[n]][i_x] is None):
                    nodes_in_level[level_of_node[n]][i_x] = n
                elif (len(nodes_in_level[level_of_node[n]]) > i_x-1 and
                        nodes_in_level[level_of_node[n]][i_x-1] is None):
                    nodes_in_level[level_of_node[n]][i_x-1] = n
                elif (len(nodes_in_level[level_of_node[n]]) > i_x+1 and
                        nodes_in_level[level_of_node[n]][i_x+1] is None):
                    nodes_in_level[level_of_node[n]][i_x+1] = n
                else:
                    nodes_in_level[level_of_node[n]].insert(i_x, n)

            # fill index_of_node
            for l in nodes_in_level:
                for n in l:
                    if n is not None:
                        index_of_node[n] = l.index(n)

            # center levels
            max_level_len = max([len(l) for l in nodes_in_level])
            width_list[layout_i] = max_level_len
            for l in nodes_in_level:
                levlen = len(l)
                l[0:] = [None]*((max_level_len-levlen)//2+1) + l + \
                    [None]*((max_level_len-levlen)//2)

            # add dummy nodes for edges spanning more than 1 level
            #~ for l in nodes_in_level:
                #~ for n in nodes_in_level[l]:
                    #~ if n is None:
                        #~ continue
                    #~ suc = g.successors(n)
                    #~ if len(suc) == 0:
                        #~ continue
                    #~ for s in suc:
                        #~ if level_of_node[s] - l > 1:
                            #~ former_node_tmp = n
                            #~ for i in range(l+1,level_of_node[s]):
                                #~ node_tmp='%d_%d_%d_%d' % \
                                #~     (n,l,level_of_node[s],i)
                                #~ g.add_node(node_tmp)
                                #~ g.add_edge(former_node_tmp,node_tmp)
                                #~ level_of_node[node_tmp] = i
                                #~ nodes_in_level[i].append(node_tmp)
                                #~ former_node_tmp = node_tmp
                            #~ g.add_edge(former_node_tmp,level_of_node[s])

            # fill index_of_node
            for l in nodes_in_level:
                i_n = 0
                for n in l:
                    if n is not None:
                        index_of_node[n] = i_n
                    i_n += 1

            for k in range(10):
                # swap nodes
                for i in range(2):  # 6
                    l_i = 0  # list index
                    for l in nodes_in_level:
                        for n in l[1:]:
                            if n is None:
                                continue
                            left = l[index_of_node[n]-1]
                            suc = [x for x in graph.successors(n)
                                   if level_of_node[x]-l_i < 2]
                            if left is None:
                                continue
                            left_suc = [x for x in graph.successors(left)
                                        if level_of_node[x]-l_i < 2]

                            #~ edge_length = 0
                            #~ edge_length_swap = 0
                            #~ for s in suc:
                                #~ edge_length += \
                                #~    abs(index_of_node[s]-index_of_node[n])
                                #~ edge_length_swap += \
                                #~    abs(index_of_node[s]-index_of_node[n]+1)
                            #~ for s in left_suc:
                                #~ edge_length += \
                                #~    abs(index_of_node[s]-index_of_node[left])
                                #~ edge_length_swap += \
                                #~  abs(index_of_node[s]-index_of_node[left]+1)
                           #~
                            #~ if edge_length_swap < edge_length:
                                #~ l[index_of_node[n]] = left
                                #~ l[index_of_node[left]] = n
                                #~
                                #~ index_tmp = index_of_node[left]
                                #~ index_of_node[left] = index_of_node[n]
                                #~ index_of_node[n] = index_tmp
                            cross_count = 0
                            cross_count_swap = 0
                            for s in suc:
                                cross_count += len(
                                    [x for x in left_suc
                                     if index_of_node[x] > index_of_node[s]])
                                cross_count_swap += len(
                                    [x for x in left_suc
                                     if index_of_node[x] < index_of_node[s]])

                            # swap nodes if it results in less crossings
                            if cross_count_swap < cross_count:

                                l[index_of_node[n]] = left
                                l[index_of_node[left]] = n

                                index_tmp = index_of_node[left]
                                index_of_node[left] = index_of_node[n]
                                index_of_node[n] = index_tmp

                        l_i += 1

                # swap nodes with None neighbours
                for i in range(2):  # 6
                    break_flag = True
                    l_i = 0  # list index
                    for l in nodes_in_level:
                        for kk in range(len(l)//2):
                            break_flag = True
                            for n in l[:]:
                                if n is None:
                                    continue
                                i_n = l.index(n)
                                if i_n == 0:  # first element
                                    left is None
                                else:
                                    left = l[i_n-1]
                                if i_n == len(l)-1:  # last element
                                    right = None
                                else:
                                    right = l[i_n+1]
                                suc = [x for x in graph.successors(n)
                                       if level_of_node[x]-l_i < 2]
                                if left is not None and right is not None:
                                    continue

                                mean_neighbour_index = 0  # nodes_tmp.index(n)
                                count = 0.0
                                for x in multi_dep_neighbours[n]:
                                    if True:  # x in successors_of_node[n]:
                                        if abs(level_of_node[n] -
                                               level_of_node[x]) < 2:
                                            mean_neighbour_index += \
                                                nodes_in_level[
                                                    level_of_node[x]].index(x)
                                            count += 1
                                if count == 0:
                                    continue
                                mean_neighbour_index /= count

                                # swap nodes for being closer to
                                # mean_neighbour_index
                                if (mean_neighbour_index < i_n-.5 and
                                        left is None):
                                    break_flag = False
                                    l[i_n] = None
                                    l[i_n-1] = n
                                    index_of_node[n] = i_n - 1
                                elif (mean_neighbour_index > i_n+.5 and
                                        right is None):
                                    break_flag = False
                                    l[i_n] = None
                                    if i_n+1 >= len(l):
                                        l.append(n)
                                    else:
                                        l[i_n+1] = n
                                    index_of_node[n] = i_n + 1

                            if break_flag is True:
                                break

                        l_i += 1

                    if break_flag is True:
                        break

            if self.global_tasks_in_first_row is True:
                for n in g.nodes():
                    if level_of_node[n] != 0:
                        if len(predecessors_of_node[n]) == 0:
                            nodes_in_level[level_of_node[n]].remove(n)
                            #nodes_in_level[0].insert(n,index_of_node[n])
                            nodes_in_level[0].append(n)
                            level_of_node[n] = 0
                for n in nodes_in_level[0]:
                    if n is not None:
                        index_of_node[n] = nodes_in_level[0].index(n)

            #~ # calculate total length of edges and number of crossings
            #~ l_i = 0
            #~ for l in nodes_in_level:
                #~ for n in l[1:]:
                    #~ if n is None:
                        #~ continue
                    #~ left = l[index_of_node[n]-1]
                    #~ if left is None:
                        #~ continue
                    #~ suc = successors_of_node[n]
                    #~ left_suc = successors_of_node[left]
                    #~ for s in suc:
                        #~ total_cross_count += len(
                        #~    [x for x in left_suc
                        #~     if index_of_node[x] > index_of_node[s]])
                        #~ total_edge_length += abs(
                        #~    index_of_node[x]-index_of_node[n])
                #~ l_i += 1

            #~ for l in nodes_in_level:
                #~ #for n in nodes_in_level[l]:
                #~ for n in l:
                    #~ if isinstance(n, str):
                        #~ nodes_in_level[l].remove(n)
            #~ g.remove_nodes_from([x for x in g if isinstance(x, str)])

            # build layout

            if nodes_in_level[0].count(None) == len(nodes_in_level[0]):
                offset = 1
            else:
                offset = 0
            for l_i in range(number_of_levels):
                for n in nodes_in_level[l_i]:
                    if n is None:  # or isinstance(n, str):
                        continue
                    x = index_of_node[n] * self.node_separation
                    y = (-l_i + offset) * self.node_separation
                    layout_tmp[n] = (x, y)

            height_list[layout_i] = number_of_levels
            layout_list.append(layout_tmp)

        # t1 = (time.time()-t0)*1e3

        return (layout_list, width_list, height_list)
        #return (layout_list,t1,total_cross_count,total_edge_length,
        #        max_level_len,number_of_levels)

    def __draw_edges_linear(self, cr, edges, nodes):
        cr_move_to = cr.move_to
        cr_line_to = cr.line_to
        cr_set_source_rgba = cr.set_source_rgba

        cr.set_font_size(1)

        graph = self.graph
        # get rid of multiple edges to iterate through them later
        edges = list(set(edges))

        #~ print nodes
        nodes.sort(key=lambda n: graph._funcId[n])
        #~ print nodes

        inx = range(len(nodes))
        inx.sort(key=lambda n: nodes[n])
        #~ print inx
        #~ print

        #~ inx = range(len(edges))
        #~ inetworkx.sort(key=lambda r: graph.funcId[edges[r][0]])
        #~ for i in inx:
            #~ print i, graph.funcId[edges[i][0]]
        #edges = [edges[i] for i in inx]
        #
        #edges.sort(key=lambda r: graph.funcId[r[0]])

        # DEBUG
        #~ cr_set_source_rgba(0.0, 0.0, 0.0, 1.0)
        #~ cr_move_to(0, 0)
        #~ cr_line_to(100,200)
        #~ cr.stroke()
        #~ print "hu"

        # manage layout direction
        #if (self.layout_direction == 'b2t' or
        #        self.layout_direction == 'r2l'):
        #    inv = -1
        #else:
        #    inv = +1

        if (self.layout_direction == 'l2r' or
                self.layout_direction == 'r2l'):
            hori = 1
            vert = 0
        else:
            hori = 0
            vert = 1

        cr.set_line_width(self.edge_width*.3)

        for n in nodes:

            x0 = vert * inx[n-1] + hori * 0
            y0 = hori * inx[n-1] + vert * 0
            x3 = vert * inx[n-1] + hori * len(nodes)
            y3 = hori * inx[n-1] + vert * len(nodes)
            x0 *= 2
            y0 *= 2
            x3 *= 2
            y3 *= 2

            cr_set_source_rgba(0.0, 0.0, 1.0, 1.0)
            cr_move_to(x0, y0)
            cr_line_to(x3, y0)
            cr.stroke()

            cr_set_source_rgba(0.7, 0.7, 0.7, 1.0)
            cr_move_to(x0-10, y0)
            cr.show_text(str(n))
            cr_move_to(x3+10, y0)
            cr.show_text(str(n))

            cr.stroke()

            (r, g, b) = self.__palette_rgb(graph._funcId[n])
            cr_set_source_rgba(r, g, b, 1.0)
            cr.arc(x0-5, y0, 1, 0, 6.2832)

            cr.stroke()

        for e in edges:
#        for i in inx:

            # indices, number of lines and coordinates
            e_0 = e[0]
            e_1 = e[1]
            #~ e_0 = edges[i][0]
            #~ e_1 = edges[i][1]

            #~ n_e = len(graph[e_0][e_1])
            (r, g, b) = self.__palette_rgb(graph._funcId[e_0])
            cr_set_source_rgba(r, g, b, 0.5)

            x0 = vert * inx[e_0-1] + hori * 0
            y0 = hori * inx[e_0-1] + vert * 0
            x3 = vert * inx[e_1-1] + hori * len(nodes)
            y3 = hori * inx[e_1-1] + vert * len(nodes)
            x0 *= 2
            y0 *= 2
            x3 *= 2
            y3 *= 2

            cr.set_line_width(self.edge_width)
            cr_move_to(x0, y0)
            cr_line_to(x3, y3)
            cr.stroke()

        self.height = hori * graph.order() + vert * len(nodes) * 1.5
        self.width = vert * graph.order() + hori * len(nodes) * 1.5
        self.x_c = hori * len(nodes) * 0.5 + vert * 0.5 * graph.order()
        self.y_c = -vert * len(nodes) * 0.5 - hori * 0.5 * graph.order()

        self.height *= 2
        self.width *= 2
        self.x_c *= 2
        self.y_c *= 2

    def __draw_edges(self, cr, edges):

        graph = self.graph

        cr_move_to = cr.move_to
        cr_line_to = cr.line_to
        cr_curve_to = cr.curve_to
        layout = self.layout
        if self.addr_key == 'addr':
            s_a_k_index = self.graph.addresses.keys().index
        elif self.addr_key == 'orig_addr':
            s_a_k_index = self.graph.orig_addresses.keys().index
        cr_set_source_rgba = cr.set_source_rgba
        sep = self.multdep_separation

        # manage layout direction
        if (self.layout_direction == 'b2t' or
                self.layout_direction == 'r2l'):
            inv = -1
        else:
            inv = +1

        if (self.layout_direction == 'l2r' or
                self.layout_direction == 'r2l'):
            hori = 1
            vert = 0
        else:
            hori = 0
            vert = 1

        # get rid of multiple edges to iterate through them later
        edges = set(edges)

        if self.addr_key != 'nothing':

            cr.set_font_size(self._edge_label_fontsize)
            self._edge_label_width = cr.text_extents('0x0123456789abc')[2]

            for e in edges:

                # indices, number of lines and coordinates
                e_0 = e[0]
                e_1 = e[1]
                n_e = len(graph[e_0][e_1])

                #print e_0, e_1, n_e

                x0 = layout[e_0][0] + hori * inv * self.node_size
                y0 = -layout[e_0][1] + vert * inv * self.node_size
                x3 = layout[e_1][0] - hori * inv * self.node_size
                y3 = -layout[e_1][1] - vert * inv * self.node_size

#                x1 = x0 + hori * inv * 20
#                y1 = y0 + vert * inv * 20
#                x2 = x3 - hori * inv * 20
#                y2 = y3 - vert * inv * 20

                # colours and linewidth
                col_list = []
                if (graph._status[e_0] == 3 and
                        (self.margins_mean == 'status' or
                            self.node_colours == 'status')):
                    # greyed out lines
                    for ee in graph[e_0][e_1]:
                        if graph[e_0][e_1][ee][self.addr_key] != 0:
                            # set colour for memory address
                            addr_i = s_a_k_index(
                                graph[e_0][e_1][ee][self.addr_key])
                            colour_i = self.hash_id(addr_i)
                            col_list.append((self.palette_edges[colour_i][0],
                                             self.palette_edges[colour_i][1],
                                             self.palette_edges[colour_i][2],
                                             0.3))
                        else:
                            # set stroke colour to grey, no address label
                            col_list.append((0.8, 0.8, 0.8, .9))
                            #cr_set_source_rgba(0.8, 0.8, 0.8, 1.0)
                            #cr.set_line_width(1)

                else:
                    # strong colours
                    for ee in graph[e_0][e_1]:
                        if graph[e_0][e_1][0][self.addr_key] != 0:
                            # set colour for memory address
                            addr_i = s_a_k_index(
                                graph[e_0][e_1][ee][self.addr_key])
                            colour_i = self.hash_id(addr_i)
                            col_list.append((self.palette_edges[colour_i][0],
                                             self.palette_edges[colour_i][1],
                                             self.palette_edges[colour_i][2],
                                             0.9))
                        else:
                            # set stroke colour to black, no address label
                            col_list.append((0.0, 0.0, 0.0, .9))
                            #cr.set_source_rgba(0.0, 0.0, 0.0, 1.0)
                            #cr.set_line_width(1)

                # draw lines
                cr.set_line_width(self.edge_width)
                self.__draw_edge_lines(cr.stroke,
                                     cr_set_source_rgba,
                                     cr_move_to,
                                     cr_line_to,
                                     cr_curve_to,
                                     n_e,
                                     x0,
                                     x3,
                                     y0,
                                     y3,
                                     hori,
                                     vert,
                                     sep,
                                     col_list)

                if self.draw_edge_labels is True:
                    if graph[e_0][e_1][ee][self.addr_key] != 0:
                        cr.set_line_width(self._edge_label_fontsize + 2*self._edge_label_margin)
                        self.__draw_label_background(cr.stroke,
                                                   cr_set_source_rgba,
                                                   cr_move_to,
                                                   cr_line_to,
                                                   n_e,
                                                   x0,
                                                   x3,
                                                   y0,
                                                   y3,
                                                   hori,
                                                   vert,
                                                   col_list)
                        cr.set_line_width(self.edge_width)

        else:
            # same colour and linewidth for every line
            cr.set_line_width(self.edge_width)
            cr_set_source_rgba(0.0, 0.0, 0.0, 1.0)

            for e in edges:

                # indices, number of lines and coordinates
                e_0 = e[0]
                e_1 = e[1]
                n_e = len(graph[e_0][e_1])

                x0 = layout[e_0][0] + hori * inv * self.node_size
                y0 = -layout[e_0][1] + vert * inv * self.node_size
                x3 = layout[e_1][0] - hori * inv * self.node_size
                y3 = -layout[e_1][1] - vert * inv * self.node_size

                # draw
                if n_e == 1:
                # straight line
                    cr_move_to(x0, y0)
                    cr_line_to(x3, y3)
                elif n_e == 2:
                    # first dependency
                    x0 -= vert*sep/2
                    y0 -= hori*sep/2
                    x3 -= vert*sep/2
                    y3 -= hori*sep/2
                    cr_move_to(x0, y0)
                    cr_line_to(x3, y3)
                    # second dependency
                    x0 += vert*sep
                    y0 += hori*sep
                    x3 += vert*sep
                    y3 += hori*sep
                    cr_move_to(x0, y0)
                    cr_line_to(x3, y3)
                elif n_e > 2:
                    # first dependency
                    x0 -= vert*sep
                    y0 -= hori*sep
                    x3 -= vert*sep
                    y3 -= hori*sep
                    cr_move_to(x0, y0)
                    cr_line_to(x3, y3)
                    # second dependency
                    x0 += vert*sep
                    y0 += hori*sep
                    x3 += vert*sep
                    y3 += hori*sep
                    cr_move_to(x0, y0)
                    cr_line_to(x3, y3)
                    # third dependency
                    x0 += vert*sep
                    y0 += hori*sep
                    x3 += vert*sep
                    y3 += hori*sep
                    cr_move_to(x0, y0)
                    cr_line_to(x3, y3)

            cr.stroke()
#                cr.curve_to(x1, y1, x2, y2, x3, y3)

    def __draw_dependentEdges(self, cr):
        # manage layout direction
        if (self.layout_direction == 'b2t' or
                self.layout_direction == 'r2l'):
            inv = -1
        else:
            inv = +1

        if (self.layout_direction == 'l2r' or
                self.layout_direction == 'r2l'):
            hori = 1
            vert = 0
        else:
            hori = 0
            vert = 1

        #draw curves
        for t1 in self.graph.dependentNodes:
            for t2 in self.graph.dependentNodes[t1]:
                x0 = self.layout[t1][0] + hori * inv * self.node_size
                y0 = -self.layout[t1][1] + vert * inv * self.node_size
                x3 = self.layout[t2][0] - hori * inv * self.node_size
                y3 = -self.layout[t2][1] - vert * inv * self.node_size
                x1 = x0 + hori * inv * (old_div(abs(x3-x0),2, expect_integer=True) + self.node_size)
                y1 = y0 + vert * inv * (old_div(abs(y3-y0),2, expect_integer=True) + self.node_size)
                x2 = x3 - hori * inv * (old_div(abs(x3-x0),2, expect_integer=True) + self.node_size)
                y2 = y3 - vert * inv * (old_div(abs(y3-y0),2, expect_integer=True) + self.node_size)
                if vert == 1 and x0 == x3:
                    x1 += inv * self.node_size*3
                    x2 -= inv * self.node_size*3
                if hori == 1 and y0 == y3:
                    y1 -= inv * self.node_size*3
                    y2 += inv * self.node_size*3

                cr.move_to(x0, y0)
                cr.curve_to(x1, y1, x2, y2, x3, y3)
        cr.set_source_rgba(.5, .5, .5, .5)
        cr.set_line_width(self.edge_width*5)
        cr.stroke()
        cr.set_line_width(self.edge_width)

    def __draw_label_background(self,
                              cr_stroke,
                              cr_set_source_rgba,
                              cr_move_to,
                              cr_line_to,
                              n_e,
                              x0,
                              x3,
                              y0,
                              y3,
                              hori,
                              vert,
                              colours):
        # background for label
        # label width
        lw = self._edge_label_width + 2*self._edge_label_margin
        y_skip = self._edge_label_fontsize+2*self._edge_label_margin       # fontsize plus 2 times margin
        y_offset = -self._edge_label_fontsize//2 + self._edge_label_margin  # up half a linewidth (=fontsize) and one margin
        y_offset += -n_e*y_skip/2. + y_skip/2.                             # center whole label group on edge
        for ee in range(n_e):
            cr_move_to(old_div((x0+x3),2, expect_integer=True) - \
                       old_div(lw,2, expect_float=True), \
                       old_div((y0+y3),2, expect_integer=True) + ee*y_skip + y_offset)
            cr_line_to(old_div((x0+x3),2, expect_integer=True) + \
                       old_div(lw,2, expect_float=True), \
                       old_div((y0+y3),2, expect_integer=True) + ee*y_skip + y_offset)
            cr_set_source_rgba(colours[ee][0],
                               colours[ee][1],
                               colours[ee][2],
                               colours[ee][3])
            cr_stroke()

    def __draw_edge_lines(self,
                        cr_stroke,
                        cr_set_source_rgba,
                        cr_move_to,
                        cr_line_to,
                        cr_curve_to,
                        n_e,
                        x0,
                        x3,
                        y0,
                        y3,
                        hori,
                        vert,
                        sep,
                        colours):

        if n_e == 1:
        # straight line
            cr_move_to(x0, y0)
            if vert == 1 and x0 == x3 and abs(y3-y0) > self.node_separation:
                cr_curve_to(x0+10, y0, x3+10, y3, x3, y3)
            elif hori == 1 and y0 == y3 and abs(x3-x0) > self.node_separation:
                cr_curve_to(x0, y0-10, x3, y3-10, x3, y3)
            else:
                cr_line_to(x3, y3)
            cr_set_source_rgba(colours[0][0],
                               colours[0][1],
                               colours[0][2],
                               colours[0][3])
            cr_stroke()
        elif n_e == 2:
            # first dependency
            x0 -= vert*sep/2
            y0 -= hori*sep/2
            x3 -= vert*sep/2
            y3 -= hori*sep/2
            cr_move_to(x0, y0)
            if vert == 1 and x0 == x3 and abs(y3-y0) > self.node_separation:
                cr_curve_to(x0+10, y0, x3+10, y3, x3, y3)
            elif hori == 1 and y0 == y3 and abs(x3-x0) > self.node_separation:
                cr_curve_to(x0, y0-10, x3, y3-10, x3, y3)
            else:
                cr_line_to(x3, y3)
            cr_set_source_rgba(colours[0][0],
                               colours[0][1],
                               colours[0][2],
                               colours[0][3])
            cr_stroke()
            # second dependency
            x0 += vert*sep
            y0 += hori*sep
            x3 += vert*sep
            y3 += hori*sep
            cr_move_to(x0, y0)
            if vert == 1 and x0 == x3 and abs(y3-y0) > self.node_separation:
                cr_curve_to(x0+10, y0, x3+10, y3, x3, y3)
            elif hori == 1 and y0 == y3 and abs(x3-x0) > self.node_separation:
                cr_curve_to(x0, y0-10, x3, y3-10, x3, y3)
            else:
                cr_line_to(x3, y3)
            cr_set_source_rgba(colours[1][0],
                               colours[1][1],
                               colours[1][2],
                               colours[1][3])
            cr_stroke()
        elif n_e > 2:
            # first dependency
            x0 -= vert*sep
            y0 -= hori*sep
            x3 -= vert*sep
            y3 -= hori*sep
            cr_move_to(x0, y0)
            if vert == 1 and x0 == x3 and abs(y3-y0) > self.node_separation:
                cr_curve_to(x0+10, y0, x3+10, y3, x3, y3)
            elif hori == 1 and y0 == y3 and abs(x3-x0) > self.node_separation:
                cr_curve_to(x0, y0-10, x3, y3-10, x3, y3)
            else:
                cr_line_to(x3, y3)
            cr_set_source_rgba(colours[0][0],
                               colours[0][1],
                               colours[0][2],
                               colours[0][3])
            cr_stroke()
            # second dependency
            x0 += vert*sep
            y0 += hori*sep
            x3 += vert*sep
            y3 += hori*sep
            cr_move_to(x0, y0)
            if vert == 1 and x0 == x3 and abs(y3-y0) > self.node_separation:
                cr_curve_to(x0+10, y0, x3+10, y3, x3, y3)
            elif hori == 1 and y0 == y3 and abs(x3-x0) > self.node_separation:
                cr_curve_to(x0, y0-10, x3, y3-10, x3, y3)
            else:
                cr_line_to(x3, y3)
            cr_set_source_rgba(colours[1][0],
                               colours[1][1],
                               colours[1][2],
                               colours[1][3])
            cr_stroke()
            # third dependency
            x0 += vert*sep
            y0 += hori*sep
            x3 += vert*sep
            y3 += hori*sep
            cr_move_to(x0, y0)
            if vert == 1 and x0 == x3 and abs(y3-y0) > self.node_separation:
                cr_curve_to(x0+10, y0, x3+10, y3, x3, y3)
            elif hori == 1 and y0 == y3 and abs(x3-x0) > self.node_separation:
                cr_curve_to(x0, y0-10, x3, y3-10, x3, y3)
            else:
                cr_line_to(x3, y3)
            cr_set_source_rgba(colours[2][0],
                               colours[2][1],
                               colours[2][2],
                               colours[2][3])
            cr_stroke()
#                cr.curve_to(x1, y1, x2, y2, x3, y3)

    def __get_col_funcId(self, n, funcId, threadId, status):
        # wait on tasks
        if funcId == client_conf.waiton_func_id:
            return (0.0, 0.0, 0.0)
        # if funcId unknown
        if funcId not in properties.propFunc:
            return (0.82, 0.82, 0.82)
        # otherwise
        return self.palette_nodes[self.hash_id(properties.propFunc.index(funcId))]

    def __get_col_taskduration(self, n, funcId, threadId, status):

        # wait on tasks
        if funcId == client_conf.waiton_func_id:
            return (0.0, 0.0, 0.0)

        # unfinished tasks are white
        if status != 3:
            return (1.0, 1.0, 1.0)
        # otherwise
        dt = self.graph.node[n]['ts_f']-self.graph.node[n]['ts_r']
        return palette_rgb_g2r(math.log(dt-self.dt_min+1)+.001,
                                   math.log(self.dt_max-self.dt_min+1)+.001)

    def __get_col_disttotask(self, n, funcId, threadId, status):

        # wait on tasks
        if funcId == client_conf.waiton_func_id:
            return (0.0, 0.0, 0.0)

        # if marked task is not connected to any other
        if self.dist_max == 0:
            return (1.0, 1.0, 1.0)

        # if dict self.distance_to_marked is empty
        if len(self.distance_to_marked) == 0:
            return (1.0, 1.0, 1.0)

        # if distance of n is not known
        if n not in self.distance_to_marked:
            return (1.0, 1.0, 1.0)

        # unconnected tasks and the marked task are white
        if (self.distance_to_marked[n] is None or
                self.distance_to_marked[n] == 0):
            return (1.0, 1.0, 1.0)
        # otherwise
        return palette_rgb_g2r(
            abs(self.distance_to_marked[n]), self.dist_max)

    def get_dtminmax(self, nodes=None):

        if nodes is None:
            nodes = self.graph.nodes()
        for n in nodes:
            if self.graph._status[n] != 3:
                continue
            dt = self.graph.node[n]['ts_f']-self.graph.node[n]['ts_r']
            if dt > self.dt_max:
                self.dt_max = dt
            if dt < self.dt_min:
                self.dt_min = dt
        return self.dt_min, self.dt_max

    def __get_None(self, n, funcId, threadId, status):

        return None

    def __get_col_threadId(self, n, funcId, threadId, status):

        # wait on tasks
        if funcId == client_conf.waiton_func_id:
            return (0.0, 0.0, 0.0)

        # tasks with no assigned thread are white
        if threadId == -1 or threadId is None:
            return (1.0, 1.0, 1.0)
        # otherwise
        return self.palette_nodes[self.hash_id(threadId)]

    def __get_col_status(self, n, funcId, threadId, status):
        # wait on tasks
        if funcId == client_conf.waiton_func_id:
            return (0.0, 0.0, 0.0)

        if status == 0:  # not queued
            return (1.0, 0.0, 0.0)
        elif status == 1:  # queued
            return (1.0, 1.0, 0.0)
        elif status == 2:  # running
            return (0.0, 1.0, 0.0)
        elif status == 4:  # failed
            return (0.0, 0.0, 0.0)

    def __get_shape_none(self, funcId, threadId):
        return None

    def __get_shape_threadId(self, funcId, threadId):
        if threadId == client_conf.no_thread_id or threadId > 12:
            return None
        if threadId == self.commthreadId:
            return 'arrow'
        return self.shape_list[threadId]

    def __get_shape_funcId(self, funcId, threadId):
        # wait on tasks
        if funcId == client_conf.waiton_func_id:
            return None

        if funcId > 12:
            return None
        return self.shape_list[funcId]

    def __draw_nodes(self, cr, nodes):

        # what should the node colour mean
        if self.node_colours == 'function':
            get_fill_col = self.__get_col_funcId
        elif self.node_colours == 'task_duration':
            get_fill_col = self.__get_col_taskduration
            self.get_dtminmax(nodes)
        elif self.node_colours == 'distance_to_marked':
            get_fill_col = self.__get_col_disttotask
        elif self.node_colours == 'status':
            get_fill_col = self.__get_col_status
        elif self.node_colours == 'thread':
            get_fill_col = self.__get_col_threadId
        elif self.node_colours == 'nothing':
            get_fill_col = self.__get_None

        # what should the node shape mean

        if self.shapes_mean == 'nothing':
            get_shape = self.__get_shape_none
        elif self.shapes_mean == 'thread':
            get_shape = self.__get_shape_threadId
        elif self.shapes_mean == 'function':
            get_shape = self.__get_shape_funcId

        # what should the node margin colour mean

        if self.margins_mean == 'status':
            get_marg_col = self.__get_col_status
        elif self.margins_mean == 'thread':
            get_marg_col = self.__get_col_threadId
        elif self.margins_mean == 'function':
            get_marg_col = self.__get_col_funcId
        elif self.margins_mean == 'task_duration':
            get_marg_col = self.__get_col_taskduration
            self.get_dtminmax(nodes)
        elif self.margins_mean == 'nothing':
            get_marg_col = self.__get_None

        for n in nodes:
            threadId = self.graph._threadId[n]
            status = self.graph._status[n]
            funcId = self.graph._funcId[n]
            #taskId = self.graph._taskId[n]

            ### set properties

            # fill colour for node
            fill_col = get_fill_col(n, funcId, threadId, status)
            # special treatment for waitons
            if funcId == client_conf.waiton_func_id:
                fill_col = self.__get_col_funcId(n, funcId, threadId, status)

            # coloured extra margins for not queued, queued,
            # running and failed tasks

            shape = get_shape(funcId, threadId)

            # coordinates and shape path
            x0 = self.layout[n][0]
            y0 = -self.layout[n][1]
            r = self.node_size
            points = self.__get_points_of_shape(shape, x0, y0, self.node_size)

            ### draw

            # filled shape
            if fill_col is not None:
                self.__draw_shape(cr, points, fill_col)

            # simple black margin
            # if no status margin or not finished or
            # margin and node colour indicates status

            #if (self.margins_mean != 'status' or
            #        status != 3 or
            #        (self.margins_mean == 'status' and
            #            self.node_colours == 'status') or
            #        self.node_colours == 'nothing'):

            # or just always draw the margin...
            if True:
                self.__draw_margin(cr, points, (0, 0, 0), 1)

                # colourful margin for further information
                marg_col = get_marg_col(n, funcId, threadId, status)

                # don't draw margin if task is wait on task
                if (marg_col is not None and
                        funcId != client_conf.waiton_func_id):
                    # coloured margin depending on status
                    points = self.__get_points_of_shape(
                        shape, x0, y0,
                        self.node_size+4+0.5*self.margin_width*1.414)
                    self.__draw_margin(cr, points, marg_col, self.margin_width)

            # blue margin to mark neighbours
            if self.graph._highlight[n] == 1:
                points = self.__get_points_of_shape(
                    shape, x0, y0, self.node_size+2*self.margin_width*1.414)
                self.__draw_margin(cr, points, (0, 0, 1), self.margin_width)

            # light blue margin to mark clicked node
            if self.graph._highlight[n] == 2:
                points = self.__get_points_of_shape(
                    shape, x0, y0, self.node_size+1.5*self.margin_width*1.414)
                self.__draw_margin(
                    cr, points, (0.5, 0.5, 1), .5*self.margin_width)

                points = self.__get_points_of_shape(
                    shape, x0, y0, self.node_size+2.5*self.margin_width*1.414)
                self.__draw_margin(
                    cr, points, (0.5, 0.5, 1), .7*self.margin_width)

            if self.show_markers is True:
                # thick red cross when blocked
                if n in self.graph.blockedNodes:
                    cr.set_source_rgba(.7, 0., 0.2, 1.)
                    cr.set_line_width(self.node_size * 0.5)
                    cr.move_to(x0-r, y0-r)
                    cr.line_to(x0+r, y0+r)
                    cr.move_to(x0-r, y0+r)
                    cr.line_to(x0+r, y0-r)
                    cr.stroke()

                # thick green cross when critical
                if n in self.graph.criticalNodes:
                    cr.set_source_rgba(0., .7, 0.3, 1.)
                    cr.set_line_width(self.node_size * 0.4)
                    cr.move_to(x0-r*1.3, y0)
                    cr.line_to(x0+r*1.3, y0)
                    cr.move_to(x0, y0+r*1.3)
                    cr.line_to(x0, y0-r*1.3)
                    cr.stroke()
                    #~ cr.set_source_rgba(0.,.6, 0., 1.)
                    #~ cr.set_line_width(4)
                    #~ cr.move_to(x0-r*.6, y0)
                    #~ cr.line_to(x0+r*.6, y0)
                    #~ cr.move_to(x0, y0+r*.6)
                    #~ cr.line_to(x0, y0-r*.6)
                    #~ cr.stroke()

    def __draw_labels(self, cr, nodes, edges):

        cr_set_source_rgba = cr.set_source_rgba
        cr_move_to = cr.move_to
        cr_show_text = cr.show_text

        ### node labels

        # set font size
        #self.font.set_absolute_size(self.node_size/2.0)
        #cr_layout.set_font_description(self.font)
        cr.select_font_face("sans-serif",
                            cairo.FONT_SLANT_NORMAL,
                            cairo.FONT_WEIGHT_BOLD)
        cr.set_font_size(self._node_label_fontsize)
        

        for n in nodes:
            # coordinates, text, colour
            x0 = self.layout[n][0]
            y0 = -self.layout[n][1]

            cr_set_source_rgba(0.0, 0.0, 0.0, 1.0)  # black

            # if (indicate status and finished) or waiton task
            if (self.node_colours != 'nothing' and
                    (self.margins_mean == 'status' and
                        self.graph._status[n] == 3 and
                        self.node_colours != 'status' and
                        self.node_colours != 'distance_to_marked') or
                    self.graph._funcId[n] == client_conf.waiton_func_id):

                    cr_set_source_rgba(1.0, 1.0, 1.0, 1.0)  # white

            # set text
            t = str(self.graph._taskId[n])
            x_bearing, y_bearing, width, height = cr.text_extents(t)[:4]
            cr_move_to(x0 - width / 2 - x_bearing, y0 - height / 2 - y_bearing)

            cr_show_text(t)

            # if finished and indicate task duration
            if (self.graph._status[n] == 3 and
                    (self.margins_mean == 'task_duration' or
                     self.node_colours == 'task_duration')):
                cr_set_source_rgba(0.3, 0.3, 1.0, 1.0)  # light blue
                t_ts = str(self.graph.node[n]['ts_f'] -
                           self.graph.node[n]['ts_r'])
                x_bearing, y_bearing, width, height = cr.text_extents(t_ts)[:4]
                cr_move_to(x0 - width / 2 - x_bearing ,
                           y0 - height / 2 - y_bearing - \
                           self.node_size//2)
                cr_show_text(t_ts)

        ### edge labels
        #self.font.set_absolute_size(5.0)
        #cr_line_to = cr.line_to
        cr.set_font_size(self._edge_label_fontsize)
        cr_set_source_rgba(0.0, 0.0, 0.0, 1.0)
        layout = self.layout

        # manage layout direction
        if (self.layout_direction == 'b2t' or
                self.layout_direction == 'r2l'):
            inv = -1
        else:
            inv = +1

        if (self.layout_direction == 'l2r' or
                self.layout_direction == 'r2l'):
            hori = 1
            vert = 0
        else:
            hori = 0
            vert = 1

        if self.draw_edge_labels is True:
            for e in edges:
                e_0 = e[0]
                e_1 = e[1]
                if (self.addr_key != 'nothing' and
                        self.graph[e_0][e_1][0][self.addr_key] != 0):

                    x_edge_0 = layout[e_0][0] + hori * inv * self.node_size
                    y_edge_0 = -layout[e_0][1] + vert * inv * self.node_size
                    x_edge_3 = layout[e_1][0] - hori * inv * self.node_size
                    y_edge_3 = -layout[e_1][1] - vert * inv * self.node_size
                    x0 = old_div((x_edge_0+x_edge_3),2, expect_integer=True)
                    y0 = old_div((y_edge_0+y_edge_3),2, expect_integer=True)
                    y_skip = self._edge_label_fontsize+2*self._edge_label_margin       # fontsize plus 2 times margin
                    y_offset = -len(self.graph[e_0][e_1])*y_skip/2. + y_skip/2.        # center whole label group on edge
                    for ee in self.graph[e_0][e_1]:
                        t = hex(self.graph[e_0][e_1][ee][self.addr_key])
                        x_bearing, y_bearing, width, height = cr.text_extents(t)[:4]
                        cr_move_to(x0 - width/2 - x_bearing, y0 + ee*y_skip + y_offset)
                        cr_show_text(t)

    def __get_points_of_shape(self, shape, x0, y0, r):

        p = [(x0, y0), r]  # default is circle

        if self.shapes_mean == 'nothing':
            return p

        if shape == 'arrow':
            p = [(x0+.4*r, y0+.9*r),
                 (x0-.4*r, y0+.9*r),
                 (x0-.4*r, y0-.2*r),
                 (x0-.8*r, y0-.2*r),
                 (x0, y0-r),
                 (x0+.8*r, y0-.2*r),
                 (x0+.4*r, y0-.2*r),
                 (x0+.4*r, y0+.9*r)]

        if shape == 'box':
            p = [(x0-r/1.414, y0+r/1.414),
                 (x0-r/1.414, y0-r/1.414),
                 (x0+r/1.414, y0-r/1.414),
                 (x0+r/1.414, y0+r/1.414)]
        if shape == 'trapezium':
            p = [(x0-1.3*r/1.414, y0+r/1.414),
                 (x0-0.7*r/1.414, y0-r/1.414),
                 (x0+0.7*r/1.414, y0-r/1.414),
                 (x0+1.3*r/1.414, y0+r/1.414)]
        if shape == 'invtrapezium':
            p = [(x0-0.7*r/1.414, y0+r/1.414),
                 (x0-1.3*r/1.414, y0-r/1.414),
                 (x0+1.3*r/1.414, y0-r/1.414),
                 (x0+0.7*r/1.414, y0+r/1.414)]
        if shape == 'diamon':
            p = [(x0, y0+r),
                 (x0-r, y0),
                 (x0, y0-r),
                 (x0+r, y0)]
        if shape == 'triangle':
            y0 -= .2*self.node_size
            r *= 1.24
            p = [(x0, y0+r),
                 (x0-r*.866, y0-r*.5),
                 (x0+r*.866, y0-r*.5)]
        if shape == 'invtriangle':
            y0 += .2*self.node_size
            r *= 1.24
            p = [(x0, y0-r),
                 (x0-r*.866, y0+r*.5),
                 (x0+r*.866, y0+r*.5)]
        if shape == 'pentago':
            p = [(x0+.7*r, y0+.8*r),
                 (x0-.7*r, y0+.8*r),
                 (x0-r, y0-.2*r),
                 (x0, y0-r),
                 (x0+r, y0-.2*r)]
        if shape == 'invpentago':
            p = [(x0+.7*r, y0-.8*r),
                 (x0-.7*r, y0-.8*r),
                 (x0-r, y0+.2*r),
                 (x0, y0+r),
                 (x0+r, y0+.2*r)]
        if shape == 'pentagra':
            p = [(x0+.7*r, y0+.8*r),
                 (x0-r, y0-.2*r),
                 (x0+r, y0-.2*r),
                 (x0-.7*r, y0+.8*r),
                 (x0, y0-r)]
        if shape == 'varpentagra':
            p = [(x0+.7*r, y0+.8*r),
                 (x0, y0+.4*r),
                 (x0-.7*r, y0+.8*r),
                 (x0-.45*r, y0+.2*r),
                 (x0-r, y0-.25*r),
                 (x0-.3*r, y0-.25*r),
                 (x0, y0-r),
                 (x0+.3*r, y0-.25*r),
                 (x0+r, y0-.25*r),
                 (x0+.45*r, y0+.2*r)]
        if shape == 'hexago':
            p = [(x0+.5*r, y0+.8*r),
                 (x0-.5*r, y0+.8*r),
                 (x0-r, y0),
                 (x0-.5*r, y0-.8*r),
                 (x0+.5*r, y0-.8*r),
                 (x0+r, y0)]
        if shape == 'house':
            p = [(x0+.8*r, y0+.8*r),
                 (x0-.8*r, y0+.8*r),
                 (x0-.8*r, y0-.2*r),
                 (x0, y0-r),
                 (x0+.8*r, y0-.2*r)]
        if shape == 'invhouse':
            p = [(x0+.8*r, y0-.8*r),
                 (x0-.8*r, y0-.8*r),
                 (x0-.8*r, y0+.2*r),
                 (x0, y0+r),
                 (x0+.8*r, y0+.2*r)]
        return p

    def __draw_shape(self, cr, points, fill_col):

        cr.set_source_rgba(fill_col[0], fill_col[1], fill_col[2], 1.0)
        if len(points) == 2:
            x0, y0 = points[0]
            cr.arc(x0, y0, points[1], 0, 6.2832)
        else:
            x0, y0 = points[-1]
            cr.move_to(x0, y0)
            for x, y in points:
                cr.line_to(x, y)
        cr.close_path()

#        cr.fill_preserve()
        cr.fill()

    def __draw_margin(self, cr, points, marg_col, marg_width):

        cr.set_source_rgba(marg_col[0], marg_col[1], marg_col[2], 1.0)
        if len(points) == 2:
            x0, y0 = points[0]
            cr.arc(x0, y0, points[1], 0, 6.2832)
        else:
            x0, y0 = points[-1]
            cr.move_to(x0, y0)
            for x, y in points:
                cr.line_to(x, y)
        cr.close_path()

        cr.set_dash(())
        cr.set_line_width(marg_width)
        cr.stroke()

    def __draw_barriers(self, cr, barriers):
        cr.select_font_face("sans-serif",
                            cairo.FONT_SLANT_NORMAL,
                            cairo.FONT_WEIGHT_BOLD)
        cr.set_font_size(self.node_size//2 +1)

        # these variables determine the start an end coordinate of the line
        if ( self.layout_direction == 't2b' or
             self.layout_direction == 'b2t' ):
            a1 = self.x_c - old_div(self.width, 2, expect_integer=True)
            a2 = self.x_c + old_div(self.width, 2, expect_integer=True)
        else:
            a1 = -self.y_c - old_div(self.height, 2, expect_integer=True)
            a2 = -self.y_c + old_div(self.height, 2, expect_integer=True)

        # draw lines and labels
        if self.layout_direction == 't2b':
            for b_i in range(len(barriers)):
                b = barriers[b_i]
                cr.move_to(a1, -b)
                cr.line_to(a2, -b)
                cr.move_to(a1, -b - self.node_size//2)
                cr.show_text(
                    "%d barriers" % self.count_consecutive_barriers[b_i])

        elif self.layout_direction == 'b2t':
            for b_i in range(len(barriers)):
                b = barriers[b_i]
                cr.move_to(a1, b)
                cr.line_to(a2, b)
                cr.move_to(a1 , b - self.node_size//2)
                cr.show_text(
                    "%d barriers" % self.count_consecutive_barriers[b_i])

        elif self.layout_direction == 'r2l':
            for b_i in range(len(barriers)):
                b = barriers[b_i]
                cr.move_to(b, a1)
                cr.line_to(b, a2)
                cr.move_to(b - self.node_size * 3, a1)
                cr.show_text(
                    "%d barriers" % self.count_consecutive_barriers[b_i])

        elif self.layout_direction == 'l2r':
            for b_i in range(len(barriers)):
                b = barriers[b_i]
                cr.move_to(-b, a1)
                cr.line_to(-b, a2)
                cr.move_to(-b - self.node_size * 3, a1 )
                cr.show_text(
                    "%d barriers" % self.count_consecutive_barriers[b_i])

        cr.set_source_rgba(0., 0., 0., 1.)
        cr.set_line_width(2)
        cr.stroke()
        cr.set_line_width(1)

    def get_nodeId_from_xy(self, x, y):
        for node in self.graph.nodes():
            if self.__square_distance(x,
                               y,
                               self.layout[node][0],
                               -self.layout[node][1]) < \
                    self.node_size*self.node_size:
                return node
        return None

    def calc_all_distances(self, node):
        if node is None:
            self.distance_to_marked = {}
            self.dist_max = 0
        for n in self.graph.nodes():
            dist = self.graph.distance(n, node)
            self.distance_to_marked[n] = dist
            if dist is not None and abs(dist) > self.dist_max:
                self.dist_max = abs(dist)

#****f* TEMANEJO_GRAPH/__palette_rgb_inf
# NAME
#   __palette_rgb_inf
#   - palette used for node colouring (infinitely)
# DESCRIPTION
#   This function returns a tuple of RGB values depending on the integer
#   input value. The colours will be different enough to be distinguished
#   easily on screen. No information about the total count of input values
#   is needed.
# INPUTS
#   * integer value i (as index)
#   * floating point value s (as saturation) defaults to 1.0
# OUTPUT
#   RGB tuple (r,g, b)

    def __palette_rgb_inf(self, i, s=1.0):
        #i += .1
        h = i/2.7345
        v = 1.  # -int(h % 100000) *.000013
        h -= int(h)
    #    print v,h
    #    return "\""+ str(h)+" "+str(s)+" "+str(v)+"\""
        rgb = colorsys.hsv_to_rgb(h, s, v)
        #for integer values [0,255]
    #    rgb = map(lambda a: int(255*a),rgb)
    #    rgb = [int(255*a) for a in rgb]
        return tuple(rgb)



#****f* TEMANEJO_GRAPH/__palette_rgb
# NAME
#   __palette_rgb
#   - palette used for node colouring (infinitely)
# DESCRIPTION
#   This function returns a tuple of RGB values depending on the integer
#   input value. The colours will be different enough to be distinguished
#   easily on screen. No information about the total count of input values
#   is needed. Will call __palette_rgb_inf.
# INPUTS
#   * integer value i (as index)
#   * floating point value s (as saturation) defaults to 1.0
# OUTPUT
#   RGB tuple (r,g, b)


    def __palette_rgb(self, i, s=1.0):

        return self.__palette_rgb_inf(i, s)

        # if i == 0:
        #     return (0, 1, 0)
        # if i == 1:
        #     return (1, 1, 0)
        # if i == 2:
        #     return (1, 0, 0)
        # if i == 3:
        #     return (0, 1, 1)
        # if i == 4:
        #     return (1, 0, 1)
        # if i == 5:
        #     return (1, 0.5, 0)
        # if i == 6:
        #     return (.75, 1, 0)
        # if i == 7:
        #     return (.5, 0, 1)
        # if i == 8:
        #     return (1, 0,.5)
        # if i == 9:
        #     return (0, 0.5, 1)
        # if i == 10:
        #     return (0, 1, 0.5)
        # if i == 11:
        #     return (.5, 0, 0)
        # if i == 12:
        #     return (0, 0.5, 0)
        # if i == 13:
        #     return (0, 0,.5)
        # if i == 14:
        #     return (0, 0, 1)

        # return __palette_rgb_inf(i-14, s)
        
#****f* TEMANEJO_GRAPH/square_distance
# NAME
#   square_distance
#   - returns the square distance of two points (2 dimensions)
# DESCRIPTION
#   This function returns the square distance of two points (x1, y1) and
#   (x2, y2)
# INPUTS
#   floating point values x1, y1, x2, y2
# OUTPUT
#   floating point value of distance = (x2-x1)^2+(y2-y1)^2
# SOURCE


    def __square_distance(self, x1, y1, x2, y2):
        deltax = x2 - x1
        deltay = y2 - y1
        return deltax*deltax + deltay*deltay


#******

#****f* TEMANEJO_GRAPH/palette_rgb_g2r
# NAME
#   palette_rgb_g2r
#   - palette used for node colouring (green to red)
# DESCRIPTION
#   This function returns a tuple of RGB values depending on the integer
#   input value. Information about the total count of input values
#   IS needed.
# INPUTS
#   * integer value i (as index)
#   * floating point value s (as saturation) defaults to 1.0
# OUTPUT
#   RGB tuple (r, g, b)
# SOURCE


def palette_rgb_g2r(i, max):
    h = 1.0*i/max
#    if h >= 1.:
#        print "TEMANEJO: WARNING in palette_rgb_g2r"
#        h = .99
    h = .4*(1.-h)
    v = 1.
    s = 1.
    rgb = colorsys.hsv_to_rgb(h, s, v)
    return tuple(rgb)

#******

def paletteFractions_to_rgb(fs):
    # fs is a 3-tupe of floats -1.0<=f<=1.0
    # return 3-tupe of values 0<=x<=255
    return [int(255*abs(f)) for f in fs]
